@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21956 $
+ * $Revision: 26128 $
  * $Author: rjongbloed $
- * $Date: 2009-01-28 03:09:29 +0000 (Wed, 28 Jan 2009) $
+ * $Date: 2011-07-07 21:36:57 -0500 (Thu, 07 Jul 2011) $
  */
 
 #ifndef OPAL_H323_H323CAPS_H
@@ -121,6 +121,8 @@ class H323Capability : public PObject
       e_Data,
       /// User Input capability
       e_UserInput,
+      /// Generic Control
+      e_GenericControl,
       /// Count of main types
       e_NumMainTypes
     };
@@ -252,11 +254,11 @@ class H323Capability : public PObject
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored.
        
          The default behaviour sets the capabilityDirection member variable
-         from the PDU and then returns PTrue. Note that this means it is very
+         from the PDU and then returns true. Note that this means it is very
          important to call the ancestor function when overriding.
      */
     virtual PBoolean OnReceivedPDU(
@@ -286,7 +288,7 @@ class H323Capability : public PObject
        application to determine if this capability should be used in TCS or
        OLC pdus.
 
-       The default behaviour returns PTrue.
+       The default behaviour returns true.
       */
     virtual PBoolean IsUsable(
       const H323Connection & connection
@@ -326,15 +328,15 @@ class H323Capability : public PObject
 
     /** Set media format option for the media data this class represents.
       */
-    bool SetMediaFormatOptions(
+    bool UpdateMediaFormat(
       const OpalMediaFormat & format
     );
 
     /// Get the payload type for the capaibility
-    RTP_DataFrame::PayloadTypes GetPayloadType() const { return rtpPayloadType; }
+    RTP_DataFrame::PayloadTypes GetPayloadType() const { return GetMediaFormat().GetPayloadType(); }
 
     /// Set the payload type for the capaibility
-    void SetPayloadType(RTP_DataFrame::PayloadTypes pt) { rtpPayloadType = pt; }
+    void SetPayloadType(RTP_DataFrame::PayloadTypes pt) { GetWritableMediaFormat().SetPayloadType(pt); }
 
     /// Attach a QoS specification to this channel
     virtual void AttachQoS(RTP_QOS *) { }
@@ -346,14 +348,13 @@ class H323Capability : public PObject
 #endif
 
   protected:
-    OpalMediaFormat & GetWritableMediaFormat();
+    OpalMediaFormat & GetWritableMediaFormat() const;
 
     unsigned            assignedCapabilityNumber;  /// Unique ID assigned to capability
     CapabilityDirection capabilityDirection;
-    RTP_DataFrame::PayloadTypes rtpPayloadType;
 
   private:
-    mutable OpalMediaFormat mediaFormat;
+    mutable OpalMediaFormat m_mediaFormat;
 
   friend class H323Capabilities;
 };
@@ -426,7 +427,7 @@ class H323NonStandardCapabilityInfo
 
     /**This function validates and uses the non-standard data field.
 
-       The default behaviour returns PTrue if data is equal to fixedData.
+       The default behaviour returns true if data is equal to fixedData.
       */
     virtual PBoolean OnReceivedPDU(
       const PBYTEArray & data  ///<  Data field in PDU received
@@ -479,10 +480,10 @@ class H323GenericCapabilityInfo
   public:
     H323GenericCapabilityInfo(
       const PString & id,     ///< generic codec identifier
-      PINDEX maxBitRate = 0   ///< maxBitRate parameter for the GenericCapability
+      unsigned maxBitRate = 0   ///< maxBitRate parameter for the GenericCapability
     );
-    H323GenericCapabilityInfo(const H323GenericCapabilityInfo & obj);
-    virtual ~H323GenericCapabilityInfo();
+
+    virtual ~H323GenericCapabilityInfo() { }
 
   protected:
     virtual PBoolean OnSendingGenericPDU(
@@ -504,8 +505,8 @@ class H323GenericCapabilityInfo
     ) const;
 
 
-    H245_CapabilityIdentifier * identifier;
-    unsigned                    maxBitRate;
+    PString  m_identifier;
+    unsigned maxBitRate;
 };
 
 /**This class describes the interface to a codec that has channels based on
@@ -680,7 +681,7 @@ class H323AudioCapability : public H323RealTimeCapability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored.
        
        The default behaviour calls the OnReceivedPDU() that takes a
@@ -862,7 +863,7 @@ class H323GenericAudioCapability : public H323AudioCapability,
       */
     H323GenericAudioCapability(
       const PString & capabilityId,    ///< generic codec identifier
-      PINDEX maxBitRate = 0	           ///< maxBitRate parameter for the GenericCapability
+      PINDEX maxBitRate = 0               ///< maxBitRate parameter for the GenericCapability
     );
   //@}
 
@@ -1039,9 +1040,9 @@ class H323VideoCapability : public H323RealTimeCapability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1213,7 +1214,7 @@ class H323GenericVideoCapability : public H323VideoCapability,
       */
     H323GenericVideoCapability(
       const PString & capabilityId,    ///< generic codec identifier (OID)
-      PINDEX maxBitRate = 0	       ///< maxBitRate parameter for the GenericCapability
+      PINDEX maxBitRate = 0           ///< maxBitRate parameter for the GenericCapability
     );
   //@}
 
@@ -1274,17 +1275,298 @@ class H323GenericVideoCapability : public H323VideoCapability,
       const H245_VideoCapability & pdu,  ///< PDU to get information from
       CommandType type                   ///<  Type of PDU to send in
     );
-  //@}
 
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
       const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
     ) const;
+  //@}
 };
 
+
+#if OPAL_H239
+
+/**This class descibes an extended ideo capability, as used in H.239.
+  */
+class H323ExtendedVideoCapability : public H323GenericVideoCapability
+{
+  PCLASSINFO(H323ExtendedVideoCapability, H323GenericVideoCapability);
+
+  public:
+  /**@name Construction */
+  //@{
+    /**Create a new Extended Video capability.
+      */
+    H323ExtendedVideoCapability(
+      const PString & identifier   ///< generic codec identifier (OID)
+    );
+  //@}
+
+  /**@name Identification functions */
+  //@{
+    /**Get the sub-type of the capability. This is a code dependent on the
+       main type of the capability.
+
+       This returns H245_VideoCapability::e_extendedVideoCapability.
+     */
+    virtual unsigned GetSubType() const;
+  //@}
+    
+  /**@name Protocol manipulation */
+  //@{
+    /**This function is called whenever and outgoing TerminalCapabilitySet
+       or OpenLogicalChannel PDU is being constructed for the control channel.
+       It allows the capability to set the PDU fields from information in
+       members specific to the class.
+
+       The default behaviour calls H323GenericCapabilityinfo::OnSendingPDU()
+       to handle the PDU.
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_VideoCapability & pdu,  ///<  PDU to set information on
+      CommandType type             ///<  Type of PDU to send in
+    ) const;
+
+    /**This function is called whenever and outgoing RequestMode
+       PDU is being constructed for the control channel. It allows the
+       capability to set the PDU fields from information in members specific
+       to the class.
+
+       The default behaviour sets the PDUs tag according to the GetSubType()
+       function (translated to different enum).
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_VideoMode & pdu  ///<  PDU to set information on
+    ) const;
+
+    /**This function is called whenever and incoming TerminalCapabilitySet
+       or OpenLogicalChannel PDU has been used to construct the control
+       channel. It allows the capability to set from the PDU fields,
+       information in members specific to the class.
+
+       The default behaviour calls H323GenericCapabilityinfo::OnReceivedPDU()
+       to handle the provided PDU.
+     */
+    virtual PBoolean OnReceivedPDU(
+      const H245_VideoCapability & pdu,  ///< PDU to get information from
+      CommandType type                   ///<  Type of PDU to send in
+    );
+
+    /**Compare the generic part of the capability, if applicable.
+     */
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+    ) const;
+  //@}
+
+  protected:
+    OpalMediaFormatList m_videoFormats;
+};
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+/**This class describes a generic control capability, as used in H.239
+ */
+class H323GenericControlCapability : public H323Capability,
+                                     public H323GenericCapabilityInfo
+{
+  PCLASSINFO(H323GenericControlCapability, H323Capability);
+
+  public:
+  /**@name Construction */
+  //@{
+    /**Create a new data capability.
+      */
+    H323GenericControlCapability(
+      const PString & identifier  ///< Indentifer (OID) for generic control
+    );
+  //@}
+
+  /**@name Identification functions */
+  //@{
+    /**Get the main type of the capability.
+       Always returns e_Data.
+     */
+    virtual MainTypes GetMainType() const;
+
+    /**Get the sub-type of the capability. This is a code dependent on the
+       main type of the capability.
+
+       This returns H245_VideoCapability::e_extendedVideoCapability.
+     */
+    virtual unsigned GetSubType() const;
+  //@}
+
+  /**@name Protocol manipulation */
+  //@{
+    /**This function is called whenever and outgoing TerminalCapabilitySet
+       PDU is being constructed for the control channel. It allows the
+       capability to set the PDU fields from information in members specific
+       to the class.
+
+       The default behaviour calls the OnSendingPDU() function with a more
+       specific PDU type.
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_Capability & pdu  ///<  PDU to set information on
+    ) const;
+
+    /**This function is called whenever and outgoing RequestMode
+       PDU is being constructed for the control channel. It allows the
+       capability to set the PDU fields from information in members specific
+       to the class.
+
+       The default behaviour is pure.
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_ModeElement & pdu  ///<  PDU to set information on
+    ) const;
+
+    /**This function is called whenever and incoming TerminalCapabilitySet
+       PDU is received on the control channel, and a new H323Capability
+       descendent was created. This completes reading fields from the PDU
+       into the classes members.
+
+       If the function returns false then the received PDU codec description
+       is not supported, so will be ignored. The default behaviour simply
+       returns true.
+     */
+    virtual PBoolean OnReceivedPDU(
+      const H245_Capability & pdu  ///<  PDU to get information from
+    );
+
+    /**Compare the generic part of the capability, if applicable.
+     */
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+    ) const;
+  //@}
+
+  /**@name Operations */
+  //@{
+    /**Create the channel instance, allocating resources as required.
+       This creates a logical channel object appropriate for the parameters
+       provided. Not if param is NULL, sessionID must be provided, otherwise
+       this is taken from the fields in param.
+     */
+    virtual H323Channel * CreateChannel(
+      H323Connection & connection,    ///<  Owner connection for channel
+      H323Channel::Directions dir,    ///<  Direction of channel
+      unsigned sessionID,             ///<  Session ID for RTP channel
+      const H245_H2250LogicalChannelParameters * param
+                                      ///<  Parameters for channel
+    ) const;
+  //@}
+};
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+class H323H239VideoCapability : public H323ExtendedVideoCapability
+{
+  PCLASSINFO(H323H239VideoCapability, H323ExtendedVideoCapability);
+
+  public:
+  /**@name Construction */
+  //@{
+    /**Create a new Extended Video capability.
+      */
+    H323H239VideoCapability(
+      const OpalMediaFormat & videoFormat
+    );
+  //@}
+
+  /**@name Overrides from class PObject */
+  //@{
+    /**Compare two capability instances. This compares the main and sub-types
+       of the capability.
+     */
+    Comparison Compare(const PObject & obj) const;
+
+    /**Create a copy of the object.
+      */
+    virtual PObject * Clone() const;
+
+    /**Print name of capability.
+      */
+    void PrintOn(ostream & strm) const;
+  //@}
+
+  /**@name Identification functions */
+  //@{
+    /**Get the name of the media data format this class represents.
+     */
+    virtual PString GetFormatName() const;
+  //@}
+
+  /**@name Identification functions */
+  //@{
+    /**This function is called whenever and outgoing TerminalCapabilitySet
+       or OpenLogicalChannel PDU is being constructed for the control channel.
+       It allows the capability to set the PDU fields from information in
+       members specific to the class.
+
+       The default behaviour calls H323GenericCapabilityinfo::OnSendingPDU()
+       to handle the PDU.
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_VideoCapability & pdu,  ///<  PDU to set information on
+      CommandType type             ///<  Type of PDU to send in
+    ) const;
+
+    /**This function is called whenever and incoming TerminalCapabilitySet
+       or OpenLogicalChannel PDU has been used to construct the control
+       channel. It allows the capability to set from the PDU fields,
+       information in members specific to the class.
+
+       The default behaviour calls H323GenericCapabilityinfo::OnReceivedPDU()
+       to handle the provided PDU.
+     */
+    virtual PBoolean OnReceivedPDU(
+      const H245_VideoCapability & pdu,  ///< PDU to get information from
+      CommandType type                   ///<  Type of PDU to send in
+    );
+  //@}
+};
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+class H323H239ControlCapability : public H323GenericControlCapability
+{
+  PCLASSINFO(H323H239ControlCapability, H323GenericControlCapability);
+
+  public:
+  /**@name Construction */
+  //@{
+    /**Create a new Extended Video capability.
+      */
+    H323H239ControlCapability();
+  //@}
+
+  /**@name Overrides from class PObject */
+  //@{
+    /**Create a copy of the object.
+      */
+    virtual PObject * Clone() const;
+  //@}
+
+  /**@name Identification functions */
+  //@{
+    /**Get the name of the media data format this class represents.
+     */
+    virtual PString GetFormatName() const;
+  //@}
+};
+
+
+#endif  // OPAL_H239
+
 #endif  // OPAL_VIDEO
 
+
 /**This class describes the interface to a data channel used to transfer data
    via the logical channels opened and managed by the H323 control channel.
 
@@ -1395,9 +1677,9 @@ class H323DataCapability : public H323Capability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1729,9 +2011,9 @@ class H323_UserInputCapability : public H323Capability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1754,7 +2036,7 @@ class H323_UserInputCapability : public H323Capability
        application to determine if this capability should be used in TCS or
        OLC pdus.
 
-       The default behaviour will check for early versions and return PFalse
+       The default behaviour will check for early versions and return false
        for RFC2833 mode.
       */
     virtual PBoolean IsUsable(
@@ -1763,9 +2045,10 @@ class H323_UserInputCapability : public H323Capability
   //@}
 
     static void AddAllCapabilities(
-      H323Capabilities & capabilities,        ///<  Table to add capabilities to
-      PINDEX descriptorNum,   ///<  The member of the capabilityDescriptor to add
-      PINDEX simultaneous     ///<  The member of the SimultaneousCapabilitySet to add
+      H323Capabilities & capabilities,  ///<  Table to add capabilities to
+      PINDEX descriptorNum,             ///<  The member of the capabilityDescriptor to add
+      PINDEX simultaneous,              ///<  The member of the SimultaneousCapabilitySet to add
+      bool includeRFC2833 = true
     );
 
   protected:
@@ -1897,7 +2180,7 @@ class H323Capabilities : public PObject
       PINDEX descriptorNum,    ///<  The member of the capabilityDescriptor to add
       PINDEX simultaneous,     ///<  The member of the SimultaneousCapabilitySet to add
       const PString & name,    ///<  New capabilities name, if using "known" one.
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     );
 
     // this function is retained for backwards compatibility
@@ -1906,7 +2189,7 @@ class H323Capabilities : public PObject
       PINDEX descriptorNum,    ///<  The member of the capabilityDescriptor to add
       PINDEX simultaneous,     ///<  The member of the SimultaneousCapabilitySet to add
       const PString & name,    ///<  New capabilities name, if using "known" one.
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     )
     { return AddAllCapabilities(descriptorNum, simultaneous, name, exact); }
 
@@ -1989,7 +2272,7 @@ class H323Capabilities : public PObject
       const PString & formatName, ///<  Wildcard format name to search for
       H323Capability::CapabilityDirection direction = H323Capability::e_Unknown,
             ///<  Optional direction to include into search criteria
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     ) const;
 
     /**Find the first capability in the table of the specified direction.
