@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23128 $
+ * $Revision: 25678 $
  * $Author: rjongbloed $
- * $Date: 2009-07-22 05:17:45 +0000 (Wed, 22 Jul 2009) $
+ * $Date: 2011-05-05 17:46:37 -0500 (Thu, 05 May 2011) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -49,6 +49,7 @@
 #include <h323/h323con.h>
 #include <h323/h323caps.h>
 #include <h323/h235auth.h>
+#include <asn/h225.h>
 
 #if OPAL_H460
 #include <h460/h4601.h>
@@ -127,12 +128,12 @@ class H323EndPoint : public OpalRTPEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -140,7 +141,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,                  ///<  Owner of connection
       const PString & party,            ///<  Remote party to call
       void * userData  = NULL,          ///<  Arbitrary data to pass to connection
@@ -382,7 +383,7 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean IsRegisteredWithGatekeeper() const;
 
     /**Unregister and delete the gatekeeper we are registered with.
-       The return value indicates PFalse if there was an error during the
+       The return value indicates false if there was an error during the
        unregistration. However the gatekeeper is still removed and its
        instance deleted regardless of this error.
      */
@@ -519,7 +520,8 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean ParsePartyName(
       const PString & party,          ///<  Party name string.
       PString & alias,                ///<  Parsed alias name
-      H323TransportAddress & address  ///<  Parsed transport address
+      H323TransportAddress & address, ///<  Parsed transport address
+      OpalConnection::StringOptions * stringOptions = NULL ///< String options parsed from party name
     );
 
     /**Find a connection that uses the specified token.
@@ -548,34 +550,36 @@ class H323EndPoint : public OpalRTPEndPoint
                                    H323SignalPDU & setupPDU);
 
     /**Adjust call proceeding PDU being sent. This function is called from
-    the OnReceivedSignalSetup() function before it sends the Call
-    Proceeding PDU. It gives an opportunity for an application to alter
-    the request before transmission to the other endpoint. If this function
-    returns PFalse then the Call Proceeding PDU is not sent at all.
+       the OnReceivedSignalSetup() function before it sends the Call
+       Proceeding PDU. It gives an opportunity for an application to alter
+       the request before transmission to the other endpoint. If this function
+       returns false then the Call Proceeding PDU is not sent at all.
 
-    The default behaviour simply returns PTrue.
-    @param connection the connection associated to the call proceeding
-    @param callProceedingPDU the call processding to modify
-    @return if false, do no send the connect pdu  
+       The default behaviour simply returns true.
+       @param connection the connection associated to the call proceeding
+       @param callProceedingPDU the call processding to modify
+       @return if false, do no send the connect pdu  
      */
-    virtual PBoolean OnSendCallProceeding(H323Connection & connection,
-                                      H323SignalPDU & callProceedingPDU
-                                     );
-    
+    virtual PBoolean OnSendCallProceeding(
+      H323Connection & connection,
+      H323SignalPDU & callProceedingPDU
+    );
+
     /**Adjust call connect PDU being sent. This function is called from
-    the H323Connection::SetConnected function before it sends the connect  PDU. 
-    It gives an opportunity for an application to alter
-    the request before transmission to the other endpoint. If this function
-    returns PFalse then the connect PDU is not sent at all.
+       the H323Connection::SetConnected function before it sends the connect  PDU. 
+       It gives an opportunity for an application to alter
+       the request before transmission to the other endpoint. If this function
+       returns false then the connect PDU is not sent at all.
 
-    The default behaviour simply returns PTrue.
-    @param connection the connection associated to the connect
-    @param connectPDU the connect to modify
-    @return if false, do no send the connect pdu  
+       The default behaviour simply returns true.
+       @param connection the connection associated to the connect
+       @param connectPDU the connect to modify
+       @return if false, do no send the connect pdu  
      */
-    virtual PBoolean OnSendConnect(H323Connection & connection,
-                               H323SignalPDU & connectPDU
-                              );
+    virtual PBoolean OnSendConnect(
+      H323Connection & connection,
+      H323SignalPDU & connectPDU
+    );
     
     /**Call back for incoming call.
        This function is called from the OnReceivedSignalSetup() function
@@ -583,10 +587,10 @@ class H323EndPoint : public OpalRTPEndPoint
        application to alter the reply before transmission to the other
        endpoint.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnIncomingCall(
       H323Connection & connection,    ///<  Connection that was established
@@ -595,19 +599,19 @@ class H323EndPoint : public OpalRTPEndPoint
     );
 
     /**Called when an outgoing call connects
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
       */
     virtual PBoolean OnOutgoingCall(
-        H323Connection & conn, 
-        const H323SignalPDU & connectPDU
+      H323Connection & conn, 
+      const H323SignalPDU & connectPDU
     );
 
     /**Handle a connection transfer.
        This gives the application an opportunity to abort the transfer.
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean OnCallTransferInitiate(
       H323Connection & connection,    ///<  Connection to transfer
@@ -616,7 +620,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /**Handle a transfer via consultation.
        This gives the transferred-to user an opportunity to abort the transfer.
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean OnCallTransferIdentify(
       H323Connection & connection    ///<  Connection to transfer
@@ -654,10 +658,10 @@ class H323EndPoint : public OpalRTPEndPoint
        receives the optional Alerting PDU from the remote endpoint. That is
        when the remote "phone" is "ringing".
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnAlerting(
       H323Connection & connection,    ///<  Connection that was established
@@ -667,27 +671,28 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /** A call back function when the alerting is about to be sent, 
         can be used by the application to alter the alerting Pdu
-        @return if PFalse, then the alerting is not sent
+        @return if false, then the alerting is not sent
      */
-      
-    virtual PBoolean OnSendAlerting(H323Connection & connection,  ///< onnection that was established
-                                H323SignalPDU & alerting,     ///< Alerting PDU to modify
-                                const PString & calleeName,   ///< Name of endpoint being alerted.
-                                PBoolean withMedia                ///< Open media with alerting
-                               );
-        
+    virtual PBoolean OnSendAlerting(
+      H323Connection & connection,  ///< onnection that was established
+      H323SignalPDU & alerting,     ///< Alerting PDU to modify
+      const PString & calleeName,   ///< Name of endpoint being alerted.
+      PBoolean withMedia                ///< Open media with alerting
+    );
+
     /** A call back function when the alerting has been sent, can be used by the application 
         to send the connect as soon as the alerting has been sent.
      */
-      
-    virtual PBoolean OnSentAlerting(H323Connection & connection);
-    
+    virtual PBoolean OnSentAlerting(
+      H323Connection & connection
+    );
+
     /**A call back function when a connection indicates it is to be forwarded.
        An H323 application may handle this call back so it can make
        complicated decisions on if the call forward ius to take place. If it
-       decides to do so it must call MakeCall() and return PTrue.
+       decides to do so it must call MakeCall() and return true.
 
-       The default behaviour simply returns PFalse and that the automatic
+       The default behaviour simply returns false and that the automatic
        call forwarding should take place. See ForwardConnection()
       */
     virtual PBoolean OnConnectionForwarded(
@@ -697,7 +702,7 @@ class H323EndPoint : public OpalRTPEndPoint
     );
 
     /**Forward the call using the same token as the specified connection.
-       Return PTrue if the call is being redirected.
+       Return true if the call is being redirected.
 
        The default behaviour will replace the current call in the endpoints
        call list using the same token as the call being redirected. Not that
@@ -726,17 +731,6 @@ class H323EndPoint : public OpalRTPEndPoint
     virtual PBoolean IsConnectionEstablished(
       const PString & token   ///<  Token for identifying connection
     );
-
-    /**A call back function whenever a connection is broken.
-       This indicates that a connection to a remote endpoint is no longer
-       available.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnConnectionCleared(
-      H323Connection & connection,    ///<  Connection that was established
-      const PString & token           ///<  Token for identifying connection
-    );
   //@}
 
 
@@ -744,7 +738,7 @@ class H323EndPoint : public OpalRTPEndPoint
   //@{
     /**Call back for opening a logical channel.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
       */
     virtual PBoolean OnStartLogicalChannel(
       H323Connection & connection,    ///<  Connection for the channel
@@ -803,7 +797,7 @@ class H323EndPoint : public OpalRTPEndPoint
        An application may override this to display call credit information
        on registration, or when a call is started.
 
-       The canDisplayAmountString member variable must also be set to PTrue
+       The canDisplayAmountString member variable must also be set to true
        for this to operate.
 
        The default behaviour does nothing.
@@ -835,7 +829,7 @@ class H323EndPoint : public OpalRTPEndPoint
     /** Called when an endpoint receives a SETUP PDU with a
         conference goal of "invite"
       
-        The default behaviour is to return PFalse, which will close the connection
+        The default behaviour is to return false, which will close the connection
      */
     virtual PBoolean OnConferenceInvite(
       const H323SignalPDU & setupPDU
@@ -844,7 +838,7 @@ class H323EndPoint : public OpalRTPEndPoint
     /** Called when an endpoint receives a SETUP PDU with a
         conference goal of "callIndependentSupplementaryService"
       
-        The default behaviour is to return PFalse, which will close the connection
+        The default behaviour is to return false, which will close the connection
      */
     virtual PBoolean OnCallIndependentSupplementaryService(
       const H323SignalPDU & setupPDU
@@ -853,7 +847,7 @@ class H323EndPoint : public OpalRTPEndPoint
     /** Called when an endpoint receives a SETUP PDU with a
         conference goal of "capability_negotiation"
       
-        The default behaviour is to return PFalse, which will close the connection
+        The default behaviour is to return false, which will close the connection
      */
     virtual PBoolean OnNegotiateConferenceCapabilities(
       const H323SignalPDU & setupPDU
@@ -964,13 +958,13 @@ class H323EndPoint : public OpalRTPEndPoint
     ) { disableH245inSetup = mode; } 
 
     /** find out if h245 is disabled or enabled 
-      * @return PTrue if h245 is disabled 
+      * @return true if h245 is disabled 
       */
     PBoolean IsH245Disabled() const
     { return m_bH245Disabled; }
 
     /**Disable/Enable H.245, used at least for h450.7 calls
-     * @param  bH245Disabled PTrue if h245 has to be disabled 
+     * @param  bH245Disabled true if h245 has to be disabled 
      */
     void DisableH245(PBoolean bH245Disabled) { m_bH245Disabled = bH245Disabled; } 
     
@@ -1040,6 +1034,10 @@ class H323EndPoint : public OpalRTPEndPoint
       e_MCUWithAVMP = 190
     };
 
+    /**Get the endpoint terminal type.
+     */
+    void SetTerminalType(TerminalTypes type) { terminalType = type; }
+
     /**Get the endpoint terminal type.
      */
     TerminalTypes GetTerminalType() const { return terminalType; }
@@ -1085,6 +1083,18 @@ class H323EndPoint : public OpalRTPEndPoint
      */
     void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
 
+#if OPAL_H239
+    /**Get the default H.239 control capability.
+     */
+    bool GetDefaultH239Control() const { return m_defaultH239Control; }
+
+    /**Set the default H.239 control capability.
+     */
+    void SetDefaultH239Control(
+      bool on   ///< H.239 control capability is to be sent to remote
+    ) { m_defaultH239Control = on; }
+#endif
+
     /**Called when an outgoing PDU requires a feature set
      */
     virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &);
@@ -1185,11 +1195,11 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /**Get the IP Type Of Service byte for RTP channels.
      */
-    BYTE GetRtpIpTypeofService() const { return manager.GetRtpIpTypeofService(); }
+    BYTE P_DEPRECATED GetRtpIpTypeofService() const { return manager.GetMediaTypeOfService(); }
 
     /**Set the IP Type Of Service byte for RTP channels.
      */
-    void SetRtpIpTypeofService(unsigned tos) { manager.SetRtpIpTypeofService(tos); }
+    void P_DEPRECATED SetRtpIpTypeofService(unsigned tos) { manager.SetMediaTypeOfService(tos); }
 
     /**Get the default timeout for calling another endpoint.
      */
@@ -1328,7 +1338,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
   protected:
     bool InternalCreateGatekeeper(H323Transport * transport);
-    PBoolean InternalMakeCall(
+    H323Connection * InternalMakeCall(
       OpalCall & call,
       const PString & existingToken,           ///<  Existing connection to be transferred
       const PString & callIdentity,            ///<  Call identity of the secondary call (if it exists)
@@ -1350,12 +1360,16 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean        canDisplayAmountString;
     PBoolean        canEnforceDurationLimit;
 #if OPAL_H450
-    unsigned    callIntrusionProtectionLevel;
+    unsigned        callIntrusionProtectionLevel;
 #endif
 
-    TerminalTypes terminalType;
+    TerminalTypes   terminalType;
 
-    PBoolean          clearCallOnRoundTripFail;
+#if OPAL_H239
+    bool            m_defaultH239Control;
+#endif
+
+    PBoolean        clearCallOnRoundTripFail;
 
     // Some more configuration variables, rarely changed.
     PTimeInterval signallingChannelCallTimeout;
@@ -1415,6 +1429,8 @@ class H323EndPoint : public OpalRTPEndPoint
     H460_FeatureSet features;
 #endif
 
+  private:
+    P_REMOVE_VIRTUAL_VOID(OnConnectionCleared(H323Connection &, const PString &));
 };
 
 #endif // OPAL_H323
