@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23275 $
+ * $Revision: 24766 $
  * $Author: rjongbloed $
- * $Date: 2009-08-27 02:17:54 -0500 (Thu, 27 Aug 2009) $
+ * $Date: 2010-09-30 02:18:52 -0500 (Thu, 30 Sep 2010) $
  */
 
 #ifndef OPAL_LIDS_LIDEP_H
@@ -87,12 +87,12 @@ class OpalLineEndPoint : public OpalEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -105,7 +105,7 @@ class OpalLineEndPoint : public OpalEndPoint
       const PString & party,    ///< Remote party to call
       void * userData = NULL,   ///< Arbitrary data to pass to connection
       unsigned int options = 0,  ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions  = NULL
+      OpalConnection::StringOptions * stringOptions  = NULL ///< Options to pass to connection
     );
 
     /**Get the data formats this endpoint is capable of operating.
@@ -138,7 +138,7 @@ class OpalLineEndPoint : public OpalEndPoint
       */
     PSafePtr<OpalLineConnection> GetLIDConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite   ///< Lock mode
     ) { return PSafePtrCast<OpalConnection, OpalLineConnection>(GetConnectionWithLock(token, mode)); }
 
     /**Add a line to the endpoint.
@@ -146,7 +146,7 @@ class OpalLineEndPoint : public OpalEndPoint
        should not be deleted directly. Use the RemoveLine() function to
        delete the line.
 
-       Returns PTrue if the lines device was open and the line was added.
+       Returns true if the lines device was open and the line was added.
       */
     PBoolean AddLine(
       OpalLine * line
@@ -186,7 +186,7 @@ class OpalLineEndPoint : public OpalEndPoint
 
        Note the device should already be open or no lines are added.
 
-       Returns PTrue if at least one line was added.
+       Returns true if at least one line was added.
       */
     virtual PBoolean AddLinesFromDevice(
       OpalLineInterfaceDevice & device  ///<  Device to add lines
@@ -202,7 +202,7 @@ class OpalLineEndPoint : public OpalEndPoint
     /**Add a line interface devices to the endpoint by name.
        This calls AddDeviceName() for each entry in the array.
 
-       Returns PTrue if at least one line from one device was added.
+       Returns true if at least one line from one device was added.
       */
     PBoolean AddDeviceNames(
       const PStringArray & descriptors  ///<  Device descritptions to add
@@ -214,7 +214,7 @@ class OpalLineEndPoint : public OpalEndPoint
        as would be returned from the OpalLineInterfaceDevice::GetAllDevices()
        function.
 
-       Returns PTrue if at least one line was added or the descriptor was
+       Returns true if at least one line was added or the descriptor was
        already present.
       */
     PBoolean AddDeviceName(
@@ -236,7 +236,7 @@ class OpalLineEndPoint : public OpalEndPoint
 
        Note the device should already be open or no lines are added.
 
-       Returns PTrue if at least one line was added.
+       Returns true if at least one line was added.
       */
     virtual PBoolean AddDevice(
       OpalLineInterfaceDevice * device    ///<  Device to add
@@ -256,7 +256,7 @@ class OpalLineEndPoint : public OpalEndPoint
     /**Get the line by name.
        The lineName parameter may be "*" to matche the first line.
 
-       If the enableAudio flag is PTrue then the EnableAudio() function is
+       If the enableAudio flag is true then the EnableAudio() function is
        called on the line and it is returns only if successful. This
        effectively locks the line for exclusive use of the caller.
       */
@@ -272,6 +272,21 @@ class OpalLineEndPoint : public OpalEndPoint
     void SetDefaultLine(
       const PString & lineName  ///<  Name of line to set to default.
     );
+
+    /**Set the country code set for the device.
+       This may change the line analogue coefficients, ring detect, call
+       disconnect detect and call progress tones to fit the countries
+       telephone network.
+      */
+    bool SetCountryCode(
+      OpalLineInterfaceDevice::T35CountryCodes country   ///<  Country code for device
+    );
+
+    /**Set the country code set for the device.
+      */
+    bool SetCountryCodeName(
+      const PString & countryName   ///<  Country code for device
+    );
   //@}
 
 
@@ -424,6 +439,17 @@ class OpalLineConnection : public OpalConnection
       OpalMediaStream & stream    ///<  New media stream being opened
     );
 
+    /**Call back for closed a media stream.
+
+       Note that this is called while the OpalMediaStream is locked. Care needs to be
+       taken to avoid deadlocks.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
+      */
+    virtual void OnClosedMediaStream(
+      const OpalMediaStream & stream     ///<  Media stream being closed
+    );
+
     /**Set  the volume (gain) for the audio media channel to the specified percentage.
       */
     virtual PBoolean SetAudioVolume(
@@ -498,12 +524,13 @@ class OpalLineConnection : public OpalConnection
       */
     void SetPromptTone(OpalLineInterfaceDevice::CallProgressTones tone) { m_promptTone = tone; }
 
-    /** delay in msec to wait between the dial tone detection and dialing the dtmf 
-      * @param uiDial the dial delay to set
+    /** Delay in msec to wait between the dial tone detection and dialing the dtmf 
      */
-    void setDialDelay(unsigned int uiDialDelay) { m_dialParams.m_dialStartDelay = uiDialDelay;}
-    
-    /** delay in msec to wait between the dial tone detection and dialing the dtmf 
+    void setDialDelay(
+      unsigned int uiDialDelay  ///< Dialing delay
+    ) { m_dialParams.m_dialStartDelay = uiDialDelay;}
+
+    /** Delay in msec to wait between the dial tone detection and dialing the dtmf 
      * @return uiDialDelay the dial delay to get
      */
     unsigned int getDialDelay() const { return m_dialParams.m_dialStartDelay; }
@@ -534,7 +561,7 @@ class OpalLineMediaStream : public OpalMediaStream
     /**Construct a new media stream for Line Interface Devices.
       */
     OpalLineMediaStream(
-      OpalLineConnection & conn,
+      OpalLineConnection & conn,            ///< Owner connection
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       PBoolean isSource,                       ///<  Is a source stream
@@ -607,7 +634,7 @@ class OpalLineMediaStream : public OpalMediaStream
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns PTrue for LID streams.
+       Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
 
