@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 26069 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Date: 2011-06-23 19:56:50 -0500 (Thu, 23 Jun 2011) $
  */
 
 #ifndef OPAL_OPAL_LOCALEP_H
@@ -67,6 +67,18 @@ class OpalLocalEndPoint : public OpalEndPoint
 
   /**@name Overrides from OpalEndPoint */
   //@{
+    /**Get the data formats this endpoint is capable of operating.
+       This provides a list of media data format names that may be used by an
+       OpalMediaStream may be created by a connection from this endpoint.
+
+       Note that a specific connection may not actually support all of the
+       media formats returned here, but should return no more.
+
+       The default behaviour returns the most basic media formats, PCM audio
+       and YUV420P video.
+      */
+    virtual OpalMediaFormatList GetMediaFormats() const;
+
     /**Set up a connection to a remote party.
        This is called from the OpalManager::MakeConnection() function once
        it has determined that this is the endpoint for the protocol.
@@ -96,12 +108,12 @@ class OpalLocalEndPoint : public OpalEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,           ///<  Owner of connection
       const PString & party,     ///<  Remote party to call
       void * userData = NULL,    ///<  Arbitrary data to pass to connection
       unsigned int options = 0,  ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions  = NULL
+      OpalConnection::StringOptions * stringOptions  = NULL ///< Options to pass to connection
     );
   //@}
 
@@ -115,7 +127,7 @@ class OpalLocalEndPoint : public OpalEndPoint
       */
     PSafePtr<OpalLocalConnection> GetLocalConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite   ///< Lock mode
     ) { return GetConnectionWithLockAs<OpalLocalConnection>(token, mode); }
 
     /**Create a connection for the PCSS endpoint.
@@ -123,7 +135,9 @@ class OpalLocalEndPoint : public OpalEndPoint
       */
     virtual OpalLocalConnection * CreateConnection(
       OpalCall & call,    ///<  Owner of connection
-      void * userData     ///<  Arbitrary data to pass to connection
+      void * userData,    ///<  Arbitrary data to pass to connection
+      unsigned options,   ///< Option bit mask to pass to connection
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
     /**Call back to indicate that remote is ringing.
@@ -135,15 +149,26 @@ class OpalLocalEndPoint : public OpalEndPoint
       const OpalLocalConnection & connection ///<  Connection having event
     );
 
-    /**Call back to indicate that remote is ringing.
-       If false is returned the call is aborted.
+    /**Call back to indicate that there is an incoming call.
+       Note this function should not block or it will impede the operation of
+       the stack.
 
-       The default implementation answers the call immediately.
+       The default implementation returns true;
+
+       @return false is returned the call is aborted with status of EndedByLocalBusy.
       */
     virtual bool OnIncomingCall(
       OpalLocalConnection & connection ///<  Connection having event
     );
 
+    /**Indicate alerting for the incoming connection.
+       Returns false if the connection token does not correspond to a valid
+       connection.
+      */
+    virtual bool AlertingIncomingCall(
+      const PString & token ///<  Token of connection to accept call
+    );
+
     /**Accept the incoming connection.
        Returns false if the connection token does not correspond to a valid
        connection.
@@ -157,7 +182,8 @@ class OpalLocalEndPoint : public OpalEndPoint
        connection.
       */
     virtual bool RejectIncomingCall(
-      const PString & token ///<  Token of connection to accept call
+      const PString & token,                 ///<  Token of connection to accept call
+      const OpalConnection::CallEndReason & reason = OpalConnection::EndedByAnswerDenied ///<  Reason for rejecting the call
     );
 
     /**Call back to indicate that the remote user has indicated something.
@@ -167,7 +193,7 @@ class OpalLocalEndPoint : public OpalEndPoint
       */
     virtual bool OnUserInput(
       const OpalLocalConnection & connection, ///<  Connection having event
-      const PString & indication
+      const PString & indication    ///< User input received
     );
 
     /**Call back to get media data for transmission.
@@ -232,11 +258,36 @@ class OpalLocalEndPoint : public OpalEndPoint
        Default returns true.
       */
     virtual bool IsSynchronous() const;
+
+    /**Indicate OnAlerting() is be deferred or immediate.
+      */
+    bool IsDeferredAlerting() const { return m_deferredAlerting; }
+
+    /**Indicate OnAlerting() is be deferred or immediate.
+      */
+    void SetDeferredAlerting(bool defer) { m_deferredAlerting = defer; }
+
+    /**Indicate AcceptIncomingCall() execution is be deferred or immediate on OnIncomingCall().
+      */
+    bool IsDeferredAnswer() const { return m_deferredAnswer; }
+
+    /**Indicate AcceptIncomingCall() execution is be deferred or immediate on OnIncomingCall().
+      */
+    void SetDeferredAnswer(bool defer) { m_deferredAnswer = defer; }
   //@}
+
+  protected:
+    bool m_deferredAlerting;
+    bool m_deferredAnswer;
+
+  private:
+    P_REMOVE_VIRTUAL(OpalLocalConnection *, CreateConnection(OpalCall &, void *), 0);
 };
 
 
-/** PC Sound System connection.
+/** Local connection.
+    This class represents a connection on the local machine that can receive
+    media via virtual functions.
  */
 class OpalLocalConnection : public OpalConnection
 {
@@ -244,15 +295,18 @@ class OpalLocalConnection : public OpalConnection
   public:
   /**@name Construction */
   //@{
-    /**Create a new endpoint.
+    /**Create a new connection.
      */
     OpalLocalConnection(
-      OpalCall & call,              ///<  Owner calll for connection
+      OpalCall & call,              ///<  Owner call for connection
       OpalLocalEndPoint & endpoint, ///<  Owner endpoint for connection
-      void * userData               ///<  Arbitrary data to pass to connection
+      void * userData,              ///<  Arbitrary data to pass to connection
+      unsigned options,             ///< Option bit mask to pass to connection
+      OpalConnection::StringOptions * stringOptions, ///< Options to pass to connection
+      char tokenPrefix = 'L'        ///< Prefix for token generation
     );
 
-    /**Destroy endpoint.
+    /**Destroy connection.
      */
     ~OpalLocalConnection();
   //@}
@@ -292,6 +346,18 @@ class OpalLocalConnection : public OpalConnection
       PBoolean withMedia            ///<  Open media with alerting
     );
 
+    /**Indicate to remote endpoint we are connected.
+
+       The default behaviour sets the phase to ConnectedPhase, sets the
+       connection start time and then checks if there is any media channels
+       opened and if so, moves on to the established phase, calling
+       OnEstablished().
+
+       In other words, this method is used to handle incoming calls,
+       and is an indication that we have accepted the incoming call.
+      */
+    virtual PBoolean SetConnected();
+
     /**Open a new media stream.
        This will create a media stream of an appropriate subclass as required
        by the underlying connection protocol. For instance H.323 would create
@@ -312,6 +378,14 @@ class OpalLocalConnection : public OpalConnection
       PBoolean isSource                    ///<  Is a source stream
     );
 
+    /**Open source or sink media stream for session.
+      */
+    virtual OpalMediaStreamPtr OpenMediaStream(
+      const OpalMediaFormat & mediaFormat, ///<  Media format to open
+      unsigned sessionID,                  ///<  Session to start stream on
+      bool isSource                        ///< Stream is a source/sink
+    );
+
     /**Send a user input indication to the remote endpoint.
        This sends an arbitrary string as a user indication. If DTMF tones in
        particular are required to be sent then the SendIndicationTone()
@@ -326,12 +400,23 @@ class OpalLocalConnection : public OpalConnection
 
   /**@name New operations */
   //@{
+    /**Indicate alerting for the incoming connection.
+      */
+    virtual void AlertingIncoming();
+
     /**Accept the incoming connection.
       */
     virtual void AcceptIncoming();
   //@}
+
+  /**@name Member variable access */
+  //@{
+    /// Get user data pointer.
     void * GetUserData() const  { return userData; }
+
+    /// Set user data pointer.
     void SetUserData(void * v)  { userData = v; }
+  //@}
 
   protected:
     OpalLocalEndPoint & endpoint;
@@ -339,8 +424,9 @@ class OpalLocalConnection : public OpalConnection
 };
 
 
-/**This class describes a media stream that transfers data to/from a Local
-   EndPoint or Connection.
+/**Local media stream.
+    This class represents a media stream on the local machine that can receive
+    media via virtual functions.
   */
 class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
 {
@@ -348,10 +434,10 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
   public:
   /**@name Construction */
   //@{
-    /**Construct a new media stream for Line Interface Devices.
+    /**Construct a new media stream for local system.
       */
     OpalLocalMediaStream(
-      OpalLocalConnection & conn,
+      OpalLocalConnection & conn,          ///<  Connection for media stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -398,7 +484,7 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns PTrue for LID streams.
+       Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
   //@}
