@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21833 $
+ * $Revision: 24163 $
  * $Author: rjongbloed $
- * $Date: 2008-12-18 05:15:31 +0000 (Thu, 18 Dec 2008) $
+ * $Date: 2010-03-29 02:52:44 -0500 (Mon, 29 Mar 2010) $
  */
 
 #ifndef OPAL_OPAL_MEDIATYPE_H
@@ -39,9 +39,6 @@
 #pragma interface
 #endif
 
-namespace PWLibStupidLinkerHacks {
-extern int mediaTypeLoader;
-}; // namespace PWLibStupidLinkerHacks
 
 class OpalMediaTypeDefinition;
 class OpalSecurityMode;
@@ -80,10 +77,9 @@ class OpalMediaType : public std::string     // do not make this PCaselessString
     static const OpalMediaType & Fax();
     static const OpalMediaType & UserInput();
 
-    void PrintOn(ostream & strm) const { strm << c_str(); }
-
     OpalMediaTypeDefinition * GetDefinition() const;
     static OpalMediaTypeDefinition * GetDefinition(const OpalMediaType & key);
+    static OpalMediaTypeDefinition * GetDefinition(unsigned sessionId);
 
     static OpalMediaTypeFactory::KeyList_T GetList() { return OpalMediaTypeFactory::GetKeyList(); }
 
@@ -92,17 +88,29 @@ class OpalMediaType : public std::string     // do not make this PCaselessString
 #endif  // OPAL_SIP
 
     enum AutoStartMode {
-      DontOffer = -1, // Do not change order of enum as useful for bitmasking rx/tx
+      // Do not change order of enum as useful for bitmasking rx/tx
       OfferInactive,
       Receive,
       Transmit,
       ReceiveTransmit,
+      DontOffer,
+
       TransmitReceive = ReceiveTransmit
     };
+
+    __inline friend AutoStartMode operator++(AutoStartMode & mode)                 { return (mode = (AutoStartMode)(mode+1)); }
+    __inline friend AutoStartMode operator--(AutoStartMode & mode)                 { return (mode = (AutoStartMode)(mode-1)); }
+    __inline friend AutoStartMode operator|=(AutoStartMode & m1, AutoStartMode m2) { return (m1 = (AutoStartMode)((m1 & ~DontOffer) | m2)); }
+    __inline friend AutoStartMode operator-=(AutoStartMode & m1, AutoStartMode m2) { return (m1 = (AutoStartMode)((int)m1 & ~((int)m2|DontOffer))); }
+
     AutoStartMode GetAutoStart() const;
 };
 
-ostream & operator << (ostream & strm, const OpalMediaType & mediaType);
+
+__inline ostream & operator << (ostream & strm, const OpalMediaType & mediaType)
+{
+  return strm << mediaType.c_str();
+}
 
 
 ////////////////////////////////////////////////////////////////////////////
@@ -128,7 +136,7 @@ class OpalMediaTypeDefinition
     OpalMediaTypeDefinition(
       const char * mediaType,          ///< name of the media type (audio, video etc)
       const char * sdpType,            ///< name of the SDP type 
-      unsigned     preferredSessionId, ///< preferred session ID
+      unsigned requiredSessionId = 0,  ///< Session ID to use, asserts if already in use
       OpalMediaType::AutoStartMode autoStart = OpalMediaType::DontOffer   ///< Default value for auto-start transmit & receive
     );
 
@@ -143,6 +151,7 @@ class OpalMediaTypeDefinition
     /** Set flag for media type can auto-start receive on call initiation.
       */
     void SetAutoStart(OpalMediaType::AutoStartMode v) { m_autoStart = v; }
+    void SetAutoStart(OpalMediaType::AutoStartMode v, bool on) { if (on) m_autoStart |= v; else m_autoStart -= v; }
 
     /** Indicate type uses RTP for transport.
         If false, then it uses a generic OpaMediaSession
@@ -172,34 +181,16 @@ class OpalMediaTypeDefinition
     virtual RTP_UDP * CreateRTPSession(
       OpalRTPConnection & conn,
       unsigned sessionID, 
-      bool remoteIsNAT);
+      bool remoteIsNAT
+    );
 
     /** Return the default session ID for this media type.
       */
-    unsigned GetDefaultSessionId() const { return GetDefaultSessionId(m_mediaType); }
-
-    /** Return the default session ID for a specified media type.
-      */
-    static unsigned GetDefaultSessionId(
-      const OpalMediaType & mediaType   ///< Media type to get default session ID for
-    );
-
-    /** Return the media type associated with a default media type
-      */
-    static OpalMediaType GetMediaTypeForSessionId(
-      unsigned sessionId      ///< Session ID to search for media type.
-    );
+    unsigned GetDefaultSessionId() const { return m_defaultSessionId; }
 
   protected:
-    static PMutex & GetMapMutex();
-
-    typedef std::map<OpalMediaType, unsigned> MediaTypeToSessionIDMap_T;
-    static MediaTypeToSessionIDMap_T & GetMediaTypeToSessionIDMap();
-
-    typedef std::map<unsigned, OpalMediaType> SessionIDToMediaTypeMap_T;
-    static SessionIDToMediaTypeMap_T & GetSessionIDToMediaTypeMap();
-
     std::string m_mediaType;
+    unsigned    m_defaultSessionId;
     OpalMediaType::AutoStartMode m_autoStart;
 
 #if OPAL_SIP
@@ -282,7 +273,7 @@ class OpalRTPAVPMediaType : public OpalMediaTypeDefinition {
     OpalRTPAVPMediaType(
       const char * mediaType, 
       const char * sdpType, 
-      unsigned     preferredSessionId,
+      unsigned     requiredSessionId = 0,
       OpalMediaType::AutoStartMode autoStart = OpalMediaType::DontOffer
     );
 
@@ -334,6 +325,8 @@ class OpalFaxMediaType : public OpalMediaTypeDefinition
 #endif
 };
 
+PFACTORY_LOAD(T38PseudoRTP_Handler);
+
 #endif // OPAL_T38_CAPABILITY
 
 
