@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 24951 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2010-12-17 01:47:00 -0600 (Fri, 17 Dec 2010) $
  */
 
-#ifndef _OPAL_OPALVXML_H
-#define _OPAL_OPALVXML_H
+#ifndef OPAL_OPAL_OPALVXML_H
+#define OPAL_OPAL_OPALVXML_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -49,34 +49,30 @@ class OpalConnection;
 #if OPAL_PTLIB_VXML
 
 class PTextToSpeech;
+class OpalIVRConnection;
+
 
 class OpalVXMLSession : public PVXMLSession 
 {
   PCLASSINFO(OpalVXMLSession, PVXMLSession);
   public:
     OpalVXMLSession(
-      OpalConnection * _conn,
+      OpalIVRConnection & conn,
       PTextToSpeech * tts = NULL,
-      PBoolean autoDelete = PFalse
+      PBoolean autoDelete = false
     );
 
-    virtual PBoolean Close();
+    virtual void OnEndDialog();
     virtual void OnEndSession();
-
-    virtual PWAVFile * CreateWAVFile(
-      const PFilePath & fn,
-      PFile::OpenMode mode,
-      int opts,
-      unsigned fmt
-    );
+    virtual void OnTransfer(const PString & destination, bool bridged);
 
   protected:
-    OpalConnection * conn;
+    OpalIVRConnection & m_connection;
 };
 
 #endif
 
-#endif // _OPAL_OPALVXML_H
+#endif // OPAL_OPAL_OPALVXML_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
