@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 25780 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2011-05-16 21:05:05 -0500 (Mon, 16 May 2011) $
  */
 
 #ifndef OPAL_OPAL_TRANSCODERS_H
@@ -80,11 +80,11 @@ class OpalMediaFormatPair : public PObject
        usually overridden by descendent classes to yield the ranking according
        to the semantics of the object.
        
-       The default function is to use the #CompareObjectMemoryDirect()#
+       The default function is to use the <code>CompareObjectMemoryDirect()</code>
        function to do a byte wise memory comparison of the two objects.
 
        @return
-       #LessThan#, #EqualTo# or #GreaterThan#
+       <code>LessThan</code>, <code>EqualTo</code> or <code>GreaterThan</code>
        according to the relative rank of the objects.
      */
     virtual Comparison Compare(
@@ -125,7 +125,7 @@ __inline OpalTranscoderKey MakeOpalTranscoderKey(const char * from, const char *
 }
 
 #define OPAL_REGISTER_TRANSCODER(cls, input, output) \
-  OpalTranscoderFactory::Worker<cls> OpalTranscoder_##cls(MakeOpalTranscoderKey(input, output))
+  PFACTORY_CREATE(OpalTranscoderFactory, cls, MakeOpalTranscoderKey(input, output), false)
 
 
 /**This class embodies the implementation of a specific transcoder instance
@@ -169,10 +169,10 @@ class OpalTranscoder : public OpalMediaFormatPair
     );
 
     /**Execute the command specified to the transcoder. The commands are
-       highly context sensitive, for example VideoFastUpdate would only apply
-       to a video transcoder.
+       highly context sensitive, for example OpalVideoUpdatePicture would only
+       apply to a video transcoder.
 
-       The default behaviour simply returns PFalse.
+       The default behaviour simply returns false.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
@@ -196,7 +196,7 @@ class OpalTranscoder : public OpalMediaFormatPair
        The default behaviour makes sure the output list has only one element
        in it and calls the Convert() function.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean ConvertFrames(
       const RTP_DataFrame & input,  ///<  Input data
@@ -207,7 +207,7 @@ class OpalTranscoder : public OpalMediaFormatPair
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
@@ -233,10 +233,14 @@ class OpalTranscoder : public OpalMediaFormatPair
        There could be many possible matches between the two lists, so
        preference is given to the order of the destination formats.
 
-       Returns PFalse if there is no registered media transcoder that can be used
-       between the two named formats.
+       At least one of the two selected medai formats (source or destination)
+       must also be of the specified media type ("audio", "video" etc).
+
+       Returns false if there is no registered media transcoder that can be
+               used to get from src to dst for the media type.
       */
     static bool SelectFormats(
+      const OpalMediaType & mediaType,        ///< Media type for selection.
       const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
       const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
       const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
@@ -294,7 +298,7 @@ class OpalTranscoder : public OpalMediaFormatPair
     ) { maxOutputSize = size; }
 
     /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
+       commands are highly context sensitive, for example OpalVideoUpdatePicture
        would only apply to a video transcoder.
       */
     void SetCommandNotifier(
@@ -302,11 +306,22 @@ class OpalTranscoder : public OpalMediaFormatPair
     ) { commandNotifier = notifier; }
 
     /**Get the notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
+       commands are highly context sensitive, for example OpalVideoUpdatePicture
        would only apply to a video transcoder.
       */
     const PNotifier & GetCommandNotifier() const { return commandNotifier; }
 
+    /// Notify command notifier of command
+    void NotifyCommand(
+      const OpalMediaCommand & command
+    ) const;
+
+    /// Get session ID for the transcoder (from OpalMediaStream)
+    unsigned GetSessionID() const { return m_sessionID; }
+
+    /// Set session ID for the transcoder (from OpalMediaStream)
+    void SetSessionID(unsigned id) { m_sessionID = id; }
+
     /** Set the unique instance identifier for transcoder
       */
     virtual void SetInstanceID(
@@ -325,6 +340,8 @@ class OpalTranscoder : public OpalMediaFormatPair
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
+
+    void CopyTimestamp(RTP_DataFrame & dst, const RTP_DataFrame & src, bool inToOut) const;
   //@}
 
   protected:
@@ -332,9 +349,12 @@ class OpalTranscoder : public OpalMediaFormatPair
     PNotifier commandNotifier;
     PMutex    updateMutex;
 
-    PBoolean outputIsRTP, inputIsRTP;
-    bool acceptEmptyPayload;
-    bool acceptOtherPayloads;
+    unsigned m_sessionID;
+    bool     outputIsRTP, inputIsRTP;
+    bool     acceptEmptyPayload;
+    bool     acceptOtherPayloads;
+    unsigned m_inClockRate;
+    unsigned m_outClockRate;
 };
 
 
@@ -355,9 +375,7 @@ class OpalFramedTranscoder : public OpalTranscoder
       */
     OpalFramedTranscoder(
       const OpalMediaFormat & inputMediaFormat,  ///<  Input media format
-      const OpalMediaFormat & outputMediaFormat, ///<  Output media format
-      PINDEX inputBytesPerFrame,  ///<  Number of bytes in an input frame
-      PINDEX outputBytesPerFrame  ///<  Number of bytes in an output frame
+      const OpalMediaFormat & outputMediaFormat  ///<  Output media format
     );
   //@}
 
@@ -421,9 +439,11 @@ class OpalFramedTranscoder : public OpalTranscoder
   //@}
 
   protected:
-    PINDEX     inputBytesPerFrame;
-    PINDEX     outputBytesPerFrame;
-    PINDEX     maxOutputDataSize;
+    void CalculateSizes();
+
+    PINDEX inputBytesPerFrame;
+    PINDEX outputBytesPerFrame;
+    PINDEX maxOutputDataSize;
 };
 
 
@@ -466,7 +486,7 @@ class OpalStreamedTranscoder : public OpalTranscoder
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
@@ -518,11 +538,11 @@ class OpalEmptyFramedAudioTranscoder : public OpalFramedTranscoder
   PCLASSINFO(OpalEmptyFramedAudioTranscoder, OpalFramedTranscoder);
   public:
     OpalEmptyFramedAudioTranscoder(const char * inFormat, const char * outFormat)
-      : OpalFramedTranscoder(inFormat, outFormat, 100, 100)
+      : OpalFramedTranscoder(inFormat, outFormat)
     {  }
 
     PBoolean ConvertFrame(const BYTE *, PINDEX &, BYTE *, PINDEX &)
-    { return PFalse; }
+    { return false; }
 };
 
 #define OPAL_DECLARE_EMPTY_TRANSCODER(fmt) \
