@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 26231 $
+ * $Author: rjongbloed $
+ * $Date: 2011-07-19 21:23:34 -0500 (Tue, 19 Jul 2011) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -55,9 +55,100 @@ class OpalInternalTransport;
 
 ////////////////////////////////////////////////////////////////
 
-class OpalTransportAddress : public PString
+/**Describe a network transport address.
+   This describes a full definition of a network transport address, which is
+   usually more than just a host address. Broadly it consists of a low level
+   protocol and a protocol specific address/port.
+
+   At present only Internet Protocol based addresses are supported.
+
+   There are some semantic differences when this is being used as an address
+   to connect to, or as an address to listen on. The latter has several
+   "wildcard" variants so it is not necessary to explicitly know a-priori
+   everything about interfaces to start listening for incoming connections.
+
+   The syntax of the OpalTransportAddress string is:
+
+      transport-address = proto "$" proto-address
+
+      proto         = "ip" | "tcp" | "udp" | "tls" | token
+
+      proto-address = addrport | any-string
+      addrport      = addr [ ":" port ]
+
+      addr          = [ host ] [ "%" interface ]
+      host          = hostname | IPv4address | IPv6address | "*"
+
+      hostname      = *( domainlabel "." ) toplabel [ "." ]
+      domainlabel   = alphanum | alphanum *( alphanum | "-" ) alphanum
+      toplabel      = alpha | alpha *( alphanum | "-" ) alphanum
+
+      IPv4address   = 1*digit "." 1*digit "." 1*digit "." 1*digit
+
+      IPv6address   = "[" hexpart [ "%" scope-id ] "]"
+      hexpart       =  hexseq | hexseq "::" [ hexseq ] / "::" [ hexseq ]
+      hexseq        =  hex4 *( ":" hex4)
+      hex4          =  1*4 hexdigit
+      hexdigit      = digit | A-F | a-f
+      scope-id      = number | token
+
+      interface     = number | any-string
+
+      port          = number | token | "*"
+      number        = *digit
+
+      token       =  alpha * (alphanum)
+      alphanum    = alpha | digit
+      alpha       = A-Z | a-z
+      digit       = 0-9
+
+      any-string  = 1* (any-char)
+      any-char    = %20-%7f
+
+   The semantics of certain elements are as follows:
+      addr         Must have at least one of host or "%" interface-name
+      host = "*"   indicates INADDR_ANY, so is a wildcard match for all IP
+                   addresses.
+      port = "*"   indicates port 0. This is only allowed when the
+                   OpalTransportAddress is being used to "listen" on a socket.
+                   It indicates any port number will do and should be allocated
+                   by the operating system.
+      port = token Indicates a string as defined by getservbyname()
+      scope-id     Is dependent on the underlying operating system. For Win32
+                   this must be a number indicating the interface id, for Linux
+                   this may be string, e.g. "eth0". Generally, the scope-id is
+                   only required to disambiguate "link-local" addresses.
+      interface    The number here is not necessarily the same as the number in
+                   scope-id. It is the index into the table returned by the
+                   PIPSocket::GetInterfaceTable() function. The text version of
+                   the interface is dependent on the operating system. For Linux
+                   this is the expected "eth0" etc, for Win32 this can be a
+                   quite long and complex name string.
+
+   Examples:
+      tcp$*:1720
+          Listen  on all interfaces for TCP port 1720.
+      udp$127.0.0.1:5060
+          Indicate UDP port 5060 at local host
+      udp$%ppp0:5060
+          Listen on interface name ppp0, this may come on and off line at any
+          time, and change the actual IP address used at any time. The system
+          will do it's best to "do the right thing" in this situation.
+      udp$10.0.1.1%Win32 Interface Name:5060
+          Explicitly use address 10.0.1.1 on interface "Win32 interface name"
+      tcp$my.host.domain.com:1720"
+          A DNS lookup of my.host.domain.com is used to determine address.
+      udp$[::]:5060
+          Listen on all IPv6 interfaces. This differs from udp$*:5060 in that
+          only IPv6 interfaces are listened to. Similarly udp$0.0.0.0:5060
+          would only listen on IPv4 interfaces.
+      udp$[fe80::224:21ff:fe9e:3e7b%26]:5060
+          Send packet out on link-local address fe80::224:21ff:fe9e:3e7b and
+          interface id 26.
+  */
+class OpalTransportAddress : public PCaselessString
 {
-  PCLASSINFO(OpalTransportAddress, PString);
+  PCLASSINFO(OpalTransportAddress, PCaselessString);
   public:
   /**@name Construction */
   //@{
@@ -73,18 +164,22 @@ class OpalTransportAddress : public PString
       const char * proto = NULL  ///<  Default is "tcp"
     );
     OpalTransportAddress(
-      const PIPSocket::Address & ip,
-      WORD port,
-      const char * proto = NULL ///<  Default is "tcp"
+      const PIPSocket::Address & ip,  ///<  IP address for transport
+      WORD port,                      ///<  Default port number
+      const char * proto = NULL       ///<  Default is "tcp"
     );
   //@}
 
   /**@name Operations */
   //@{
     /**Determine if the two transport addresses are equivalent.
+       The second parameter indicates if wildcards are allowed. A wildcard
+       IP address is one for which isAny() returns true, a wildcard port
+       is the value 65535.
       */
     PBoolean IsEquivalent(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,   ///< Address to compare
+      bool wildcards = false   ///< Allow wildcards
     ) const;
 
     /**Determine if the two transport addresses are compatible.
@@ -93,15 +188,20 @@ class OpalTransportAddress : public PString
       const OpalTransportAddress & address
     ) const;
 
+    /**Return the underlying protocol for the transport address.
+      */
+    PCaselessString GetProto(bool withDollar = false) const { return Left(Find('$')+(withDollar?1:0)); }
+
     /**Extract the ip address from transport address.
-       Returns PFalse, if the address is not an IP transport address.
+       Returns false, if the address is not an IP transport address.
       */
     PBoolean GetIpAddress(PIPSocket::Address & ip) const;
 
     /**Extract the ip address and port number from transport address.
-       Returns PFalse, if the address is not an IP transport address.
+       Returns false, if the address is not an IP transport address.
       */
     PBoolean GetIpAndPort(PIPSocket::Address & ip, WORD & port) const;
+    PBoolean GetIpAndPort(PIPSocketAddressAndPort & ipPort) const;
 
     /**Translate the transport address to a more human readable form.
        Returns the hostname if using IP.
@@ -283,9 +383,9 @@ class OpalListener : public PObject
         OpalTransport instance created by the listener. If this is NULL then
         it indicates an error occurred during the accept.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -317,16 +417,14 @@ class OpalListener : public PObject
     ) const = 0;
 
     /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & preferredAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
     ) const = 0;
 
-    /**Get the local transport address on which this listener may be accessed.
-       For backward compatibility with OpenH323, now deprecated.
-      */
-    OpalTransportAddress GetTransportAddress() const { return GetLocalAddress(); }
-
     /**Close channel and wait for associated thread to terminate.
       */
     void CloseWait();
@@ -375,7 +473,7 @@ class OpalListenerIP : public OpalListener
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerIP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -387,9 +485,12 @@ class OpalListenerIP : public OpalListener
   /**@name Overrides from OpalListener */
   //@{
     /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & preferredAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
     ) const;
   //@}
 
@@ -420,7 +521,7 @@ class OpalListenerTCP : public OpalListenerIP
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerTCP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -444,9 +545,9 @@ class OpalListenerTCP : public OpalListenerIP
         new connection is created. The INT parameter to the acceptHandler is
         a pointer to the new OpalTransport instance created by the listener.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -502,7 +603,7 @@ class OpalListenerUDP : public OpalListenerIP
       OpalEndPoint & endpoint,  ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,            ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true  ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerUDP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -526,9 +627,9 @@ class OpalListenerUDP : public OpalListenerIP
         new connection is created. The INT parameter to the acceptHandler is
         a pointer to the new OpalTransport instance created by the listener.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -562,6 +663,24 @@ class OpalListenerUDP : public OpalListenerIP
       const OpalTransportAddress & localAddress,
       const OpalTransportAddress & remoteAddress
     ) const;
+
+    /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
+      */
+    virtual OpalTransportAddress GetLocalAddress(
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+    ) const;
+  //@}
+
+  /**@name Member access */
+  //@{
+    /**Set the size of UDP packet reads.
+      */
+    void SetBufferSize(
+      PINDEX size
+    ) { m_bufferSize = size; }
   //@}
 
 
@@ -569,6 +688,7 @@ class OpalListenerUDP : public OpalListenerIP
     virtual const char * GetProtoPrefix() const;
 
     PMonitoredSocketsPtr listenerBundle;
+    PINDEX               m_bufferSize;
 };
 
 
@@ -628,7 +748,9 @@ class OpalTransport : public PIndirectChannel
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const = 0;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const = 0;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
@@ -718,6 +840,10 @@ class OpalTransport : public PIndirectChannel
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & packet   ///<  Packet read from transport
@@ -796,7 +922,9 @@ class OpalTransportIP : public OpalTransport
   //@{
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
@@ -845,7 +973,7 @@ class OpalTransportTCP : public OpalTransportIP
       OpalEndPoint & endpoint,    ///<  Endpoint object
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
       WORD port = 0,              ///<  Local port to bind to
-      PBoolean reuseAddr = PFalse      ///<  Flag for binding to already bound interface
+      PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
     );
     OpalTransportTCP(
       OpalEndPoint & endpoint,    ///<  Endpoint object
@@ -876,6 +1004,10 @@ class OpalTransportTCP : public OpalTransportIP
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & pdu  ///<  PDU read from transport
@@ -901,10 +1033,10 @@ class OpalTransportTCP : public OpalTransportIP
        open channels. It may be used by descendent channels to do any
        handshaking required by the protocol that channel embodies.
 
-       The default behaviour is to simply return PTrue.
+       The default behaviour is to simply return true.
 
        @return
-       Returns PTrue if the protocol handshaking is successful.
+       Returns true if the protocol handshaking is successful.
      */
     virtual PBoolean OnOpen();
 
@@ -933,11 +1065,8 @@ class OpalTransportUDP : public OpalTransportIP
      */
     OpalTransportUDP(
       OpalEndPoint & endpoint,              ///<  Endpoint object
-      const PBYTEArray & preReadPacket,     ///<  Packet already read by OpalListenerUDP
       const PMonitoredSocketsPtr & sockets, ///<  Bundle of sockets from OpalListenerUDP
-      const PString & iface,                ///<  Local interface to use
-      PIPSocket::Address remoteAddress,     ///<  Remote address received PDU on
-      WORD remotePort                       ///<  Remote port received PDU on
+      const PString & iface                 ///<  Local interface to use
     );
 
     /// Destroy the UDP channel
@@ -989,7 +1118,9 @@ class OpalTransportUDP : public OpalTransportIP
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
@@ -1039,6 +1170,10 @@ class OpalTransportUDP : public OpalTransportIP
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & packet   ///<  Packet read from transport
@@ -1067,6 +1202,12 @@ class OpalTransportUDP : public OpalTransportIP
       WriteConnectCallback function,  ///<  Function for writing data
       void * userData                 ///<  User data to pass to write function
     );
+
+    /**Set the size of UDP packet reads.
+      */
+    void SetBufferSize(
+      PINDEX size
+    ) { m_bufferSize = size; }
   //@}
 
   protected:
@@ -1075,7 +1216,11 @@ class OpalTransportUDP : public OpalTransportIP
     virtual const char * GetProtoPrefix() const;
 
     OpalManager & manager;
-    PBYTEArray    preReadPacket;
+    PINDEX        m_bufferSize;
+    PBYTEArray    m_preReadPacket;
+    bool          m_preReadOK;
+
+  friend class OpalListenerUDP;
 };
 
 
@@ -1179,7 +1324,7 @@ class OpalListenerTCPS : public OpalListenerTCP
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerTCPS(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -1208,7 +1353,7 @@ class OpalTransportTCPS : public OpalTransportTCP
         OpalEndPoint & endpoint,    ///<  Endpoint object
         PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
         WORD port = 0,              ///<  Local port to bind to
-        PBoolean reuseAddr = PFalse      ///<  Flag for binding to already bound interface
+        PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
       );
       OpalTransportTCPS(
         OpalEndPoint & endpoint,    ///<  Endpoint object
