@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21986 $
+ * $Revision: 26293 $
  * $Author: rjongbloed $
- * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
+ * $Date: 2011-08-08 21:25:33 -0500 (Mon, 08 Aug 2011) $
  */
 
 #ifndef OPAL_RTP_RTP_H
@@ -41,6 +41,11 @@
 #include <opal/buildopts.h>
 
 #include <ptlib/sockets.h>
+#include <ptlib/safecoll.h>
+
+#include <list>
+
+#include <rtp/metrics.h>
 
 class RTP_JitterBuffer;
 class PNatMethod;
@@ -68,8 +73,8 @@ class RTP_DataFrame : public PBYTEArray
   PCLASSINFO(RTP_DataFrame, PBYTEArray);
 
   public:
-    RTP_DataFrame(PINDEX payloadSize, PINDEX bufferSize = 0);
-    RTP_DataFrame(const BYTE * data, PINDEX len, PBoolean dynamic = PTrue);
+    RTP_DataFrame(PINDEX payloadSize = 0, PINDEX bufferSize = 0);
+    RTP_DataFrame(const BYTE * data, PINDEX len, PBoolean dynamic = true);
 
     enum {
       ProtocolVersion = 2,
@@ -108,6 +113,8 @@ class RTP_DataFrame : public PBYTEArray
       MP2T,         // MPEG2 transport system
       H263,         // H.263
 
+      T38 = 38,     // T.38 (internal)
+
       LastKnownPayloadType,
 
       DynamicBase = 96,
@@ -125,8 +132,10 @@ class RTP_DataFrame : public PBYTEArray
 
     bool GetPadding() const { return (theArray[0]&0x20) != 0; }
     void SetPadding(bool v)  { if (v) theArray[0] |= 0x20; else theArray[0] &= 0xdf; }
+    BYTE * GetPaddingPtr() const { return (BYTE *)(theArray+m_headerSize+m_payloadSize); }
 
-    unsigned GetPaddingSize() const;
+    unsigned GetPaddingSize() const { return m_paddingSize; }
+    bool     SetPaddingSize(PINDEX sz);
 
     PayloadTypes GetPayloadType() const { return (PayloadTypes)(theArray[1]&0x7f); }
     void         SetPayloadType(PayloadTypes t);
@@ -144,22 +153,29 @@ class RTP_DataFrame : public PBYTEArray
     DWORD  GetContribSource(PINDEX idx) const;
     void   SetContribSource(PINDEX idx, DWORD src);
 
-    PINDEX GetHeaderSize() const;
+    PINDEX GetHeaderSize() const { return m_headerSize; }
 
     int GetExtensionType() const; // -1 is no extension
     void   SetExtensionType(int type);
-    PINDEX GetExtensionSize() const;
-    PBoolean   SetExtensionSize(PINDEX sz);
+    PINDEX GetExtensionSizeDWORDs() const;      // get the number of 32 bit words in the extension (excluding the header).
+    bool   SetExtensionSizeDWORDs(PINDEX sz);   // set the number of 32 bit words in the extension (excluding the header)
     BYTE * GetExtensionPtr() const;
 
-    PINDEX GetPayloadSize() const { return payloadSize - GetPaddingSize(); }
-    PBoolean   SetPayloadSize(PINDEX sz);
-    BYTE * GetPayloadPtr()     const { return (BYTE *)(theArray+GetHeaderSize()); }
+    PINDEX GetPayloadSize() const { return m_payloadSize; }
+    bool   SetPayloadSize(PINDEX sz);
+    BYTE * GetPayloadPtr()     const { return (BYTE *)(theArray+m_headerSize); }
 
+    virtual PObject * Clone() const { return new RTP_DataFrame(*this); }
     virtual void PrintOn(ostream & strm) const;
 
+    // Note this sets the whole packet length, and calculates the various
+    // sub-section sizes: header payload and padding.
+    bool SetPacketSize(PINDEX sz);
+
   protected:
-    PINDEX payloadSize;
+    PINDEX m_headerSize;
+    PINDEX m_payloadSize;
+    PINDEX m_paddingSize;
 
 #if PTRACING
     friend ostream & operator<<(ostream & o, PayloadTypes t);
@@ -184,12 +200,15 @@ class RTP_ControlFrame : public PBYTEArray
     void     SetCount(unsigned count);
 
     enum PayloadTypes {
-      e_IntraFrameRequest = 192,
-      e_SenderReport = 200,
-      e_ReceiverReport,
-      e_SourceDescription,
-      e_Goodbye,
-      e_ApplDefined
+      e_IntraFrameRequest       = 192,
+      e_SenderReport            = 200,
+      e_ReceiverReport          = 201,
+      e_SourceDescription       = 202,
+      e_Goodbye                 = 203,
+      e_ApplDefined             = 204,
+      e_TransportLayerFeedBack  = 205, // RFC4585
+      e_PayloadSpecificFeedBack = 206,
+      e_ExtendedReport          = 207  // RFC3611
     };
 
     unsigned GetPayloadType() const { return (BYTE)theArray[compoundOffset+1]; }
@@ -230,6 +249,35 @@ class RTP_ControlFrame : public PBYTEArray
       PUInt32b osent;     /* octets sent */ 
     };
 
+    struct ExtendedReport {
+      /* VoIP Metrics Report Block */
+      BYTE bt;                     /* block type */
+      BYTE type_specific;          /* determined by the block definition */
+      PUInt16b length;             /* length of the report block */
+      PUInt32b ssrc;               /* data source being reported */
+      BYTE loss_rate;              /* fraction of RTP data packets lost */ 
+      BYTE discard_rate;           /* fraction of RTP data packets discarded */
+      BYTE burst_density;          /* fraction of RTP data packets within burst periods */
+      BYTE gap_density;            /* fraction of RTP data packets within inter-burst gaps */
+      PUInt16b burst_duration;     /* the mean duration, in ms, of burst periods */
+      PUInt16b gap_duration;       /* the mean duration, in ms, of gap periods */
+      PUInt16b round_trip_delay;   /* the most recently calculated round trip time */    
+      PUInt16b end_system_delay;   /* the most recently estimates end system delay */
+      BYTE signal_level;           /* voice signal level related to 0 dBm */
+      BYTE noise_level;            /* ratio of the silent background level to 0 dBm */
+      BYTE rerl;                   /* residual echo return loss */
+      BYTE gmin;                   /* gap threshold */
+      BYTE r_factor;               /* voice quality metric of the call */
+      BYTE ext_r_factor;           /* external R factor */
+      BYTE mos_lq;                 /* MOS for listen quality */
+      BYTE mos_cq;                 /* MOS for conversational quality */
+      BYTE rx_config;              /* receiver configuration byte */
+      BYTE reserved;               /* reserved for future definition */
+      PUInt16b jb_nominal;         /* current nominal jitter buffer delay, in ms */ 
+      PUInt16b jb_maximum;         /* current maximum jitter buffer delay, in ms */
+      PUInt16b jb_absolute;        /* current absolute maximum jitter buffer delay, in ms */
+    };
+
     enum DescriptionTypes {
       e_END,
       e_CNAME,
@@ -268,6 +316,41 @@ class RTP_ControlFrame : public PBYTEArray
       unsigned type,            ///<  Description type
       const PString & data      ///<  Data for description
     );
+
+    // RFC4585 Feedback Message Type (FMT)
+    unsigned GetFbType() const { return (BYTE)theArray[compoundOffset]&0x1f; }
+    void     SetFbType(unsigned type, PINDEX fciSize);
+
+    enum PayloadSpecificFbTypes {
+      e_PictureLossIndication = 1,
+      e_SliceLostIndication,
+      e_ReferencePictureSelectionIndication,
+      e_FullIntraRequest,                     //RFC5104
+      e_TemporalSpatialTradeOffRequest,
+      e_TemporalSpatialTradeOffNotification,
+      e_VideoBackChannelMessage,
+      e_ApplicationLayerFbMessage = 15
+    };
+
+    struct FbFCI {
+      PUInt32b senderSSRC;  /* data source of sender of message */
+      PUInt32b mediaSSRC;   /* data source of media */
+    };
+
+    struct FbFIR {
+      FbFCI    fci;
+      PUInt32b requestSSRC;
+      BYTE     sequenceNUmber;
+    };
+
+    struct FbTSTO {
+      FbFCI    fci;
+      PUInt32b requestSSRC;
+      BYTE     sequenceNUmber;
+      BYTE     reserver[2];
+      BYTE     tradeOff;
+    };
+
 #pragma pack()
 
   protected:
@@ -304,10 +387,51 @@ class OpalMediaStatistics : public PObject
     // Audio
     unsigned m_averageJitter;
     unsigned m_maximumJitter;
+    unsigned m_jitterBufferDelay;
 
     // Video
     unsigned m_totalFrames;
     unsigned m_keyFrames;
+
+    // Fax
+#if OPAL_FAX
+    enum {
+      FaxNotStarted = -2,
+      FaxInProgress = -1,
+      FaxSuccessful = 0,
+      FaxErrorBase  = 1
+    };
+    enum FaxCompression {
+      FaxCompressionUnknown,
+      FaxCompressionT4_1d,
+      FaxCompressionT4_2d,
+      FaxCompressionT6,
+    };
+    friend ostream & operator<<(ostream & strm, FaxCompression compression);
+    struct Fax {
+      Fax();
+
+      int  m_result;      // -2=not started, -1=progress, 0=success, >0=ended with error
+      char m_phase;       // 'A', 'B', 'D'
+      int  m_bitRate;     // e.g. 14400, 9600
+      FaxCompression m_compression; // 0=N/A, 1=T.4 1d, 2=T.4 2d, 3=T.6
+      bool m_errorCorrection;
+      int  m_txPages;
+      int  m_rxPages;
+      int  m_totalPages;
+      int  m_imageSize;   // In bytes
+      int  m_resolutionX; // Pixels per inch
+      int  m_resolutionY; // Pixels per inch
+      int  m_pageWidth;
+      int  m_pageHeight;
+      int  m_badRows;     // Total number of bad rows
+      int  m_mostBadRows; // Longest run of bad rows
+      int  m_errorCorrectionRetries;
+
+      PString m_stationId; // Remote station identifier
+      PString m_errorText;
+    } m_fax;
+#endif
 };
 
 #endif
@@ -406,7 +530,7 @@ class RTP_Session : public PObject
     );
 
     /**Delete a session.
-       This deletes the userData field if autoDeleteUserData is PTrue.
+       This deletes the userData field if autoDeleteUserData is true.
      */
     ~RTP_Session();
   //@}
@@ -414,19 +538,17 @@ class RTP_Session : public PObject
   /**@name Operations */
   //@{
     /**Sets the size of the jitter buffer to be used by this RTP session.
-       A session default to not having any jitter buffer enabled for reading
-       and the ReadBufferedData() function simply calls ReadData(). Once a
-       jitter buffer has been created it cannot be removed, though its size
-       may be adjusted.
-       
-       If the jitterDelay paramter is zero, it destroys the jitter buffer
+       A session defaults to not having any jitter buffer enabled for reading
+       and the ReadBufferedData() function simply calls ReadData().
+
+       If either jitter delay parameter is zero, it destroys the jitter buffer
        attached to this RTP session.
       */
     void SetJitterBufferSize(
       unsigned minJitterDelay, ///<  Minimum jitter buffer delay in RTP timestamp units
       unsigned maxJitterDelay, ///<  Maximum jitter buffer delay in RTP timestamp units
-      unsigned timeUnits = 8,  ///<  Time Units
-      PINDEX stackSize = 30000 ///<  Stack size for jitter thread
+      unsigned timeUnits = 0,  ///<  Time Units, zero uses default
+      PINDEX packetSize = 2048 ///<  Receive RTP packet size
     );
 
     /**Get current size of the jitter buffer.
@@ -435,14 +557,15 @@ class RTP_Session : public PObject
        the SetJitterBufferSize() function.
       */
     unsigned GetJitterBufferSize() const;
+    unsigned GetJitterBufferDelay() const { return GetJitterBufferSize()/GetJitterTimeUnits(); }
     
     /**Get current time units of the jitter buffer.
      */
-    unsigned GetJitterTimeUnits() const;
+    unsigned GetJitterTimeUnits() const { return m_timeUnits; }
 
     /**Modifies the QOS specifications for this RTP session*/
     virtual PBoolean ModifyQOS(RTP_QOS * )
-    { return PFalse; }
+    { return false; }
 
     /**Read a data frame from the RTP channel.
        This function will conditionally read data from the jitter buffer or
@@ -459,8 +582,7 @@ class RTP_Session : public PObject
        available or an error occurs.
       */
     virtual PBoolean ReadData(
-      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop               ///<  If PTrue, loop as long as data is available, if PFalse, only process once
+      RTP_DataFrame & frame   ///<  Frame read from the RTP session
     ) = 0;
 
     /**Write a data frame from the RTP channel.
@@ -489,7 +611,7 @@ class RTP_Session : public PObject
 
     /**Close down the RTP session.
       */
-    virtual void Close(
+    virtual bool Close(
       PBoolean reading    ///<  Closing the read side of the session
     ) = 0;
 
@@ -552,10 +674,12 @@ class RTP_Session : public PObject
         DWORD packetsSent;
         DWORD octetsSent;
     };
+
     virtual void OnRxSenderReport(const SenderReport & sender,
                                   const ReceiverReportArray & reports);
     virtual void OnRxReceiverReport(DWORD src,
                                     const ReceiverReportArray & reports);
+    virtual void OnReceiverReports(const ReceiverReportArray & reports);
 
     class SourceDescription : public PObject  {
         PCLASSINFO(SourceDescription, PObject);
@@ -574,6 +698,31 @@ class RTP_Session : public PObject
 
     virtual void OnRxApplDefined(const PString & type, unsigned subtype, DWORD src,
                                  const BYTE * data, PINDEX size);
+
+#if OPAL_RTCP_XR
+    class ExtendedReport : public PObject  {
+        PCLASSINFO(ExtendedReport, PObject);
+      public:
+        void PrintOn(ostream &) const;
+
+        DWORD sourceIdentifier;
+        DWORD lossRate;            /* fraction of RTP data packets lost */ 
+        DWORD discardRate;         /* fraction of RTP data packets discarded */
+        DWORD burstDensity;        /* fraction of RTP data packets within burst periods */
+        DWORD gapDensity;          /* fraction of RTP data packets within inter-burst gaps */
+        DWORD roundTripDelay;  /* the most recently calculated round trip time */    
+        DWORD RFactor;            /* voice quality metric of the call */
+        DWORD mosLQ;               /* MOS for listen quality */
+        DWORD mosCQ;               /* MOS for conversational quality */
+        DWORD jbNominal;      /* current nominal jitter buffer delay, in ms */ 
+        DWORD jbMaximum;      /* current maximum jitter buffer delay, in ms */
+        DWORD jbAbsolute;     /* current absolute maximum jitter buffer delay, in ms */
+    };
+    PARRAY(ExtendedReportArray, ExtendedReport);
+
+    virtual void OnRxExtendedReport(DWORD src,
+                                    const ExtendedReportArray & reports);                             
+#endif
   //@}
 
   /**@name Member variable access */
@@ -582,6 +731,10 @@ class RTP_Session : public PObject
       */
     unsigned GetSessionID() const { return sessionID; }
 
+    /**Set the ID for the RTP session.
+      */
+    void SetSessionID(unsigned id) { sessionID = id; }
+
     /**Get flag for is audio RTP.
       */
     bool IsAudio() const { return isAudio; }
@@ -616,7 +769,7 @@ class RTP_Session : public PObject
       */
     void SetUserData(
       RTP_UserData * data,            ///<  New user data to be used
-      PBoolean autoDeleteUserData = PTrue  ///<  Delete optional data with session.
+      PBoolean autoDeleteUserData = true  ///<  Delete optional data with session.
     );
 
     /**Get the source output identifier.
@@ -633,16 +786,6 @@ class RTP_Session : public PObject
       bool allow    ///<  Flag for allow any SSRC values
     ) { allowAnySyncSource = allow; }
 
-    /**Indicate if will ignore out of order packets.
-      */
-    PBoolean WillIgnoreOutOfOrderPackets() const { return ignoreOutOfOrderPackets; }
-
-    /**Indicate if will ignore out of order packets.
-      */
-    void SetIgnoreOutOfOrderPackets(
-      PBoolean ignore   ///<  Flag for ignore out of order packets
-    ) { ignoreOutOfOrderPackets = ignore; }
-
     /**Indicate if will ignore rtp payload type changes in received packets.
      */
     void SetIgnorePayloadTypeChanges(
@@ -708,6 +851,11 @@ class RTP_Session : public PObject
       */
     DWORD GetPacketsLost() const { return packetsLost; }
 
+    /**Get total number transmitted packets lost by remote in session.
+       Determined via RTCP.
+      */
+    DWORD GetPacketsLostByRemote() const { return packetsLostByRemote; }
+
     /**Get total number of packets received out of order in session.
       */
     DWORD GetPacketsOutOfOrder() const { return packetsOutOfOrder; }
@@ -768,24 +916,23 @@ class RTP_Session : public PObject
       */
     DWORD GetMinimumReceiveTime() const { return minimumReceiveTime; }
 
+    enum { JitterRoundingGuardBits = 4 };
     /**Get averaged jitter time for received packets.
        This is the calculated statistical variance of the interarrival
        time of received packets in milliseconds.
       */
-    DWORD GetAvgJitterTime() const { return jitterLevel>>7; }
+    DWORD GetAvgJitterTime() const { return (jitterLevel>>JitterRoundingGuardBits)/GetJitterTimeUnits(); }
 
     /**Get averaged jitter time for received packets.
        This is the maximum value of jitterLevel for the session.
       */
-    DWORD GetMaxJitterTime() const { return maximumJitterLevel>>7; }
-  //@}
+    DWORD GetMaxJitterTime() const { return (maximumJitterLevel>>JitterRoundingGuardBits)/GetJitterTimeUnits(); }
 
-  /**@name Functions added to support RTP aggregation */
-  //@{
-    virtual int GetDataSocketHandle() const
-    { return -1; }
-    virtual int GetControlSocketHandle() const
-    { return -1; }
+    /**Get jitter time for received packets on remote.
+       This is the calculated statistical variance of the interarrival
+       time of received packets in milliseconds.
+      */
+    DWORD GetJitterTimeOnRemote() const { return jitterLevelOnRemote/GetJitterTimeUnits(); }
   //@}
 
     virtual void SetCloseOnBYE(PBoolean v)  { closeOnBye = v; }
@@ -794,7 +941,13 @@ class RTP_Session : public PObject
         This is called when the media stream receives an OpalVideoUpdatePicture
         media command.
       */
-    virtual void SendIntraFrameRequest();
+    virtual void SendIntraFrameRequest(bool rfc2032, bool pictureLoss);
+
+    /** Tell the rtp session to send out an temporal spatial trade off request
+        control packet. This is called when the media stream receives an
+        OpalTemporalSpatialTradeOff media command.
+      */
+    virtual void SendTemporalSpatialTradeOff(unsigned tradeOff);
 
     void SetNextSentSequenceNumber(WORD num) { lastSentSequenceNumber = (WORD)(num-1); }
 
@@ -824,12 +977,27 @@ class RTP_Session : public PObject
     bool HasFailed() const
     { return failed; }
 
-    void AddFilter(const PNotifier & filter);
+    typedef PNotifierTemplate<SendReceiveStatus &> FilterNotifier;
+    #define PDECLARE_RTPFilterNotifier(cls, fn) PDECLARE_NOTIFIER2(RTP_DataFrame, cls, fn, RTP_Session::SendReceiveStatus &)
+    #define PCREATE_RTPFilterNotifier(fn) PCREATE_NOTIFIER2(fn, RTP_Session::SendReceiveStatus &)
+
+    void AddFilter(const FilterNotifier & filter);
+
+#if OPAL_RTCP_XR
+    const RTCP_XR_Metrics & GetMetrics() const { return m_metrics; }
+#endif
+
+    virtual void SendBYE();
 
   protected:
-    virtual void SendBYE();
     void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
+
     PBoolean InsertReportPacket(RTP_ControlFrame & report);
+    
+#if OPAL_RTCP_XR
+    void InsertExtendedReportPacket(RTP_ControlFrame & report);
+    void OnRxSenderReportToMetrics(const RTP_ControlFrame & frame, PINDEX offset);    
+#endif
 
     PString             m_encoding;
     PMutex              m_encodingMutex;
@@ -837,13 +1005,15 @@ class RTP_Session : public PObject
 
     unsigned           sessionID;
     bool               isAudio;
+    unsigned           m_timeUnits;
     PString            canonicalName;
     PString            toolName;
     RTP_UserData     * userData;
     PBoolean           autoDeleteUserData;
-    RTP_JitterBuffer * jitter;
 
-    PBoolean      ignoreOutOfOrderPackets;
+    typedef PSafePtr<RTP_JitterBuffer, PSafePtrMultiThreaded> JitterBufferPtr;
+    JitterBufferPtr m_jitterBuffer;
+
     DWORD         syncSourceOut;
     DWORD         syncSourceIn;
     DWORD         lastSentTimestamp;
@@ -858,22 +1028,34 @@ class RTP_Session : public PObject
     WORD          expectedSequenceNumber;
     PTimeInterval lastSentPacketTime;
     PTimeInterval lastReceivedPacketTime;
+    PTime         lastSRTimestamp;
+    PTime         lastSRReceiveTime;
+    PTimeInterval delaySinceLastSR;
     WORD          lastRRSequenceNumber;
-    PINDEX        consecutiveOutOfOrderPackets;
+    bool          resequenceOutOfOrderPackets;
+    unsigned      consecutiveOutOfOrderPackets;
+    PTimeInterval outOfOrderPacketTime;
+
+    std::list<RTP_DataFrame> m_outOfOrderPackets;
+    void SaveOutOfOrderPacket(RTP_DataFrame & frame);
 
     PMutex        dataMutex;
     DWORD         timeStampOffs;               // offset between incoming media timestamp and timeStampOut
-    PBoolean      oobTimeStampBaseEstablished; // PTrue if timeStampOffs has been established by media
+    PBoolean      oobTimeStampBaseEstablished; // true if timeStampOffs has been established by media
     DWORD         oobTimeStampOutBase;         // base timestamp value for oob data
     PTimeInterval oobTimeStampBase;            // base time for oob timestamp
 
     // Statistics
+    PTime firstPacketSent;
     DWORD packetsSent;
     DWORD rtcpPacketsSent;
     DWORD octetsSent;
+    PTime firstPacketReceived;
     DWORD packetsReceived;
+    DWORD senderReportsReceived;
     DWORD octetsReceived;
     DWORD packetsLost;
+    DWORD packetsLostByRemote;
     DWORD packetsOutOfOrder;
     DWORD averageSendTime;
     DWORD maximumSendTime;
@@ -882,6 +1064,7 @@ class RTP_Session : public PObject
     DWORD maximumReceiveTime;
     DWORD minimumReceiveTime;
     DWORD jitterLevel;
+    DWORD jitterLevelOnRemote;
     DWORD maximumJitterLevel;
 
     DWORD markerSendCount;
@@ -889,6 +1072,11 @@ class RTP_Session : public PObject
 
     unsigned txStatisticsCount;
     unsigned rxStatisticsCount;
+    
+#if OPAL_RTCP_XR
+    // Calculate the VoIP Metrics for RTCP-XR
+    RTCP_XR_Metrics m_metrics;
+#endif
 
     DWORD    averageSendTimeAccum;
     DWORD    maximumSendTimeAccum;
@@ -909,13 +1097,7 @@ class RTP_Session : public PObject
     PBoolean byeSent;
     bool                failed;      ///<  set to true if session has received too many ICMP destination unreachable
 
-    class Filter : public PObject {
-        PCLASSINFO(Filter, PObject);
-      public:
-        Filter(const PNotifier & n) : notifier(n) { }
-        PNotifier notifier;
-    };
-    PList<Filter> filters;
+    list<FilterNotifier> m_filters;
 };
 
 /**This class is for the IETF Real Time Protocol interface on UDP/IP.
@@ -944,8 +1126,8 @@ class RTP_UDP : public RTP_Session
        returned by this function. It will block until a data frame is
        available or an error occurs.
       */
-    virtual PBoolean ReadData(RTP_DataFrame & frame, PBoolean loop);
-    virtual PBoolean Internal_ReadData(RTP_DataFrame & frame, PBoolean loop);
+    virtual PBoolean ReadData(RTP_DataFrame & frame);
+    virtual PBoolean Internal_ReadData(RTP_DataFrame & frame);
 
     /** Write a data frame to the RTP channel.
       */
@@ -963,7 +1145,7 @@ class RTP_UDP : public RTP_Session
 
     /**Close down the RTP session.
       */
-    virtual void Close(
+    virtual bool Close(
       PBoolean reading    ///<  Closing the read side of the session
     );
 
@@ -1073,7 +1255,7 @@ class RTP_UDP : public RTP_Session
       PINDEX frameSize,
       PBoolean fromDataChannel
     );
-    
+
     virtual bool WriteDataPDU(RTP_DataFrame & frame);
     virtual bool WriteDataOrControlPDU(
       const BYTE * framePtr,
@@ -1081,6 +1263,8 @@ class RTP_UDP : public RTP_Session
       bool toDataChannel
     );
 
+    virtual void SetEncoding(const PString & newEncoding);
+
 
   protected:
     PIPSocket::Address localAddress;
@@ -1101,9 +1285,13 @@ class RTP_UDP : public RTP_Session
     bool appliedQOS;
     bool remoteIsNAT;
     bool localHasNAT;
-    bool first;
+    bool m_firstData;
+    bool m_firstControl;
     int  badTransmitCounter;
     PTime badTransmitStart;
+
+    PTimer timerWriteDataIdle;
+    PDECLARE_NOTIFIER(PTimer,  RTP_UDP, OnWriteDataIdle);
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1115,16 +1303,19 @@ class RTP_Encoding
   public:
     RTP_Encoding();
     virtual ~RTP_Encoding();
+    virtual void ApplyStringOptions(const PStringToString & /*stringOptions*/) {}
     virtual void OnStart(RTP_Session & _rtpSession);
     virtual void OnFinish();
     virtual RTP_Session::SendReceiveStatus OnSendData(RTP_DataFrame & frame);
-    virtual PBoolean WriteData(RTP_DataFrame & frame);
+    virtual PBoolean WriteData(RTP_DataFrame & frame, bool oob);
     virtual PBoolean WriteDataPDU(RTP_DataFrame & frame);
+    virtual void OnWriteDataIdle() {}
+    virtual void SetWriteDataIdleTimer(PTimer &) {}
     virtual RTP_Session::SendReceiveStatus OnSendControl(RTP_ControlFrame & frame, PINDEX & len);
     virtual RTP_Session::SendReceiveStatus ReadDataPDU(RTP_DataFrame & frame);
     virtual RTP_Session::SendReceiveStatus OnReceiveData(RTP_DataFrame & frame);
     virtual RTP_Session::SendReceiveStatus OnReadTimeout(RTP_DataFrame & frame);
-    virtual PBoolean ReadData(RTP_DataFrame & frame, PBoolean loop);
+    virtual PBoolean ReadData(RTP_DataFrame & frame);
     virtual int WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, const PTimeInterval &);
 
     PMutex      mutex;
@@ -1134,6 +1325,9 @@ class RTP_Encoding
     RTP_UDP     * rtpUDP;
 };
 
+PFACTORY_LOAD(RTP_Encoding);
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 class SecureRTP_UDP : public RTP_UDP
