@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23287 $
+ * $Revision: 26283 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 01:42:00 +0000 (Fri, 28 Aug 2009) $
+ * $Date: 2011-08-08 02:41:14 -0500 (Mon, 08 Aug 2011) $
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -43,8 +43,7 @@
 
 #if OPAL_SIP
 
-#include <ptlib/safecoll.h>
-
+#include <opal/pres_ent.h>
 #include <opal/connection.h>
 #include <sip/sippdu.h>
 
@@ -58,18 +57,16 @@ class SIPHandler : public PSafeObject
 
 protected:
   SIPHandler(
-    SIPEndPoint & ep, 
-    const PString & target,
-    const PString & remote,
-    int expireTime = 0,
-    int offlineExpire = 30,
-    const PTimeInterval & retryMin = PMaxTimeInterval,
-    const PTimeInterval & retryMax = PMaxTimeInterval
+    SIP_PDU::Methods method,
+    SIPEndPoint & ep,
+    const SIPParameters & params
   );
 
 public:
   ~SIPHandler();
 
+  virtual Comparison Compare(const PObject & other) const;
+
   virtual bool ShutDown();
 
   enum State {
@@ -79,13 +76,14 @@ public:
     Refreshing,       // The registration is being refreshed
     Restoring,        // The registration is trying to be restored after being offline
     Unsubscribing,    // The unregistration is in process
-    Unsubscribed      // The registrating is inactive
+    Unsubscribed,     // The registrating is inactive
+    NumStates
   };
 
   void SetState (SIPHandler::State s);
 
   inline SIPHandler::State GetState () 
-  { return state; }
+  { return m_state; }
 
   virtual OpalTransport * GetTransport();
 
@@ -105,16 +103,14 @@ public:
   virtual PString GetCallID()
     { return callID; }
 
-  virtual void SetBody(const PString & b)
-    { body = b;}
+  virtual void SetBody(const PString & /*body*/) { }
 
   virtual bool IsDuplicateCSeq(unsigned ) { return false; }
 
   virtual SIPTransaction * CreateTransaction(OpalTransport & t) = 0;
 
-  virtual SIP_PDU::Methods GetMethod() = 0;
-  virtual SIPSubscribe::EventPackage GetEventPackage() const
-  { return PString::Empty(); }
+  SIP_PDU::Methods GetMethod() const { return m_method; }
+  virtual SIPSubscribe::EventPackage GetEventPackage() const { return SIPEventPackage(); }
 
   virtual void OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response);
@@ -122,9 +118,11 @@ public:
   virtual void OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnTransactionFailed(SIPTransaction & transaction);
+
+  virtual void OnFailed(const SIP_PDU & response);
   virtual void OnFailed(SIP_PDU::StatusCodes);
 
-  bool ActivateState(SIPHandler::State state, unsigned msecs = 1000);
+  bool ActivateState(SIPHandler::State state);
   virtual bool SendNotify(const PObject * /*body*/) { return false; }
 
   SIPEndPoint & GetEndPoint() const { return endpoint; }
@@ -135,12 +133,16 @@ public:
   const PString & GetPassword() const     { return m_password; }
   const PString & GetRealm() const        { return m_realm; }
   const SIPURL & GetRemoteAddress() const { return m_remoteAddress; }
+  const SIPURL & GetProxy() const         { return m_proxy; }
+
+  SIPMIMEInfo m_mime;
 
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
+  void RetryLater(unsigned after);
   PDECLARE_NOTIFIER(PTimer, SIPHandler, OnExpireTimeout);
   static PBoolean WriteSIPHandler(OpalTransport & transport, void * info);
-  bool WriteSIPHandler(OpalTransport & transport);
+  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
 
   SIPEndPoint               & endpoint;
 
@@ -149,22 +151,34 @@ protected:
   PString                     m_password;
   PString                     m_realm;
 
-  PSafeList<SIPTransaction>   transactions;
+  PSafeList<SIPTransaction>   m_transactions;
   OpalTransport             * m_transport;
+
+  SIP_PDU::Methods            m_method;
   SIPURL                      m_addressOfRecord;
   SIPURL                      m_remoteAddress;
   PString                     callID;
   int                         expire;
   int                         originalExpire;
   int                         offlineExpire;
-  PString                     body;
   unsigned                    authenticationAttempts;
-  State                       state;
+  State                       m_state;
+  queue<State>                m_stateQueue;
+  bool                        m_receivedResponse;
   PTimer                      expireTimer; 
   PTimeInterval               retryTimeoutMin; 
   PTimeInterval               retryTimeoutMax; 
   SIPURL                      m_proxy;
   OpalProductInfo             m_productInfo;
+
+  // Keep a copy of the keys used for easy removal on destruction
+  typedef std::map<PString, PSafePtr<SIPHandler> > IndexMap;
+  std::pair<IndexMap::iterator, bool> m_byCallID;
+  std::pair<IndexMap::iterator, bool> m_byAorAndPackage;
+  std::pair<IndexMap::iterator, bool> m_byAuthIdAndRealm;
+  std::pair<IndexMap::iterator, bool> m_byAorUserAndRealm;
+
+  friend class SIPHandlersList;
 };
 
 #if PTRACING
@@ -182,23 +196,26 @@ public:
     const SIPRegister::Params & params
   );
 
-  ~SIPRegisterHandler();
-
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
-  virtual SIP_PDU::Methods GetMethod()
-    { return SIP_PDU::Method_REGISTER; }
 
   virtual void OnFailed(SIP_PDU::StatusCodes r);
 
   void UpdateParameters(const SIPRegister::Params & params);
 
+  const SIPRegister::Params & GetParams() const { return m_parameters; }
+
+  const SIPURLList & GetContacts() const { return m_contactAddresses; }
+  const SIPURLList & GetServiceRoute() const { return m_serviceRoute; }
+
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
   void SendStatus(SIP_PDU::StatusCodes code, State state);
 
   SIPRegister::Params m_parameters;
   unsigned            m_sequenceNumber;
+  SIPURLList          m_contactAddresses;
+  SIPURLList          m_serviceRoute;
 };
 
 
@@ -212,9 +229,7 @@ public:
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual PBoolean OnReceivedNOTIFY(SIP_PDU & response);
-  virtual void OnFailed(SIP_PDU::StatusCodes r);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_SUBSCRIBE; }
+  virtual void OnFailed(const SIP_PDU & response);
   virtual SIPEventPackage GetEventPackage() const
     { return m_parameters.m_eventPackage; }
 
@@ -222,14 +237,20 @@ public:
 
   virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
 
+  const SIPSubscribe::Params & GetParams() const { return m_parameters; }
+
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
+  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
   void SendStatus(SIP_PDU::StatusCodes code, State state);
+  bool DispatchNOTIFY(SIP_PDU & request, SIP_PDU & response);
 
   SIPSubscribe::Params     m_parameters;
   SIPDialogContext         m_dialog;
   bool                     m_unconfirmed;
   SIPEventPackageHandler * m_packageHandler;
+
+  SIP_PDU                  * m_previousResponse;
 };
 
 
@@ -246,11 +267,11 @@ public:
   ~SIPNotifyHandler();
 
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_NOTIFY; }
   virtual SIPEventPackage GetEventPackage() const
     { return m_eventPackage; }
 
+  virtual void SetBody(const PString & body) { m_body = body; }
+
   virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
   virtual bool SendNotify(const PObject * body);
 
@@ -265,11 +286,13 @@ public:
 
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
+  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
 
   SIPEventPackage          m_eventPackage;
   SIPDialogContext         m_dialog;
   Reasons                  m_reason;
   SIPEventPackageHandler * m_packageHandler;
+  PString                  m_body;
 };
 
 
@@ -281,20 +304,17 @@ public:
   SIPPublishHandler(SIPEndPoint & ep, 
                     const SIPSubscribe::Params & params,
                     const PString & body);
-  ~SIPPublishHandler();
+
+  virtual void SetBody(const PString & body) { m_body = body; }
 
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
-  virtual SIP_PDU::Methods GetMethod()
-    { return SIP_PDU::Method_PUBLISH; }
-  virtual SIPEventPackage GetEventPackage() const
-    { return m_parameters.m_eventPackage; }
-  virtual void SetBody(const PString & body);
+  virtual SIPEventPackage GetEventPackage() const { return m_parameters.m_eventPackage; }
 
-private:
+protected:
   SIPSubscribe::Params m_parameters;
+  PString              m_body;
   PString              m_sipETag;
-  bool                 m_stateChanged;
 };
 
 
@@ -302,21 +322,32 @@ class SIPMessageHandler : public SIPHandler
 {
   PCLASSINFO(SIPMessageHandler, SIPHandler);
 public:
-  SIPMessageHandler(SIPEndPoint & ep, 
-                    const PString & to,
-                    const PString & body,
-                    const PString & remoteContact,
-                    const PString & callId);
-  ~SIPMessageHandler();
+  SIPMessageHandler(SIPEndPoint & ep, const SIPMessage::Params & params);
 
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_MESSAGE; }
-  virtual void OnFailed (SIP_PDU::StatusCodes);
-  virtual void SetBody(const PString & b);
+  virtual void OnFailed(SIP_PDU::StatusCodes);
+  virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
 
-private:
-  virtual void OnExpireTimeout(PTimer &, INT);
+  void UpdateParameters(const SIPMessage::Params & params);
+
+protected:
+  SIPMessage::Params m_parameters;
+};
+
+
+class SIPOptionsHandler : public SIPHandler
+{
+  PCLASSINFO(SIPOptionsHandler, SIPHandler);
+public:
+  SIPOptionsHandler(SIPEndPoint & ep, const SIPOptions::Params & params);
+
+  virtual SIPTransaction * CreateTransaction (OpalTransport &);
+  virtual void OnFailed(SIP_PDU::StatusCodes);
+  virtual void OnFailed(const SIP_PDU & response);
+  virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
+
+protected:
+  SIPOptions::Params m_parameters;
 };
 
 
@@ -324,11 +355,8 @@ class SIPPingHandler : public SIPHandler
 {
   PCLASSINFO(SIPPingHandler, SIPHandler);
 public:
-  SIPPingHandler(SIPEndPoint & ep, 
-                 const PString & to);
+  SIPPingHandler(SIPEndPoint & ep, const PURL & to);
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_MESSAGE; }
 };
 
 
@@ -340,15 +368,17 @@ class SIPHandlersList
   public:
     /** Append a new handler to the list
       */
-    void Append(SIPHandler * handler)
-      { m_handlersList.Append(handler); }
+    void Append(SIPHandler * handler);
 
     /** Remove a handler from the list.
         Handler is not immediately deleted but marked for deletion later by
         DeleteObjectsToBeRemoved() when all references are done with the handler.
       */
-    void Remove(SIPHandler * handler)
-      { m_handlersList.Remove(handler); }
+    void Remove(SIPHandler * handler);
+
+    /** Update indexes for handler in the list
+      */
+    void Update(SIPHandler * handler);
 
     /** Clean up lists of handler.
       */
@@ -388,8 +418,8 @@ class SIPHandlersList
      * or 6001@seconix.com when registering 6001@seconix.com to
      * sip.seconix.com
      */
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, PSafetyMode m);
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
 
     /**
      * Find the SIPHandler object with the specified registration host.
@@ -399,30 +429,59 @@ class SIPHandlersList
     PSafePtr <SIPHandler> FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode m);
 
   protected:
+    void RemoveIndexes(SIPHandler * handler);
+
+    PMutex m_extraMutex;
     PSafeList<SIPHandler> m_handlersList;
+
+    typedef SIPHandler::IndexMap IndexMap;
+    PSafePtr<SIPHandler> FindBy(IndexMap & by, const PString & key, PSafetyMode m);
+
+    IndexMap m_byCallID;
+    IndexMap m_byAorAndPackage;
+    IndexMap m_byAuthIdAndRealm;
+    IndexMap m_byAorUserAndRealm;
 };
 
 
 /** Information for SIP "presence" event package notification messages.
   */
-struct SIPPresenceInfo
+class SIPPresenceInfo : public OpalPresenceInfo
 {
-  enum BasicStates {
-    Unknown,
-    Open,
-    Closed
-  };
+public:
+  SIPPresenceInfo(
+    State state = Unchanged
+  );
 
-  SIPPresenceInfo() : m_basic(Unknown) { }
+  // basic presence defined by RFC 3863
+  PString m_tupleId;
+  PString m_contact;
 
-  PString     m_address;
-  PString     m_entity;
-  BasicStates m_basic;
-  PString     m_note;
-  PString     m_contact;
-  PString     m_to;
+  // presence extensions defined by RFC 4480
+  PStringArray m_activities;  // list of activities, seperated by newline
 
-  PString AsString() const;
+  // presence agent
+  PString m_presenceAgent;
+
+  PString AsXML() const;
+
+#if P_EXPAT
+  static bool ParseXML(
+    const PString & body,
+    list<SIPPresenceInfo> & info,
+    PString & error
+  );
+#endif
+
+  void PrintOn(ostream & strm) const;
+  friend ostream & operator<<(ostream & strm, const SIPPresenceInfo & info) { info.PrintOn(strm); return strm; }
+
+  static State FromSIPActivityString(const PString & str);
+
+  static bool AsSIPActivityString(State state, PString & str);
+  bool AsSIPActivityString(PString & str) const;
+
+  PString m_personId;
 };
 
 
