@@ -31,20 +31,20 @@
  *                 Derek Smithies (derek@indranet.co.nz)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 25877 $
+ * $Revision: 26277 $
  * $Author: rjongbloed $
- * $Date: 2011-05-24 02:58:44 -0500 (Tue, 24 May 2011) $
+ * $Date: 2011-08-05 05:10:16 -0500 (Fri, 05 Aug 2011) $
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
 #include "plugin-config.h"
 #endif
 
-#include <codec/opalplugin.hpp>
-
 #include "ffmpeg.h"
 #include "dyna.h"
 
+#include <codec/opalplugin.hpp>
+
 #include "shared/h264frame.h"
 
 #ifdef WIN32
@@ -749,7 +749,9 @@ class MyEncoder : public PluginCodec<MY_CODEC>
     unsigned m_constraints;
     unsigned m_packetisationMode;
     unsigned m_maxRTPSize;
+    unsigned m_maxNALUSize;
     unsigned m_tsto;
+    unsigned m_keyFramePeriod;
 
     H264EncCtx m_encoder;
 
@@ -765,7 +767,9 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       , m_constraints(0)
       , m_packetisationMode(1)
       , m_maxRTPSize(1400)
+      , m_maxNALUSize(1400)
       , m_tsto(31)
+      , m_keyFramePeriod(0)
     {
     }
 
@@ -806,12 +810,18 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_TARGET_BIT_RATE) == 0)
         return SetOptionUnsigned(m_bitRate, optionValue, 1000);
 
-      if (STRCMPI(optionName, PLUGINCODEC_OPTION_MAX_FRAME_SIZE) == 0)
-        return SetOptionUnsigned(m_maxRTPSize, optionValue, 65535);
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE) == 0)
+        return SetOptionUnsigned(m_maxRTPSize, optionValue, 256, 8192);
+
+      if (strcasecmp(optionName, MaxNaluSize.m_name) == 0)
+        return SetOptionUnsigned(m_maxNALUSize, optionValue, 256, 8192);
 
       if (STRCMPI(optionName, PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF) == 0)
         return SetOptionUnsigned(m_tsto, optionValue, 1, 31);
 
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD) == 0)
+        return SetOptionUnsigned(m_keyFramePeriod, optionValue, 0);
+
       if (strcasecmp(optionName, H241Level.m_name) == 0) {
         size_t i;
         for (i = 0; i < sizeof(LevelInfo)/sizeof(LevelInfo[0])-1; i++) {
@@ -882,8 +892,9 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       m_encoder.call(SET_FRAME_HEIGHT, m_height);
       m_encoder.call(SET_FRAME_RATE, m_frameRate);
       m_encoder.call(SET_TARGET_BITRATE, m_bitRate/1000);
-      m_encoder.call(SET_MAX_FRAME_SIZE, m_maxRTPSize);
+      m_encoder.call(SET_MAX_FRAME_SIZE, std::min(m_maxRTPSize, m_maxNALUSize));
       m_encoder.call(SET_TSTO, m_tsto);
+      m_encoder.call(SET_MAX_KEY_FRAME_PERIOD, m_keyFramePeriod);
       m_encoder.call(APPLY_OPTIONS);
       PTRACE(3, MY_CODEC_LOG, "Applied options: "
                               "prof=" << m_profile << " "
@@ -904,7 +915,7 @@ class MyEncoder : public PluginCodec<MY_CODEC>
                              unsigned & flags)
     {
       int ret = 0;
-      unsigned headerSize = PluginCodec_RTP_MinHeaderSize;
+      unsigned headerSize = PluginCodec_RTP_GetHeaderLength(toPtr);
       m_encoder.call(ENCODE_FRAMES,
                      (const u_char *)fromPtr, fromLen,
                      (u_char *)toPtr, toLen,
