@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26023 $
+ * $Revision: 24051 $
  * $Author: rjongbloed $
- * $Date: 2011-06-14 03:37:04 -0500 (Tue, 14 Jun 2011) $
+ * $Date: 2010-02-10 17:59:12 -0600 (Wed, 10 Feb 2010) $
  */
 
 #ifndef _OpenPhone_VERSION_H
@@ -42,7 +42,7 @@
 
 #define MAJOR_VERSION OPAL_MAJOR
 #define MINOR_VERSION OPAL_MINOR
-#define BUILD_TYPE    ReleaseCode
+#define BUILD_TYPE    BetaCode
 #define BUILD_NUMBER  OPAL_BUILD
 
 
