@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23525 $
- * $Author: csoutheren $
- * $Date: 2009-09-24 01:02:24 -0500 (Thu, 24 Sep 2009) $
+ * $Revision: 24813 $
+ * $Author: rjongbloed $
+ * $Date: 2010-10-21 03:18:03 -0500 (Thu, 21 Oct 2010) $
  */
 
 #include <ptlib.h>
@@ -34,6 +34,10 @@
 #include <iax2/iax2.h>
 #endif
 
+#if OPAL_CAPI
+#include <lids/capi_ep.h>
+#endif
+
 #if OPAL_SIP
 #include <sip/sip.h>
 #endif
@@ -164,7 +168,11 @@ void SimpleOpalProcess::Main()
              "x-vxml:"
 #endif
 #if OPAL_IAX2
-	     "X-no-iax2."
+             "X-no-iax2."
+	     "-iaxport:"
+#endif
+#if OPAL_CAPI
+             "-no-capi."
 #endif
           , PFalse);
 
@@ -292,6 +300,10 @@ void SimpleOpalProcess::Main()
 #endif
 #if OPAL_IAX2
             "  -X --no-iax2            : Remove support for iax2\n"
+            "     --iaxport n          : Set port to use (def. 4569)\n"
+#endif
+#if OPAL_CAPI
+            "     --no-capi            : Remove support for CAPI ISDN\n"
 #endif
             "  -h --help               : This help message.\n"
             "\n"
@@ -387,6 +399,9 @@ MyManager::MyManager()
 #if OPAL_IAX2
   iax2EP = NULL;
 #endif
+#if OPAL_CAPI
+  capiEP = NULL;
+#endif
 #if OPAL_IVR
   ivrEP  = NULL;
 #endif
@@ -508,13 +523,13 @@ PBoolean MyManager::Initialise(PArgList & args)
       cerr << "IP Type Of Service bits must be 0 to 255.\n";
       return PFalse;
     }
-    SetRtpIpTypeofService(tos);
+    SetMediaTypeOfService(tos);
   }
 
   cout << "TCP ports: " << GetTCPPortBase() << '-' << GetTCPPortMax() << "\n"
           "UDP ports: " << GetUDPPortBase() << '-' << GetUDPPortMax() << "\n"
           "RTP ports: " << GetRtpIpPortBase() << '-' << GetRtpIpPortMax() << "\n"
-          "RTP IP TOS: 0x" << hex << (unsigned)GetRtpIpTypeofService() << dec << "\n"
+          "RTP IP TOS: 0x" << hex << (unsigned)GetMediaTypeOfService() << dec << "\n"
           "STUN server: " << flush;
 
   if (args.HasOption("stun"))
@@ -607,11 +622,19 @@ PBoolean MyManager::Initialise(PArgList & args)
   // Create IAX2 protocol handler
 
   if (!args.HasOption("no-iax2")) {
-    iax2EP = new IAX2EndPoint(*this);
+    PINDEX port = 0;
+    if (args.HasOption("iaxport"))
+      port = args.GetOptionString("iaxport").AsInteger();
+    cerr << " port is " << port << endl;
+
+    if (port > 0)
+      iax2EP = new IAX2EndPoint(*this, port);
+    else
+      iax2EP = new IAX2EndPoint(*this);
     
     if (!iax2EP->InitialisedOK()) {
       cerr << "IAX2 Endpoint is not initialised correctly" << endl;
-      cerr << "Is there another application using port 4569? " << endl;
+      cerr << "Is there another application using the iax port? " << endl;
       return PFalse;
     }
 
@@ -625,6 +648,18 @@ PBoolean MyManager::Initialise(PArgList & args)
   }
 #endif
 
+#if OPAL_CAPI
+  ///////////////////////////////////////
+  // Create CAPI handler
+
+  if (!args.HasOption("no-capi")) {
+    capiEP = new OpalCapiEndPoint(*this);
+    
+    if (!capiEP->OpenControllers())
+      cerr << "CAPI Endpoint failed to initialise any controllers." << endl;
+  }
+#endif
+
 #if OPAL_SIP
 
   ///////////////////////////////////////
@@ -638,7 +673,7 @@ PBoolean MyManager::Initialise(PArgList & args)
 
     PString str = args.GetOptionString("sip-ui");
     if (str *= "rfc2833")
-      sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsSeparateRFC2833);
+      sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsRFC2833);
     else if (str *= "info-tone")
       sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsTone);
     else if (str *= "info-string")
@@ -656,8 +691,6 @@ PBoolean MyManager::Initialise(PArgList & args)
       sipEP->SetDefaultLocalPartyName(aliases[0]);
     }
 
-    sipEP->SetRetryTimeouts(10000, 30000);
-
     // Start the listener thread for incoming calls.
     PStringArray listeners = args.GetOptionString("sip-listen").Lines();
     if (!sipEP->StartListeners(listeners)) {
@@ -791,11 +824,18 @@ PBoolean MyManager::Initialise(PArgList & args)
                                                                                                                                             
 #if OPAL_IAX2
   if (pcssEP != NULL) {
-    AddRouteEntry("iax2:.*  = pc:");
+    AddRouteEntry("iax2:.* = pc:");
     AddRouteEntry("pc:.*   = iax2:<da>");
   }
 #endif
 
+#if OPAL_CAPI
+  if (capiEP != NULL) {
+    AddRouteEntry("isdn:.* = pc:");
+    AddRouteEntry("pc:.*   = isdn:<da>");
+  }
+#endif
+
 #if OPAL_FAX
   if (faxEP != NULL) {
     AddRouteEntry("sip:.*  = t38:<da>");
@@ -1139,6 +1179,13 @@ void MyManager::Main(PArgList & args)
           cout << "Could not reject connection " << pcssEP->incomingConnectionToken << endl;
         break;
 
+      case 'b' :
+        if ( pcssEP != NULL &&
+            !pcssEP->incomingConnectionToken &&
+            !pcssEP->RejectIncomingConnection(pcssEP->incomingConnectionToken, OpalConnection::EndedByLocalBusy))
+          cout << "Could not reject connection " << pcssEP->incomingConnectionToken << endl;
+        break;
+
 #if OPAL_PTLIB_CONFIG_FILE
       case 'l' :
         ListSpeedDials();
@@ -1498,7 +1545,7 @@ PBoolean MyPCSSEndPoint::OnShowIncoming(const OpalPCSSConnection & connection)
     PTime now;
     cout << "\nCall on " << now.AsString("w h:mma")
          << " from " << connection.GetRemotePartyName()
-         << ", answer (Y/N)? " << flush;
+         << ", answer (Yes/No/Busy)? " << flush;
   }
 
   return PTrue;
