@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22640 $
+ * $Revision: 25091 $
  * $Author: rjongbloed $
- * $Date: 2009-05-18 22:19:20 -0500 (Mon, 18 May 2009) $
+ * $Date: 2011-01-22 12:19:24 -0600 (Sat, 22 Jan 2011) $
  */
 
 #include <ptlib.h>
@@ -78,8 +78,6 @@ H245NegMasterSlaveDetermination::H245NegMasterSlaveDetermination(H323EndPoint &
 
 PBoolean H245NegMasterSlaveDetermination::Start(PBoolean renegotiate)
 {
-  PWaitAndSignal wait(mutex);
-
   if (state != e_Idle) {
     PTRACE(3, "H245\tMasterSlaveDetermination already in progress");
     return PTrue;
@@ -110,26 +108,22 @@ PBoolean H245NegMasterSlaveDetermination::Restart()
 
 void H245NegMasterSlaveDetermination::Stop()
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStopping MasterSlaveDetermination: state=" << state);
 
   if (state == e_Idle)
     return;
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Idle;
 }
 
 
 PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveDetermination & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDetermination: state=" << state);
 
   if (state == e_Incoming) {
-    replyTimer.Stop();
+    replyTimer.Stop(false);
     state = e_Idle;
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Duplicate MasterSlaveDetermination");
@@ -167,7 +161,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
     if (retryCount < endpoint.GetMasterSlaveDeterminationRetries())
       return Restart(); // Try again
 
-    replyTimer.Stop();
+    replyTimer.Stop(false);
     state = e_Idle;
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Retries exceeded");
@@ -182,8 +176,6 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
 
 PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterminationAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationAck: state=" << state);
 
   if (state == e_Idle)
@@ -207,7 +199,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
       return PFalse;
   }
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Idle;
 
   if (status != newStatus)
@@ -220,8 +212,6 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
 
 PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDeterminationReject & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationReject: state=" << state);
 
   switch (state) {
@@ -239,7 +229,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDet
       break;
   }
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Idle;
 
   return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
@@ -249,14 +239,12 @@ PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDet
 
 PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDeterminationRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationRelease: state=" << state);
 
   if (state == e_Idle)
     return PTrue;
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Idle;
 
   return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
@@ -266,7 +254,8 @@ PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDe
 
 void H245NegMasterSlaveDetermination::HandleTimeout(PTimer &, INT)
 {
-  PWaitAndSignal wait(mutex);
+  if (state == e_Idle)
+    return;
 
   PTRACE(3, "H245\tTimeout on MasterSlaveDetermination: state=" << state);
 
@@ -318,8 +307,6 @@ H245NegTerminalCapabilitySet::H245NegTerminalCapabilitySet(H323EndPoint & end,
 
 PBoolean H245NegTerminalCapabilitySet::Start(PBoolean renegotiate, PBoolean empty)
 {
-  PWaitAndSignal wait(mutex);
-
   if (state == e_InProgress) {
     PTRACE(2, "H245\tTerminalCapabilitySet already in progress: outSeq=" << outSequenceNumber);
     return PTrue;
@@ -345,14 +332,12 @@ PBoolean H245NegTerminalCapabilitySet::Start(PBoolean renegotiate, PBoolean empt
 
 void H245NegTerminalCapabilitySet::Stop(PBoolean dec)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStopping TerminalCapabilitySet: state=" << state);
 
   if (state == e_Idle)
     return;
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Idle;
   receivedCapabilites = PFalse;
 
@@ -367,23 +352,18 @@ void H245NegTerminalCapabilitySet::Stop(PBoolean dec)
 
 PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabilitySet & pdu)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySet:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
                      " inSeq=" << inSequenceNumber);
 
   if (pdu.m_sequenceNumber == inSequenceNumber) {
-    mutex.Signal();
     PTRACE(2, "H245\tIgnoring TerminalCapabilitySet, already received sequence number");
     return PTrue;  // Already had this one
   }
 
   inSequenceNumber = pdu.m_sequenceNumber;
 
-  mutex.Signal();
-
   H323Capabilities remoteCapabilities(connection, pdu);
 
   const H245_MultiplexCapability * muxCap = NULL;
@@ -408,8 +388,6 @@ PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabil
 
 PBoolean H245NegTerminalCapabilitySet::HandleAck(const H245_TerminalCapabilitySetAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySetAck:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
@@ -421,7 +399,7 @@ PBoolean H245NegTerminalCapabilitySet::HandleAck(const H245_TerminalCapabilitySe
   if (pdu.m_sequenceNumber != outSequenceNumber)
     return PTrue;
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   state = e_Confirmed;
   PTRACE(3, "H245\tTerminalCapabilitySet Sent.");
   return PTrue;
@@ -430,8 +408,6 @@ PBoolean H245NegTerminalCapabilitySet::HandleAck(const H245_TerminalCapabilitySe
 
 PBoolean H245NegTerminalCapabilitySet::HandleReject(const H245_TerminalCapabilitySetReject & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySetReject:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
@@ -444,7 +420,7 @@ PBoolean H245NegTerminalCapabilitySet::HandleReject(const H245_TerminalCapabilit
     return PTrue;
 
   state = e_Idle;
-  replyTimer.Stop();
+  replyTimer.Stop(false);
   return connection.OnControlProtocolError(H323Connection::e_CapabilityExchange,
                                            "Rejected");
 }
@@ -452,8 +428,6 @@ PBoolean H245NegTerminalCapabilitySet::HandleReject(const H245_TerminalCapabilit
 
 PBoolean H245NegTerminalCapabilitySet::HandleRelease(const H245_TerminalCapabilitySetRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilityRelease: state=" << state);
 
   receivedCapabilites = PFalse;
@@ -464,7 +438,8 @@ PBoolean H245NegTerminalCapabilitySet::HandleRelease(const H245_TerminalCapabili
 
 void H245NegTerminalCapabilitySet::HandleTimeout(PTimer &, INT)
 {
-  PWaitAndSignal wait(mutex);
+  if (state == e_Idle)
+    return;
 
   PTRACE(3, "H245\tTimeout on TerminalCapabilitySet: state=" << state);
 
@@ -516,24 +491,13 @@ H245NegLogicalChannel::~H245NegLogicalChannel()
   replyTimer.Stop();
   PThread::Yield(); // Do this to avoid possible race condition with timer
 
-  mutex.Wait();
   delete channel;
-  mutex.Signal();
 }
 
 
 PBoolean H245NegLogicalChannel::Open(const H323Capability & capability,
                                  unsigned sessionID,
                                  unsigned replacementFor)
-{
-  PWaitAndSignal wait(mutex);
-  return OpenWhileLocked(capability, sessionID, replacementFor);
-}
-
-
-PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capability,
-                                            unsigned sessionID,
-                                            unsigned replacementFor)
 {
   if (state != e_Released && state != e_AwaitingRelease) {
     PTRACE(2, "H245\tOpen of channel currently in negotiations: " << channelNumber);
@@ -573,15 +537,6 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
            << ", channel->OnSendingPDU() failed");
     return PFalse;
   }
-  
-  // ensure the correct (external) session ID is used
-  if (open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
-    H245_H2250LogicalChannelParameters & param = open.m_reverseLogicalChannelParameters.m_multiplexParameters;
-    param.m_sessionID = connection.GetExternalSessionID(sessionID, capability.GetMediaFormat().GetMediaType());
-  } else {
-    H245_H2250LogicalChannelParameters & param = open.m_forwardLogicalChannelParameters.m_multiplexParameters;
-    param.m_sessionID = connection.GetExternalSessionID(sessionID, capability.GetMediaFormat().GetMediaType());
-  }
 
   if (replacementFor > 0) {
     if (open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
@@ -609,13 +564,6 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
 
 
 PBoolean H245NegLogicalChannel::Close()
-{
-  PWaitAndSignal wait(mutex);
-  return CloseWhileLocked();
-}
-
-
-PBoolean H245NegLogicalChannel::CloseWhileLocked()
 {
   PTRACE(3, "H245\tClosing channel: " << channelNumber << ", state=" << state);
 
@@ -690,9 +638,7 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
     }
   }
 
-  if (ok)
-    mutex.Signal();
-  else {
+  if (!ok) {
     reply.BuildOpenLogicalChannelReject(channelNumber, cause);
     Release();
   }
@@ -703,8 +649,6 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
 
 PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived open channel ack: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -713,11 +657,11 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
                                                "Ack unknown channel");
     case e_AwaitingEstablishment :
       state = e_Established;
-      replyTimer.Stop();
+      replyTimer.Stop(false);
 
       if (!channel->OnReceivedAckPDU(pdu)) {
         if (connection.GetRemoteProductInfo().name != "Cisco IOS")
-          return CloseWhileLocked();
+          return Close();
         PTRACE(4, "H245\tWorkaround for Cisco bug, cannot close channel on illegal ack or it hangs up on you.");
         return true;
       }
@@ -732,7 +676,7 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
       // Channel was already opened when OLC sent, if have error here it is
       // somthing other than an asymmetric codec conflict, so close it down.
       if (!channel->Start())
-        return CloseWhileLocked();
+        return Close();
 
     default :
       break;
@@ -744,8 +688,6 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
 
 PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelConfirm & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived open channel confirm: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -756,12 +698,12 @@ PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelC
       return connection.OnControlProtocolError(H323Connection::e_LogicalChannel,
                                                "Confirm established channel");
     case e_AwaitingConfirmation :
-      replyTimer.Stop();
+      replyTimer.Stop(false);
       state = e_Established;
       // Channel was already opened when OLC sent, if have error here it is
       // somthing other than an asymmetric codec conflict, so close it down.
       if (!channel->Start())
-        return CloseWhileLocked();
+        return Close();
 
     default :
       break;
@@ -773,13 +715,10 @@ PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelC
 
 PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject & pdu)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived open channel reject: " << channelNumber << ", state=" << state);
 
   switch (state) {
     case e_Released :
-      mutex.Signal();
       return connection.OnControlProtocolError(H323Connection::e_LogicalChannel,
                                                "Reject unknown channel");
     case e_Established :
@@ -797,7 +736,6 @@ PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject
       break;
 
     default :
-      mutex.Signal();
       break;
   }
 
@@ -807,8 +745,6 @@ PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject
 
 PBoolean H245NegLogicalChannel::HandleClose(const H245_CloseLogicalChannel & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived close channel: " << channelNumber << ", state=" << state);
 
   //if (pdu.m_source.GetTag() == H245_CloseLogicalChannel_source::e_user)
@@ -824,8 +760,6 @@ PBoolean H245NegLogicalChannel::HandleClose(const H245_CloseLogicalChannel & /*p
 
 PBoolean H245NegLogicalChannel::HandleCloseAck(const H245_CloseLogicalChannelAck & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived close channel ack: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -838,7 +772,6 @@ PBoolean H245NegLogicalChannel::HandleCloseAck(const H245_CloseLogicalChannelAck
       break;
 
     default :
-      mutex.Signal();
       break;
   }
 
@@ -848,8 +781,6 @@ PBoolean H245NegLogicalChannel::HandleCloseAck(const H245_CloseLogicalChannelAck
 
 PBoolean H245NegLogicalChannel::HandleRequestClose(const H245_RequestChannelClose & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close channel: " << channelNumber << ", state=" << state);
 
   if (state != e_Established)
@@ -882,14 +813,10 @@ PBoolean H245NegLogicalChannel::HandleRequestClose(const H245_RequestChannelClos
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseAck(const H245_RequestChannelCloseAck & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived request close ack channel: " << channelNumber << ", state=" << state);
 
   if (state == e_AwaitingResponse)
     Release();  // Other end says close OK, so do so.
-  else
-    mutex.Signal();
 
   return PTrue;
 }
@@ -897,8 +824,6 @@ PBoolean H245NegLogicalChannel::HandleRequestCloseAck(const H245_RequestChannelC
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseReject(const H245_RequestChannelCloseReject & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close reject channel: " << channelNumber << ", state=" << state);
 
   // Other end refused close, so go back to still having channel open
@@ -911,8 +836,6 @@ PBoolean H245NegLogicalChannel::HandleRequestCloseReject(const H245_RequestChann
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseRelease(const H245_RequestChannelCloseRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close release channel: " << channelNumber << ", state=" << state);
 
   // Other end refused close, so go back to still having channel open
@@ -924,8 +847,6 @@ PBoolean H245NegLogicalChannel::HandleRequestCloseRelease(const H245_RequestChan
 
 void H245NegLogicalChannel::HandleTimeout(PTimer &, INT)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tTimeout on open channel: " << channelNumber << ", state=" << state);
 
   H323ControlPDU reply;
@@ -941,7 +862,6 @@ void H245NegLogicalChannel::HandleTimeout(PTimer &, INT)
       break;
 
     case e_Released :
-      mutex.Signal();
       return;
 
     default :
@@ -958,9 +878,8 @@ void H245NegLogicalChannel::Release()
   state = e_Released;
   H323Channel * chan = channel;
   channel = NULL;
-  mutex.Signal();
 
-  replyTimer.Stop();
+  replyTimer.Stop(false);
 
   if (chan != NULL) {
     chan->Close();
@@ -1003,9 +922,7 @@ H245NegLogicalChannels::H245NegLogicalChannels(H323EndPoint & end,
 
 void H245NegLogicalChannels::Add(H323Channel & channel)
 {
-  mutex.Wait();
   channels.SetAt(channel.GetNumber(), new H245NegLogicalChannel(endpoint, connection, channel));
-  mutex.Signal();
 }
 
 
@@ -1013,15 +930,11 @@ PBoolean H245NegLogicalChannels::Open(const H323Capability & capability,
                                   unsigned sessionID,
                                   unsigned replacementFor)
 {
-  mutex.Wait();
-
   lastChannelNumber++;
 
   H245NegLogicalChannel * negChan = new H245NegLogicalChannel(endpoint, connection, lastChannelNumber);
   channels.SetAt(lastChannelNumber, negChan);
 
-  mutex.Signal();
-
   return negChan->Open(capability, sessionID, replacementFor);
 }
 
@@ -1042,8 +955,6 @@ PBoolean H245NegLogicalChannels::HandleOpen(const H245_OpenLogicalChannel & pdu)
   H323ChannelNumber chanNum(pdu.m_forwardLogicalChannelNumber, PTrue);
   H245NegLogicalChannel * chan;
 
-  mutex.Wait();
-
   if (channels.Contains(chanNum))
     chan = &channels[chanNum];
   else {
@@ -1051,10 +962,6 @@ PBoolean H245NegLogicalChannels::HandleOpen(const H245_OpenLogicalChannel & pdu)
     channels.SetAt(chanNum, chan);
   }
 
-  chan->mutex.Wait();
-
-  mutex.Signal();
-
   return chan->HandleOpen(pdu);
 }
 
@@ -1160,7 +1067,6 @@ PBoolean H245NegLogicalChannels::HandleRequestCloseRelease(const H245_RequestCha
 
 H323ChannelNumber H245NegLogicalChannels::GetNextChannelNumber()
 {
-  PWaitAndSignal wait(mutex);
   lastChannelNumber++;
   return lastChannelNumber;
 }
@@ -1168,18 +1074,13 @@ H323ChannelNumber H245NegLogicalChannels::GetNextChannelNumber()
 
 H323Channel * H245NegLogicalChannels::GetChannelAt(PINDEX i)
 {
-  mutex.Wait();
-  H323Channel * chan =  channels.GetDataAt(i).GetChannel();
-  mutex.Signal();
-  return chan;
+  return channels.GetDataAt(i).GetChannel();
 }
 
 
 H323Channel * H245NegLogicalChannels::FindChannel(unsigned channelNumber,
                                                   PBoolean fromRemote)
 {
-  PWaitAndSignal wait(mutex);
-
   H323ChannelNumber chanNum(channelNumber, fromRemote);
 
   if (channels.Contains(chanNum))
@@ -1191,7 +1092,6 @@ H323Channel * H245NegLogicalChannels::FindChannel(unsigned channelNumber,
 
 H245NegLogicalChannel & H245NegLogicalChannels::GetNegLogicalChannelAt(PINDEX i)
 {
-  PWaitAndSignal wait(mutex);
   return channels.GetDataAt(i);
 }
 
@@ -1200,20 +1100,13 @@ H245NegLogicalChannel * H245NegLogicalChannels::FindNegLogicalChannel(unsigned c
                                                                       PBoolean fromRemote)
 {
   H323ChannelNumber chanNum(channelNumber, fromRemote);
-
-  mutex.Wait();
-  H245NegLogicalChannel * channel = channels.GetAt(chanNum);
-  mutex.Signal();
-
-  return channel;
+  return channels.GetAt(chanNum);
 }
 
 
 H323Channel * H245NegLogicalChannels::FindChannelBySession(unsigned rtpSessionId,
                                                            PBoolean fromRemote)
 {
-  PWaitAndSignal wait(mutex);
-
   PINDEX i;
   H323Channel::Directions desiredDirection = fromRemote ? H323Channel::IsReceiver : H323Channel::IsTransmitter;
   for (i = 0; i < GetSize(); i++) {
@@ -1232,15 +1125,11 @@ H323Channel * H245NegLogicalChannels::FindChannelBySession(unsigned rtpSessionId
 
 void H245NegLogicalChannels::RemoveAll()
 {
-  PWaitAndSignal wait(mutex);
-
   for (PINDEX i = 0; i < channels.GetSize(); i++) {
     H245NegLogicalChannel & neg = channels.GetDataAt(i);
-    neg.mutex.Wait();
     H323Channel * channel = neg.GetChannel();
     if (channel != NULL)
       channel->Close();
-    neg.mutex.Signal();
   }
 
   channels.RemoveAll();
@@ -1361,7 +1250,7 @@ PBoolean H245NegRequestMode::HandleAck(const H245_RequestModeAck & pdu)
 
   if (awaitingResponse && pdu.m_sequenceNumber == outSequenceNumber) {
     awaitingResponse = PFalse;
-    replyTimer.Stop();
+    replyTimer.Stop(false);
     connection.OnAcceptModeChange(pdu);
   }
 
@@ -1375,7 +1264,7 @@ PBoolean H245NegRequestMode::HandleReject(const H245_RequestModeReject & pdu)
 
   if (awaitingResponse && pdu.m_sequenceNumber == outSequenceNumber) {
     awaitingResponse = PFalse;
-    replyTimer.Stop();
+    replyTimer.Stop(false);
     connection.OnRefusedModeChange(&pdu);
   }
 
@@ -1401,9 +1290,8 @@ void H245NegRequestMode::HandleTimeout(PTimer &, INT)
     pdu.Build(H245_IndicationMessage::e_requestModeRelease);
     connection.WriteControlPDU(pdu);
     connection.OnRefusedModeChange(NULL);
+    connection.OnControlProtocolError(H323Connection::e_ModeRequest, "Timeout");
   }
-
-  connection.OnControlProtocolError(H323Connection::e_ModeRequest, "Timeout");
 }
 
 
@@ -1423,8 +1311,6 @@ H245NegRoundTripDelay::H245NegRoundTripDelay(H323EndPoint & end, H323Connection
 
 PBoolean H245NegRoundTripDelay::StartRequest()
 {
-  PWaitAndSignal wait(mutex);
-
   replyTimer = endpoint.GetRoundTripDelayTimeout();
   sequenceNumber = (sequenceNumber + 1)%256;
   awaitingResponse = PTrue;
@@ -1444,8 +1330,6 @@ PBoolean H245NegRoundTripDelay::StartRequest()
 
 PBoolean H245NegRoundTripDelay::HandleRequest(const H245_RoundTripDelayRequest & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStarted round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
@@ -1457,15 +1341,13 @@ PBoolean H245NegRoundTripDelay::HandleRequest(const H245_RoundTripDelayRequest &
 
 PBoolean H245NegRoundTripDelay::HandleResponse(const H245_RoundTripDelayResponse & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTimeInterval tripEndTime = PTimer::Tick();
 
   PTRACE(3, "H245\tHandling round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
   if (awaitingResponse && pdu.m_sequenceNumber == sequenceNumber) {
-    replyTimer.Stop();
+    replyTimer.Stop(false);
     awaitingResponse = PFalse;
     roundTripTime = tripEndTime - tripStartTime;
     retryCount = 3;
@@ -1477,8 +1359,6 @@ PBoolean H245NegRoundTripDelay::HandleResponse(const H245_RoundTripDelayResponse
 
 void H245NegRoundTripDelay::HandleTimeout(PTimer &, INT)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tTimeout on round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
