@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23491 $
+ * $Revision: 24880 $
  * $Author: rjongbloed $
- * $Date: 2009-09-17 21:17:05 -0500 (Thu, 17 Sep 2009) $
+ * $Date: 2010-11-18 01:23:46 -0600 (Thu, 18 Nov 2010) $
  */
 
 #include <ptlib.h>
@@ -141,12 +141,6 @@ void H323SetAliasAddresses(const PStringList & names,
 }
 
 
-static PBoolean IsE164(const PString & str)
-{
-  return !str && str.FindSpan("1234567890*#") == P_MAX_INDEX;
-}
-
-
 void H323SetAliasAddress(const H323TransportAddress & address, H225_AliasAddress & alias)
 {
   alias.SetTag(H225_AliasAddress::e_transportID);
@@ -185,7 +179,7 @@ void H323SetAliasAddress(const PString & _name, H225_AliasAddress & alias, int t
   
   // otherwise guess it from the string: if all digits then assume an e164 address.
   if (tag < 0)
-    tag = IsE164(name) ? H225_AliasAddress::e_dialedDigits : H225_AliasAddress::e_h323_ID;
+    tag = OpalIsE164(name, true) ? H225_AliasAddress::e_dialedDigits : H225_AliasAddress::e_h323_ID;
 
   alias.SetTag(tag);
   switch (alias.GetTag()) {
@@ -304,7 +298,7 @@ PString H323GetAliasAddressString(const H225_AliasAddress & alias)
 PString H323GetAliasAddressE164(const H225_AliasAddress & alias)
 {
   PString str = H323GetAliasAddressString(alias);
-  if (IsE164(str))
+  if (OpalIsE164(str))
     return str;
 
   return PString();
@@ -466,15 +460,183 @@ PString H323GetRTPPacketization(const H245_RTPPayloadType & rtpPacketization)
 }
 
 
-bool H323GetRTPPacketization(OpalMediaFormat & mediaFormat, const H245_RTPPayloadType & rtpPacketization)
+PString H323GetCapabilityIdentifier(const H245_CapabilityIdentifier & capId)
 {
-  PString mediaPacketization = H323GetRTPPacketization(rtpPacketization);
-  if (mediaPacketization.IsEmpty())
+  switch (capId.GetTag()) {
+    case H245_CapabilityIdentifier::e_standard :
+      return ((const PASN_ObjectId &)capId).AsString();
+
+    case H245_CapabilityIdentifier::e_h221NonStandard :
+    {
+      PString str;
+      const H245_NonStandardParameter & nonStd = capId;
+      if (nonStd.m_nonStandardIdentifier.GetTag() == H245_NonStandardIdentifier::e_object)
+        str = ((const PASN_ObjectId &)nonStd.m_nonStandardIdentifier).AsString();
+      else {
+        const H245_NonStandardIdentifier_h221NonStandard & h221 = nonStd.m_nonStandardIdentifier;
+        str.sprintf("c=%u,cx=%u,o=%u",
+                    h221.m_t35CountryCode.GetValue(),
+                    h221.m_t35Extension.GetValue(),
+                    h221.m_manufacturerCode.GetValue());
+      }
+      if (nonStd.m_data.GetSize() > 0)
+        str += ':' + nonStd.m_data.AsString();
+      return str;
+    }
+  }
+
+  return PString::Empty();
+}
+
+
+static int ExtractVar(const PString & str, const PString & var)
+{
+  PRegularExpression regex("(^|[ \t\n,]+)" + var + "[ \t\n]*=[ \t\n]*[0-9]",
+                           PRegularExpression::IgnoreCase|PRegularExpression::Extended);
+
+  PINDEX pos, len;
+  if (!str.FindRegEx(regex, pos, len))
+    return -1;
+
+  return str.Mid(pos+len-1).AsUnsigned();
+}
+
+
+bool H323SetCapabilityIdentifier(const PString & str, H245_CapabilityIdentifier & capId)
+{
+  PASN_ObjectId oid;
+  oid.SetValue(str);
+  if (oid.AsString() == str) {
+    capId.SetTag(H245_CapabilityIdentifier::e_standard);
+    ((PASN_ObjectId &)capId) = oid;
+    return true;
+  }
+
+  PINDEX colon = str.Find(':');
+  if (colon == 0)
     return false;
-  
-  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), mediaPacketization);
-  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(), mediaPacketization.Left(mediaPacketization.Find(',')));
-  return true;
+
+  if (colon != P_MAX_INDEX && oid.AsString() == str.Left(colon)) {
+    capId.SetTag(H245_CapabilityIdentifier::e_h221NonStandard);
+    H245_NonStandardParameter & nonStd = capId;
+    nonStd.m_nonStandardIdentifier.SetTag(H245_NonStandardIdentifier::e_object);
+    ((PASN_ObjectId &)nonStd.m_nonStandardIdentifier) = oid;
+    nonStd.m_data = str.Mid(colon+1);
+    return true;
+  }
+
+  int country = ExtractVar(str, 'c');
+  int manufacturer = ExtractVar(str, 'o');
+  if (country >= 0 && manufacturer >= 0) {
+    capId.SetTag(H245_CapabilityIdentifier::e_h221NonStandard);
+    H245_NonStandardParameter & nonStd = capId;
+    nonStd.m_nonStandardIdentifier.SetTag(H245_NonStandardIdentifier::e_h221NonStandard);
+    H245_NonStandardIdentifier_h221NonStandard & h221 = nonStd.m_nonStandardIdentifier;
+
+    h221.m_t35CountryCode = country;
+    h221.m_manufacturerCode = manufacturer;
+
+    int extension = ExtractVar(str, "cx");
+    if (extension >= 0)
+      h221.m_t35Extension = extension;
+
+    if (colon != P_MAX_INDEX)
+      nonStd.m_data = str.Mid(colon+1);
+    return true;
+  }
+
+  return false;
+}
+
+
+const H245_ParameterValue * H323GetGenericParameter(const H245_ArrayOf_GenericParameter & params, unsigned ordinal)
+{
+  for (PINDEX i = 0; i < params.GetSize(); i++) {
+    const H245_GenericParameter & param = params[i];
+    if (param.m_parameterIdentifier.GetTag() == H245_ParameterIdentifier::e_standard &&
+                         ((const PASN_Integer &)param.m_parameterIdentifier) == ordinal)
+      return &param.m_parameterValue;
+  }
+
+  return NULL;
+}
+
+
+bool H323GetGenericParameterBoolean(const H245_ArrayOf_GenericParameter & params, unsigned ordinal)
+{
+  const H245_ParameterValue * param = H323GetGenericParameter(params, ordinal);
+  return param != NULL && param->GetTag() == H245_ParameterValue::e_logical;
+}
+
+
+unsigned H323GetGenericParameterInteger(const H245_ArrayOf_GenericParameter & params,
+                                        unsigned ordinal,
+                                        unsigned defValue,
+                                        H245_ParameterValue::Choices subType)
+{
+  const H245_ParameterValue * param = H323GetGenericParameter(params, ordinal);
+  if (param == NULL || param->GetTag() != (unsigned)subType)
+    return defValue;
+  return (const PASN_Integer &)*param;
+}
+
+
+H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool reorder)
+{
+  PINDEX size = params.GetSize();
+  params.SetSize(size+1);
+
+  PINDEX pos = size;
+  if (reorder) {
+    for (pos = size; pos > 0; pos--) {
+      const H245_GenericParameter & param = params[pos-1];
+      if (param.m_parameterIdentifier.GetTag() == H245_ParameterIdentifier::e_standard &&
+                            ((const PASN_Integer &)param.m_parameterIdentifier) < ordinal)
+        break;
+      params[pos] = param;
+    }
+  }
+
+  H245_GenericParameter & param = params[pos];
+  param.m_parameterIdentifier.SetTag(H245_ParameterIdentifier::e_standard);
+  (PASN_Integer &)param.m_parameterIdentifier = ordinal;
+  return &param.m_parameterValue;
+}
+
+
+void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool value, bool reorder)
+{
+  // Do not include a logical at all if it is false
+  if (value)
+    H323AddGenericParameter(params, ordinal, reorder)->SetTag(H245_ParameterValue::e_logical);
+}
+
+
+void H323AddGenericParameterInteger(H245_ArrayOf_GenericParameter & params,
+                                    unsigned ordinal,
+                                    unsigned value,
+                                    H245_ParameterValue::Choices subType,
+                                    bool reorder)
+{
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  param->SetTag(subType);
+  (PASN_Integer &)*param = value;
+}
+
+
+void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value, bool reorder)
+{
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  param->SetTag(H245_ParameterValue::e_octetString);
+  (PASN_OctetString &)*param = value;
+}
+
+
+void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value, bool reorder)
+{
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  param->SetTag(H245_ParameterValue::e_octetString);
+  (PASN_OctetString &)*param = value;
 }
 
 
@@ -605,6 +767,10 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
       q931pdu.SetCalledPartyNumber(destAlias);
   }
 
+  PString redir(connection.GetStringOptions()(OPAL_OPT_REDIRECTING_PARTY, connection.GetRedirectingParty()));
+  if (!redir.IsEmpty())
+    q931pdu.SetRedirectingNumber(redir);
+
   endpoint.SetEndpointTypeInfo(setup.m_sourceInfo);
   
 #if OPAL_H460
@@ -729,12 +895,11 @@ H225_Information_UUIE & H323SignalPDU::BuildInformation(const H323Connection & c
 }
 
 
-H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cause,
-                                                           unsigned reason)
+static OpalConnection::CallEndReason H323TranslateToCallEndReasonCode(Q931::CauseValues cause, unsigned reasonTag)
 {
   switch (cause) {
     case Q931::ErrorInCauseIE :
-      switch (reason) {
+      switch (reasonTag) {
         case H225_ReleaseCompleteReason::e_noBandwidth :
           return H323Connection::EndedByNoBandwidth;
 
@@ -775,6 +940,9 @@ H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cau
     case Q931::ResourceUnavailable :
       return H323Connection::EndedByRemoteCongestion;
 
+    case Q931::CallRejected :
+      return H323Connection::EndedByRefusal;
+
     case Q931::NoResponse :
     case Q931::NoAnswer :
       return H323Connection::EndedByNoAnswer;
@@ -802,6 +970,11 @@ H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cau
   }
 }
 
+H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cause, unsigned reasonTag)
+{
+  return OpalConnection::CallEndReason(H323TranslateToCallEndReasonCode(cause, reasonTag), cause);
+}
+
 
 Q931::CauseValues H323TranslateFromCallEndReason(H323Connection::CallEndReason callEndReason,
                                                  H225_ReleaseCompleteReason & releaseCompleteReason)
@@ -840,9 +1013,10 @@ Q931::CauseValues H323TranslateFromCallEndReason(H323Connection::CallEndReason c
     Q931::ExchangeRoutingError,                             /// EndedByGkAdmissionFailed,    Call cleared because gatekeeper admission request failed.
   };
 
+  if (callEndReason.q931 != Q931::UnknownCauseIE)
+    return (Q931::CauseValues)callEndReason.q931;
+
   int code = ReasonCodes[callEndReason];
-  if (code == Q931::UnknownCauseIE)
-    return (Q931::CauseValues)callEndReason;
   if (code >= 0)
     return (Q931::CauseValues)code;
 
@@ -863,14 +1037,12 @@ H225_ReleaseComplete_UUIE &
   SetH225Version(connection, release.m_protocolIdentifier);
   release.m_callIdentifier.m_guid = connection.GetCallIdentifier();
 
-  Q931::CauseValues cause = (Q931::CauseValues)connection.GetQ931Cause();
-  if (cause == Q931::ErrorInCauseIE)
-    cause = H323TranslateFromCallEndReason(connection.GetCallEndReason(), release.m_reason);
+  Q931::CauseValues cause = H323TranslateFromCallEndReason(connection.GetCallEndReason(), release.m_reason);
   if (cause != Q931::ErrorInCauseIE)
     q931pdu.SetCause(cause);
   else
     release.IncludeOptionalField(H225_ReleaseComplete_UUIE::e_reason);
-  
+
 #if OPAL_H460
   SendFeatureSet<H225_ReleaseComplete_UUIE>(&connection, H460_MessageType::e_releaseComplete, m_h323_uu_pdu, release);
 #endif
@@ -986,7 +1158,7 @@ void H323SignalPDU::BuildQ931()
 
 void H323SignalPDU::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  std::streamsize indent = strm.precision() + 2;
   strm << "{\n"
        << setw(indent+10) << "q931pdu = " << setprecision(indent) << q931pdu << '\n'
        << setw(indent+10) << "h225pdu = " << setprecision(indent);
@@ -1169,7 +1341,7 @@ PBoolean H323SignalPDU::GetSourceE164(PString & number) const
 
   for (i = 0; i < setup.m_sourceAddress.GetSize(); i++) {
     PString str = H323GetAliasAddressString(setup.m_sourceAddress[i]);
-    if (IsE164(str)) {
+    if (OpalIsE164(str)) {
       number = str;
       return PTrue;
     }
@@ -1201,7 +1373,7 @@ PBoolean H323SignalPDU::GetDestinationE164(PString & number) const
 
   for (i = 0; i < setup.m_destinationAddress.GetSize(); i++) {
     PString str = H323GetAliasAddressString(setup.m_destinationAddress[i]);
-    if (IsE164(str)) {
+    if (OpalIsE164(str)) {
       number = str;
       return PTrue;
     }
@@ -1221,12 +1393,7 @@ unsigned H323SignalPDU::GetDistinctiveRing() const
 }
 
 
-void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
-                                  PBoolean insertPartyNumbers,
-                                  unsigned plan,
-                                  unsigned type,
-                                  int presentation,
-                                  int screening)
+void H323SignalPDU::SetQ931Fields(const H323Connection & connection, bool insertPartyNumbers)
 {
   const PStringList & aliases = connection.GetLocalAliasNames();
 
@@ -1240,11 +1407,11 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     displayName = stringOptions(OPAL_OPT_CALLING_DISPLAY_NAME, connection.GetDisplayName());
   }
 
-  if (IsE164(localName)) {
+  if (OpalIsE164(localName)) {
     number = localName;
     if (displayName.IsEmpty()) {
       for (PStringList::const_iterator alias = aliases.begin(); alias != aliases.end(); ++alias) {
-        if (!IsE164(*alias)) {
+        if (!OpalIsE164(*alias)) {
           displayName = *alias;
           break;
         }
@@ -1255,7 +1422,7 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     if (displayName.IsEmpty())
       displayName = localName;
     for (PStringList::const_iterator alias = aliases.begin(); alias != aliases.end(); ++alias) {
-      if (IsE164(*alias)) {
+      if (OpalIsE164(*alias)) {
         number = *alias;
         break;
       }
@@ -1268,10 +1435,15 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     PString otherNumber = connection.GetRemotePartyNumber();
     if (otherNumber.IsEmpty()) {
       PString otherName = connection.GetRemotePartyName();
-      if (IsE164(otherName))
+      if (OpalIsE164(otherName))
         otherNumber = otherName;
     }
 
+    int plan = 1; // ISDN/Telephony numbering system
+    int type = 0; // Unknown number type
+    int presentation = connection.IsPresentationBlocked() ? 1 : -1; // Presentation Blocked, or no octet3a
+    int screening = presentation == -1 ? -1 : 0; // User-provided, not screened
+
     if (connection.HadAnsweredCall()) {
       if (!number)
         q931pdu.SetCalledPartyNumber(number, plan, type);
@@ -1636,6 +1808,53 @@ H245_MiscellaneousIndication & H323ControlPDU::BuildMiscellaneousIndication(unsi
 }
 
 
+H245_GenericMessage & H323ControlPDU::BuildGenericRequest(const PString & identifier, unsigned subMsgId)
+{
+  H245_GenericMessage & msg = Build(H245_RequestMessage::e_genericRequest);
+  H323SetCapabilityIdentifier(identifier, msg.m_messageIdentifier);
+  msg.m_subMessageIdentifier = subMsgId;
+  return msg;
+}
+
+
+H245_GenericMessage & H323ControlPDU::BuildGenericResponse(const PString & identifier, unsigned subMsgId)
+{
+  H245_GenericMessage & msg = Build(H245_ResponseMessage::e_genericResponse);
+  H323SetCapabilityIdentifier(identifier, msg.m_messageIdentifier);
+
+  msg.IncludeOptionalField(H245_GenericMessage::e_subMessageIdentifier);
+  msg.m_subMessageIdentifier = subMsgId;
+
+  // Assume always have a content
+  msg.IncludeOptionalField(H245_GenericMessage::e_messageContent);
+  return msg;
+}
+
+
+H245_GenericMessage & H323ControlPDU::BuildGenericCommand(const PString & identifier, unsigned subMsgId)
+{
+  H245_GenericMessage & msg = Build(H245_CommandMessage::e_genericCommand);
+  H323SetCapabilityIdentifier(identifier, msg.m_messageIdentifier);
+
+  msg.IncludeOptionalField(H245_GenericMessage::e_subMessageIdentifier);
+  msg.m_subMessageIdentifier = subMsgId;
+
+  return msg;
+}
+
+
+H245_GenericMessage & H323ControlPDU::BuildGenericIndication(const PString & identifier, unsigned subMsgId)
+{
+  H245_GenericMessage & msg = Build(H245_IndicationMessage::e_genericIndication);
+  H323SetCapabilityIdentifier(identifier, msg.m_messageIdentifier);
+
+  msg.IncludeOptionalField(H245_GenericMessage::e_subMessageIdentifier);
+  msg.m_subMessageIdentifier = subMsgId;
+
+  return msg;
+}
+
+
 H245_FunctionNotUnderstood &
       H323ControlPDU::BuildFunctionNotUnderstood(const H323ControlPDU & pdu)
 {
