@@ -24,11 +24,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23294 $
- * $Author: rjongbloed $
- * $Date: 2009-08-28 06:26:26 +0000 (Fri, 28 Aug 2009) $
+ * $Revision: 24728 $
+ * $Author: dereksmithies $
+ * $Date: 2010-09-24 04:09:27 -0500 (Fri, 24 Sep 2010) $
  */
 
+
+
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
@@ -66,6 +68,8 @@
  *
  ********************************************************************************/
 
+
+
 IAX2CallProcessor::IAX2CallProcessor(IAX2EndPoint &ep)
   : IAX2Processor(ep)
 {
@@ -77,7 +81,7 @@ IAX2CallProcessor::IAX2CallProcessor(IAX2EndPoint &ep)
   
   audioCanFlow = PFalse;
   audioFramesNotStarted = PTrue;
-
+  suppressHangupFrame = PFalse;
   con = NULL;
 
   firstMediaFrame = PTrue;
@@ -167,7 +171,7 @@ void IAX2CallProcessor::PutSoundPacketToNetwork(PBYTEArray *sound)
 
 void IAX2CallProcessor::CallStopSounds()
 {
-  cout << "We have received a call stop sounds packet " << endl;
+  //cout << "We have received a call stop sounds packet " << endl;
 }
 
 void IAX2CallProcessor::ReceivedHookFlash() 
@@ -205,6 +209,11 @@ void IAX2CallProcessor::CheckForHangupMessages()
   if (hangList.IsEmpty()) 
     return;
 
+  if (suppressHangupFrame == PTrue) {
+    Terminate();
+    return;
+  }
+
   if (!IsCallTerminating()) {
     IAX2FullFrameProtocol * f = 
       new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdHangup, IAX2FullFrame::callIrrelevant);
@@ -224,7 +233,7 @@ void IAX2CallProcessor::CheckForHangupMessages()
 void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
 {      // Parse a string like guest@node.name.com/23234
   
-  PTRACE(2, "Connect to remote node " << newRemoteNode);
+  PTRACE(2, "CallProc\tConnect to remote node " << newRemoteNode);
   PStringArray res = IAX2EndPoint::DissectRemoteParty(newRemoteNode);
 
   if (res[IAX2EndPoint::addressIndex].IsEmpty()) {
@@ -242,16 +251,44 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
   PIPSocket::Address ip;
   if (!PIPSocket::GetHostAddress(res[IAX2EndPoint::addressIndex], ip)) {
     PTRACE(1, "Conection\tFailed to make call to " << res[IAX2EndPoint::addressIndex]);
-      cout << "Could not make a call to " << res[IAX2EndPoint::addressIndex] 
-     << " as IP resolution failed" << endl;
-      return;
-    }
+    return;
+  }
   PTRACE(4, "Resolve " << res[IAX2EndPoint::addressIndex]  << " as ip address " << ip);
-    
-  remote.SetRemotePort(con->GetEndPoint().ListenPortNumber());
+
+  if (!res[IAX2EndPoint::portIndex].IsEmpty())
+	  remote.SetRemotePort(res[IAX2EndPoint::portIndex].AsUnsigned());
+  else
+	  remote.SetRemotePort(con->GetEndPoint().ListenPortNumber());
   remote.SetRemoteAddress(ip);
-    
-  IAX2FullFrameProtocol * f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew);
+
+  if (res[IAX2EndPoint::userIndex].IsEmpty()) {
+    callingName = userName;
+  } else {
+    callingName = res[IAX2EndPoint::userIndex];
+    userName = callingName;
+  }
+
+  callingExtension = res[IAX2EndPoint::extensionIndex];
+  callingDnid = res[IAX2EndPoint::extensionIndex];
+  callingContext = res[IAX2EndPoint::contextIndex];
+
+  IAX2FullFrameProtocol * f = BuildNewFrameForSending();
+#if P_SSL
+  f->AppendIe(new IAX2IeCallToken());
+#endif
+  TransmitFrameToRemoteEndpoint(f);
+  StartNoResponseTimer();
+  return;
+}
+
+
+IAX2FullFrameProtocol *IAX2CallProcessor::BuildNewFrameForSending(IAX2FullFrameProtocol *inReplyTo)
+{
+  IAX2FullFrameProtocol * f;
+  if (inReplyTo == NULL)
+    f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew);
+  else
+    f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew, inReplyTo);
   f->AppendIe(new IAX2IeVersion());
   f->AppendIe(new IAX2IeFormat(con->GetPreferredCodec()));
   f->AppendIe(new IAX2IeCapability(con->GetSupportedCodecs()));
@@ -259,34 +296,29 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
   if (!endpoint.GetLocalNumber().IsEmpty())
     f->AppendIe(new IAX2IeCallingNumber(endpoint.GetLocalNumber()));
 
-  if (!res[IAX2EndPoint::userIndex].IsEmpty()) {
-    f->AppendIe(new IAX2IeCallingName(res[IAX2EndPoint::userIndex]));
-    f->AppendIe(new IAX2IeUserName(res[IAX2EndPoint::userIndex]));
-  } else if (!GetUserName().IsEmpty()) {
-    f->AppendIe(new IAX2IeCallingName(GetUserName()));
-    f->AppendIe(new IAX2IeUserName(GetUserName()));
-  }
+  f->AppendIe(new IAX2IeCallingName(GetCallingName()));
+  f->AppendIe(new IAX2IeUserName(GetUserName()));
 
-  if (!res[IAX2EndPoint::extensionIndex].IsEmpty())
-    f->AppendIe(new IAX2IeCalledNumber(res[IAX2EndPoint::extensionIndex] ));
+  if (!callingExtension.IsEmpty())
+    f->AppendIe(new IAX2IeCalledNumber(callingExtension));
 
-  if (!res[IAX2EndPoint::extensionIndex].IsEmpty())
-    f->AppendIe(new IAX2IeDnid(res[IAX2EndPoint::extensionIndex]));
+  if (!callingDnid.IsEmpty())
+    f->AppendIe(new IAX2IeDnid(callingDnid));
 
-  if (!res[IAX2EndPoint::contextIndex].IsEmpty())
-    f->AppendIe(new IAX2IeCalledContext(res[IAX2EndPoint::contextIndex]));
+  if (!callingContext.IsEmpty())
+    f->AppendIe(new IAX2IeCalledContext(callingContext));
 
 #if OPAL_PTLIB_SSL_AES
   f->AppendIe(new IAX2IeEncryption());
 #endif
 
-  TransmitFrameToRemoteEndpoint(f);
-  StartNoResponseTimer();
-  return;
+  return f;
 }
 
+
 void IAX2CallProcessor::ReportStatistics()
 {
+#if 0
   cout << "  Call has been up for " << setprecision(0) << setw(8)
        << (PTimer::Tick() - callStartTick) << " milliseconds" << endl
        << "  Control frames sent " << controlFramesSent  << endl
@@ -295,6 +327,7 @@ void IAX2CallProcessor::ReportStatistics()
        << "  Audio frames rcvd   " << audioFramesRcvd      << endl
        << "  Video frames sent   " << videoFramesSent      << endl
        << "  Video frames rcvd   " << videoFramesRcvd      << endl;
+#endif
 }
 
 void IAX2CallProcessor::ProcessFullFrame(IAX2FullFrame & fullFrame)
@@ -402,6 +435,7 @@ void IAX2CallProcessor::ProcessLists()
   CheckForHangupMessages();
 }
 
+
 void IAX2CallProcessor::ProcessIncomingAudioFrame(IAX2Frame *newFrame)
 {
   PTRACE(5, "Processor\tProcessIncomingAudioframe " << newFrame->IdString());
@@ -418,7 +452,6 @@ void IAX2CallProcessor::ProcessIncomingVideoFrame(IAX2Frame *newFrame)
   delete newFrame;
 }
 
-
 void IAX2CallProcessor::SendSoundMessage(PBYTEArray *sound)
 {
   if (sound == NULL)
@@ -538,6 +571,8 @@ void IAX2CallProcessor::SendUnQuelchMessage()
   TransmitFrameToRemoteEndpoint(f);
 }
 
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2MiniFrame * src)
 {
   src->AlterTimeStamp(lastFullFrameTimeStamp);
@@ -615,9 +650,13 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameVideo * src)
   return;
 }
 
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameSessionControl * src)
-{ /* these frames are labelled as AST_FRAME_CONTROL in the asterisk souces.
-     We could get an Answer message from here., or a hangup., or...congestion control... */
+{
+ /* these frames are labelled as AST_FRAME_CONTROL in the asterisk
+     souces.  We could get an Answer message from here., or a hangup.,
+     or...congestion control... */
   
   PTRACE(4, "ProcessNetworkFrame(IAX2FullFrameSessionControl * src)");
   SendAckFrame(src);  
@@ -625,7 +664,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameSessionControl * src)
   switch(src->GetSubClass()) {
   case IAX2FullFrameSessionControl::hangup:          // Other end has hungup
     SetCallTerminating(PTrue);
-    cout << "Other end has hungup, so exit" << endl;
+    //cout << "Other end has hungup, so exit" << endl;
     con->EndCallNow();
     break;
     
@@ -698,6 +737,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameSessionControl * src)
   return;
 }
 
+
 PBoolean IAX2CallProcessor::SetUpConnection()
 {
   PTRACE(3, "IAX2\tSet Up Connection to remote node " 
@@ -765,7 +805,8 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
   case IAX2FullFrameProtocol::cmdDial:
     ProcessIaxCmdDial(src);
     break;
-  /*case IAX2FullFrameProtocol::cmdTxreq:
+#if DONTUSE
+  case IAX2FullFrameProtocol::cmdTxreq:
     ProcessIaxCmdTxreq(src);
     break;
   case IAX2FullFrameProtocol::cmdTxcnt:
@@ -782,7 +823,7 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdTxrej:
     ProcessIaxCmdTxrej(src);
-    break;*/
+    break;
   case IAX2FullFrameProtocol::cmdQuelch:
     ProcessIaxCmdQuelch(src);
     break;
@@ -791,7 +832,7 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdPage:
     ProcessIaxCmdPage(src);
-    break;/*
+    break;
   case IAX2FullFrameProtocol::cmdMwi:
     ProcessIaxCmdMwi(src);
     break;
@@ -809,7 +850,11 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdFwData:
     ProcessIaxCmdFwData(src);
-    break;*/
+    break;
+#endif
+  case IAX2FullFrameProtocol::cmdCallToken:
+    ProcessIaxCmdCallToken(src);
+    break;
   default:
     PTRACE(1, "Process Full Frame Protocol, Type not expected");
     SendUnsupportedFrame(src);   //This method will delete the frame src.
@@ -818,6 +863,9 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
   return PFalse;
 }
 
+
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameText * src)
 {
   PTRACE(4, "ProcessNetworkFrame(IAX2FullFrameText * src)");
@@ -845,7 +893,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameCng * src)
   delete src;
   return;
 }
-
+  
 void IAX2CallProcessor::CheckForRemoteCapabilities(IAX2FullFrameProtocol *src)
 {
   unsigned int remoteCapability, format;
@@ -859,6 +907,8 @@ void IAX2CallProcessor::CheckForRemoteCapabilities(IAX2FullFrameProtocol *src)
   con->BuildRemoteCapabilityTable(remoteCapability, format);
 }
 
+
+  
 PBoolean IAX2CallProcessor::RemoteSelectedCodecOk()
 {
   selectedCodec = con->ChooseCodec();
@@ -875,6 +925,9 @@ PBoolean IAX2CallProcessor::RemoteSelectedCodecOk()
   
   return PTrue;
 }
+  
+
+ 
 
 void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
 { /*That we are here indicates this connection is already in place */
@@ -882,6 +935,32 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   remote.SetRemoteAddress(src->GetRemoteInfo().RemoteAddress());
   remote.SetRemotePort(src->GetRemoteInfo().RemotePort());
 
+  PIPSocket::Address addr(remote.RemoteAddress());
+  IAX2IeCallToken token;
+  if (src->GetCallTokenIe(token)) {
+    PTRACE(3, "CallProc\tReceived a callToken");
+    if (token.GetLengthOfData() < 5) {
+      IAX2IeCallToken *tSend = new IAX2IeCallToken();
+      tSend->WriteKeySequence(addr);
+      IAX2FullFrameProtocol * reply = 
+	 new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdCallToken, src);
+      reply->AppendIe(tSend);
+      TransmitFrameToRemoteEndpoint(reply);
+      delete src;
+      return;
+    } else {
+      bool validCallToken = IAX2IeCallToken::ValidKeySequence(token, addr);
+      if (!validCallToken) {
+	PTRACE(3, "CallProc\tfailed CALLTOKEN test");
+	IAX2FullFrameProtocol * reply = 
+	  new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdReject, src);
+	TransmitFrameToRemoteEndpoint(reply);
+	delete src;
+	return;
+      }
+      PTRACE(3, "CallProc\tValid CALLTOKEN test");
+    }
+  }
   if (IsCallHappening()) {
     PTRACE(3, "Remote node has sent us a second new message. ignore");
     delete src;
@@ -889,15 +968,14 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   }
 
   IAX2FullFrameProtocol * reply;
-  
-  
+    
   if (!RemoteSelectedCodecOk()) {
-    PTRACE(3, "Remote node sected a bad codec, hangup call ");
+    PTRACE(3, "CallProc\tRemote node sected a bad codec, hangup call ");
     reply = new  IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdInval, src, IAX2FullFrame::callIrrelevant);
     TransmitFrameToRemoteEndpoint(reply);
     
     reply= new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdHangup, IAX2FullFrame::callIrrelevant);
-    PTRACE(3, "Send a hangup frame to the remote endpoint as there is no codec available");    
+    PTRACE(3, "CallProc\tSend a hangup frame to the remote endpoint as there is no codec available");    
     reply->AppendIe(new IAX2IeCause("No matching codec"));
     SetCallTerminating();
     //              f->AppendIe(new IeCauseCode(IeCauseCode::NormalClearing));
@@ -936,6 +1014,7 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   delete src;
 }
 
+
 void IAX2CallProcessor::ProcessIaxCmdAck(IAX2FullFrameProtocol *src)
 {
   PTRACE(4, "ProcessIaxCmdAck(IAX2FullFrameProtocol * /*src*/)");
@@ -1019,9 +1098,10 @@ void IAX2CallProcessor::ProcessIaxCmdHangup(IAX2FullFrameProtocol *src)
 
 void IAX2CallProcessor::ProcessIaxCmdReject(IAX2FullFrameProtocol *src)
 {
+  suppressHangupFrame = PTrue;
   PTRACE(3, "Processor\tProcessIaxCmdReject(IAX2FullFrameProtocol *src)");
-  cout << "Remote endpoint has rejected our call " << endl;
-  cout << "Cause \"" << ieData.cause << "\"" << endl;
+  //cout << "Remote endpoint has rejected our call " << endl;
+  //cout << "Cause \"" << ieData.cause << "\"" << endl;
   SendAckFrame(src);
   con->EndCallNow(OpalConnection::EndedByRefusal);
 
@@ -1031,7 +1111,7 @@ void IAX2CallProcessor::ProcessIaxCmdReject(IAX2FullFrameProtocol *src)
 void IAX2CallProcessor::ProcessIaxCmdAccept(IAX2FullFrameProtocol *src)
 {
   if (IsCallAccepted()) {
-    PTRACE(3, "Second accept packet received. Ack, delete, & do nothing.");
+    PTRACE(3, "Processor\t2nd accept packet received. ack+del+ignore.");
     SendAckFrame(src);
     delete src;
     return;
@@ -1044,7 +1124,6 @@ void IAX2CallProcessor::ProcessIaxCmdAccept(IAX2FullFrameProtocol *src)
   PString calleeName = con->GetRemotePartyName();
   con->SetAlerting(calleeName, PTrue);
 
-
   SendAckFrame(src);
   SetCallAccepted();
   
@@ -1217,6 +1296,23 @@ void IAX2CallProcessor::ProcessIaxCmdFwData(IAX2FullFrameProtocol *src)
   delete src;
 }
 
+void IAX2CallProcessor::ProcessIaxCmdCallToken(IAX2FullFrameProtocol *src)
+{
+  PTRACE(4, "Processor\tProcessIaxCmdCallToken(IAX2FullFrameProtocol *src)");
+  IAX2IeCallToken *ie = new IAX2IeCallToken();
+  if (src->GetCallTokenIe(*ie)) {
+    IAX2FullFrameProtocol *f = BuildNewFrameForSending();
+    f->AppendIe(ie);
+    GetSequenceInfo().ZeroAllValues();
+    TransmitFrameToRemoteEndpoint(f);
+    StopNoResponseTimer();
+    StartNoResponseTimer();
+  } else
+    delete ie;
+  
+  delete src;
+}
+
 void IAX2CallProcessor::StartStatusCheckTimer(PINDEX msToWait)
 {
   PTRACE(4, "Processor\tStatusCheck time. Now set flag to send a ping+lagrq packets");
@@ -1295,15 +1391,13 @@ PBoolean IAX2CallProcessor::IncomingMessageOutOfOrder(IAX2FullFrame *f)
   return PFalse;
 }
 
-
 #endif // OPAL_IAX2
 
 ////////////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
