@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23294 $
+ * $Revision: 25368 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 01:26:26 -0500 (Fri, 28 Aug 2009) $
+ * $Date: 2011-03-21 01:34:07 -0500 (Mon, 21 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -52,7 +52,7 @@ static const char PrefixPOTS[] = "pots";
 /////////////////////////////////////////////////////////////////////////////
 
 OpalLineEndPoint::OpalLineEndPoint(OpalManager & mgr)
-  : OpalEndPoint(mgr, PrefixPOTS, CanTerminateCall),
+  : OpalEndPoint(mgr, PrefixPOTS, CanTerminateCall|SupportsE164),
     defaultLine("*")
 {
   PTRACE(4, "LID EP\tOpalLineEndPoint created");
@@ -82,11 +82,11 @@ OpalLineEndPoint::~OpalLineEndPoint()
 }
 
 
-PBoolean OpalLineEndPoint::MakeConnection(OpalCall & call,
-                                     const PString & remoteParty,
-                                     void * userData,
-                                     unsigned int /*options*/,
-                                     OpalConnection::StringOptions *)
+PSafePtr<OpalConnection> OpalLineEndPoint::MakeConnection(OpalCall & call,
+                                                     const PString & remoteParty,
+                                                              void * userData,
+                                                        unsigned int /*options*/,
+                                     OpalConnection::StringOptions * /*stringOptions*/)
 {
   PTRACE(3, "LID EP\tMakeConnection to " << remoteParty);  
 
@@ -120,7 +120,7 @@ PBoolean OpalLineEndPoint::MakeConnection(OpalCall & call,
   }  
   if (line == NULL){
     PTRACE(1,"LID EP\tMakeConnection cannot find the default line " << defaultLine);
-    return false;
+    return NULL;
 
   }
 
@@ -130,10 +130,7 @@ PBoolean OpalLineEndPoint::MakeConnection(OpalCall & call,
 
 OpalMediaFormatList OpalLineEndPoint::GetMediaFormats() const
 {
-  OpalMediaFormatList mediaFormats;
-#if OPAL_VIDEO
-  AddVideoMediaFormats(mediaFormats);
-#endif
+  OpalMediaFormatList mediaFormats = manager.GetCommonMediaFormats(false, false);
 
   PWaitAndSignal mutex(linesMutex);
 
@@ -359,7 +356,7 @@ OpalLine * OpalLineEndPoint::GetLine(const PString & lineName, bool enableAudio,
     PString lineToken = line->GetToken();
     if (lineName != defaultLine && lineToken != lineName)
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", default=" << defaultLine);
-    else if (line->IsTerminal() == terminating)
+    else if (line->IsTerminal() != terminating)
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", terminating=" << line->IsTerminal());
     else if (!line->IsPresent())
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", not present");
@@ -385,6 +382,30 @@ void OpalLineEndPoint::SetDefaultLine(const PString & lineName)
 }
 
 
+bool OpalLineEndPoint::SetCountryCode(OpalLineInterfaceDevice::T35CountryCodes country)
+{
+  PWaitAndSignal mutex(linesMutex);
+  for (OpalLIDList::iterator iterDev = devices.begin(); iterDev != devices.end(); ++iterDev) {
+    if (!iterDev->SetCountryCode(country))
+      return false;
+  }
+
+  return true;
+}
+
+
+bool OpalLineEndPoint::SetCountryCodeName(const PString & countryName)
+{
+  PWaitAndSignal mutex(linesMutex);
+  for (OpalLIDList::iterator iterDev = devices.begin(); iterDev != devices.end(); ++iterDev) {
+    if (!iterDev->SetCountryCodeName(countryName))
+      return false;
+  }
+
+  return true;
+}
+
+
 void OpalLineEndPoint::MonitorLines(PThread &, INT)
 {
   PTRACE(4, "LID EP\tMonitor thread started for " << GetPrefixName());
@@ -458,9 +479,13 @@ OpalLineConnection::OpalLineConnection(OpalCall & call,
                                        OpalLineEndPoint & ep,
                                        OpalLine & ln,
                                        const PString & number)
-  : OpalConnection(call, ep, ln.GetToken()),
-    endpoint(ep),
-    line(ln)
+  : OpalConnection(call, ep, ln.GetToken())
+  , endpoint(ep)
+  , line(ln)
+  , wasOffHook(false)
+  , minimumRingCount(2)
+  , m_promptTone(OpalLineInterfaceDevice::DialTone)
+  , handlerThread(NULL)
 {
   localPartyName = ln.GetToken();
   remotePartyNumber = number.Right(number.FindSpan("0123456789*#,"));
@@ -473,11 +498,6 @@ OpalLineConnection::OpalLineConnection(OpalCall & call,
 
   silenceDetector = new OpalLineSilenceDetector(line, (endpoint.GetManager().GetSilenceDetectParams()));
 
-  minimumRingCount = 2;
-
-  wasOffHook = false;
-  handlerThread = NULL;
-
   PTRACE(3, "LID Con\tConnection " << callToken << " created to " << (number.IsEmpty() ? "local" : number));
   
 }
@@ -508,8 +528,6 @@ void OpalLineConnection::OnReleased()
   else
     line.SetOnHook();
 
-  SetPhase(ReleasedPhase);
-
   OpalConnection::OnReleased();
 }
 
@@ -529,7 +547,6 @@ PBoolean OpalLineConnection::SetAlerting(const PString & /*calleeName*/, PBoolea
 
   // switch phase 
   SetPhase(AlertingPhase);
-  alertingTime = PTime();
 
   if (line.IsTerminal() && GetMediaStream(OpalMediaType::Audio(), false) == NULL) {
     // Start ringing if we don't have an audio media stream
@@ -567,18 +584,15 @@ PBoolean OpalLineConnection::SetConnected()
     wasOffHook = true;
   }
 
-  if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+  AutoStartMediaStreams();
   return OpalConnection::SetConnected();
 }
 
 
 OpalMediaFormatList OpalLineConnection::GetMediaFormats() const
 {
-  OpalMediaFormatList formats = line.GetDevice().GetMediaFormats();
-#if OPAL_VIDEO
-  AddVideoMediaFormats(formats);
-#endif
+  OpalMediaFormatList formats = endpoint.GetManager().GetCommonMediaFormats(false, false);
+  formats += line.GetDevice().GetMediaFormats();
   return formats;
 }
 
@@ -600,17 +614,22 @@ PBoolean OpalLineConnection::OnOpenMediaStream(OpalMediaStream & mediaStream)
   if (!OpalConnection::OnOpenMediaStream(mediaStream))
     return false;
 
-  if (mediaStream.IsSource()) {
-    OpalMediaPatch * patch = mediaStream.GetPatch();
-    if (patch != NULL)
-      patch->AddFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
-  }
+  if (mediaStream.IsSource())
+    mediaStream.AddFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
 
   line.StopTone(); // In case a RoutingTone or RingTone is going
   return true;
 }
 
 
+void OpalLineConnection::OnClosedMediaStream(const OpalMediaStream & mediaStream)
+{
+  mediaStream.RemoveFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
+
+  OpalConnection::OnClosedMediaStream(mediaStream);
+}
+
+
 PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage)
 {
   PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
@@ -653,7 +672,7 @@ PBoolean OpalLineConnection::PromptUserInput(PBoolean play)
   PTRACE(3, "LID Con\tConnection " << callToken << " dial tone " << (play ? "started" : "stopped"));
 
   if (play) {
-    if (line.PlayTone(OpalLineInterfaceDevice::DialTone)) {
+    if (line.PlayTone(m_promptTone)) {
       PTRACE(3, "LID Con\tPlaying dial tone");
       return true;
     }
@@ -695,9 +714,8 @@ void OpalLineConnection::Monitor()
         StartIncoming(); // We are A-party
       else {
         // If we are in alerting state then we are B-Party
+        AutoStartMediaStreams();
         OnConnectedInternal();
-        if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
-          ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
       }
     }
   }
@@ -752,7 +770,7 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
         return;
       }
       PThread::Sleep(100);
-      if (GetPhase() >= ReleasingPhase)
+      if (IsReleased())
         return;
     } while (count < minimumRingCount); // Wait till we have CLID
 
@@ -779,13 +797,12 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
 
     // switch phase 
     SetPhase(AlertingPhase);
-    alertingTime = PTime();
   }
 
   if (!OnIncomingConnection(0, NULL)) {
     PTRACE(3, "LID\tWaiting for RING to stop on " << *this);
     while (line.GetRingCount() > 0) {
-      if (GetPhase() >= ReleasingPhase)
+      if (IsReleased())
         return;
       PThread::Sleep(100);
     }
@@ -831,9 +848,8 @@ PBoolean OpalLineConnection::SetUpConnection()
     }
 
     PTRACE(3, "LID Con\tNo remote party indicated, going off hook without dialing.");
+    AutoStartMediaStreams();
     OnConnectedInternal();
-    if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
-      ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
     return true;
   }
 
@@ -858,8 +874,7 @@ PBoolean OpalLineConnection::SetUpConnection()
 
   PTRACE(3, "LID Con\tGot ring back on " << line);
   // Start media before the OnAlerting to get progress tones (e.g. SIP 183 response)
-  if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+  AutoStartMediaStreams();
   SetPhase(AlertingPhase);
   OnAlerting();
 
@@ -921,6 +936,11 @@ OpalLineMediaStream::OpalLineMediaStream(OpalLineConnection & conn,
   lastSID[0] = 2;
 }
 
+OpalLineMediaStream::~OpalLineMediaStream()
+{
+  Close();
+}
+
 
 PBoolean OpalLineMediaStream::Open()
 {
@@ -1130,23 +1150,23 @@ PBoolean OpalLineMediaStream::IsSynchronous() const
 }
 
 
-PBoolean OpalLineMediaStream::RequiresPatchThread(OpalMediaStream * sinkStream) const
+PBoolean OpalLineMediaStream::RequiresPatchThread(OpalMediaStream * stream) const
 {
-  OpalLineMediaStream * sinkLine = dynamic_cast<OpalLineMediaStream *>(sinkStream);
-  if (sinkLine != NULL && &line.GetDevice() == &sinkLine->line.GetDevice()) {
-    if (line.GetDevice().SetLineToLineDirect(line.GetLineNumber(), sinkLine->line.GetLineNumber(), true)) {
+  OpalLineMediaStream * lineStream = dynamic_cast<OpalLineMediaStream *>(stream);
+  if (lineStream != NULL && &line.GetDevice() == &lineStream->line.GetDevice()) {
+    if (line.GetDevice().SetLineToLineDirect(line.GetLineNumber(), lineStream->line.GetLineNumber(), true)) {
       PTRACE(3, "LineMedia\tDirect line connection between "
-             << line.GetLineNumber() << " and " << sinkLine->line.GetLineNumber()
+             << line.GetLineNumber() << " and " << lineStream->line.GetLineNumber()
              << " on device " << line.GetDevice());
-      const_cast<OpalLineMediaStream *>(this)->directLineNumber = sinkLine->line.GetLineNumber();
-      sinkLine->directLineNumber = line.GetLineNumber();
+      const_cast<OpalLineMediaStream *>(this)->directLineNumber = lineStream->line.GetLineNumber();
+      lineStream->directLineNumber = line.GetLineNumber();
       return false; // Do not start threads
     }
     PTRACE(2, "LineMedia\tCould not do direct line connection between "
-           << line.GetLineNumber() << " and " << sinkLine->line.GetLineNumber()
+           << line.GetLineNumber() << " and " << lineStream->line.GetLineNumber()
            << " on device " << line.GetDevice());
   }
-  return OpalMediaStream::RequiresPatchThread(sinkStream);
+  return OpalMediaStream::RequiresPatchThread(stream);
 }
 
 
