@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24261 $
+ * $Revision: 26320 $
  * $Author: rjongbloed $
- * $Date: 2010-04-22 23:24:13 -0500 (Thu, 22 Apr 2010) $
+ * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -88,61 +88,104 @@ public:
   }
 };
 
-
-const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
+const OpalAudioFormat & GetOpalPCM16S()
 {
-  static OpalStereoAudioFormat stereo48k("Linear-16-Stereo-48kHz",    // name of the meda format
-                                         RTP_DataFrame::DynamicBase,  // RTP payload code
-                                         "L16",                       // encoding name
-                                         16,                          // frame size in bytes
-                                         48,                          // frame time (1 ms in clock units)
-                                         20,                          // recommended rx frames/packet
-                                         20,                          // recommended tx frames/packet
-                                         50,                          // max tx frame size
-                                         48000);                      // clock rate
+  static OpalStereoAudioFormat stereo8k(OPAL_PCM16S,        		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					64,				// frame size in bytes
+					16,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					8000);				// clock rate
+  return stereo8k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_16KHZ()
+{
+  static OpalStereoAudioFormat stereo16k(OPAL_PCM16S_16KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					64,				// frame size in bytes
+					16,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					16000);				// clock rate
+  return stereo16k;
+};
+
+const OpalAudioFormat & GetOpalL16_STEREO_16KHZ()
+{
+  static OpalStereoAudioFormat stereo16k(OPAL_L16_STEREO_16KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					64,                             // frame size in bytes
+					16,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					16000);                         // clock rate
+  return stereo16k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_32KHZ()
+{
+  static OpalStereoAudioFormat stereo32k(OPAL_PCM16S_32KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					128,				// frame size in bytes
+					32,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					32000);				// clock rate
+  return stereo32k;
+};
+
+const OpalAudioFormat & GetOpalL16_STEREO_32KHZ()
+{
+  static OpalStereoAudioFormat stereo32k(OPAL_L16_STEREO_32KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					128,                            // frame size in bytes
+					32,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					32000);                         // clock rate
+  return stereo32k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_48KHZ()
+{
+  static OpalStereoAudioFormat stereo48k(OPAL_PCM16S_48KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					192,				// frame size in bytes
+					48,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					48000);				// clock rate
   return stereo48k;
 };
 
-
-const OpalMediaFormat & GetOpalRFC2833()
+const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
 {
-  static struct OpalRFC2833MediaFormat : public OpalMediaFormat {
-    OpalRFC2833MediaFormat()
-      : OpalMediaFormat(OPAL_RFC2833,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
-                        "telephone-event",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "0-16,32,36"));
-    }
-  } const RFC2833;
-  return RFC2833;
-}
+  static OpalStereoAudioFormat stereo48k(OPAL_L16_STEREO_48KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					192,                            // frame size in bytes
+					48,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					48000);                         // clock rate
+  return stereo48k;
+};
 
-const OpalMediaFormat & GetOpalCiscoNSE()
-{
-  static struct OpalCiscoNSEMediaFormat : public OpalMediaFormat {
-    OpalCiscoNSEMediaFormat()
-      : OpalMediaFormat(OPAL_CISCONSE,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
-                        "NSE",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "192,193"));
-    }
-  } const CiscoNSE;
-  return CiscoNSE;
-}
 
 static OpalMediaFormatList & GetMediaFormatsList()
 {
@@ -249,7 +292,8 @@ bool OpalMediaOption::Merge(const OpalMediaOption & option)
   }
 
   if (assign) {
-    PTRACE(4, "MediaFormat\tChanged media option \"" << m_name << "\" from " << *this << " to " << option);
+    PTRACE(4, "MediaFormat\tChanged media option \"" << m_name << "\" "
+              "from \"" << *this << "\" to \"" << option << '"');
     Assign(option);
   }
 
@@ -470,6 +514,37 @@ void OpalMediaOptionString::ReadFrom(istream & strm)
 }
 
 
+bool OpalMediaOptionString::Merge(const OpalMediaOption & option)
+{
+  if (m_merge != IntersectionMerge)
+    return OpalMediaOption::Merge(option);
+
+  const OpalMediaOptionString * otherOption = PDownCast(const OpalMediaOptionString, &option);
+  if (otherOption == NULL)
+    return false;
+
+  PStringArray mySet = m_value.Tokenise(',');
+  PStringArray otherSet = otherOption->m_value.Tokenise(',');
+  PINDEX i = 0;
+  while (i < mySet.GetSize()) {
+    if (otherSet.GetValuesIndex(mySet[i]) != P_MAX_INDEX)
+      ++i;
+    else
+      mySet.RemoveAt(i);
+  }
+
+  if (mySet.IsEmpty())
+    m_value.MakeEmpty();
+  else {
+    m_value = mySet[0];
+    for (i = 1; i < mySet.GetSize(); ++i)
+      m_value += ',' + mySet[i];
+  }
+
+  return true;
+}
+
+
 PObject::Comparison OpalMediaOptionString::CompareValue(const OpalMediaOption & option) const
 {
   const OpalMediaOptionString * otherOption = PDownCast(const OpalMediaOptionString, &option);
@@ -539,9 +614,9 @@ void OpalMediaOptionOctets::PrintOn(ostream & strm) const
     ios::fmtflags flags = strm.flags();
     char fill = strm.fill();
 
-    int fillLength = width - m_value.GetSize()*2;
+    streamsize fillLength = width - m_value.GetSize()*2;
     if (fillLength > 0 && (flags&ios_base::adjustfield) == ios::right) {
-      for (int i = 0; i < fillLength; i++)
+      for (streamsize i = 0; i < fillLength; i++)
         strm << fill;
     }
 
@@ -551,7 +626,7 @@ void OpalMediaOptionOctets::PrintOn(ostream & strm) const
 
     if (fillLength > 0 && (flags&ios_base::adjustfield) == ios::left) {
       strm << setw(1);
-      for (int i = 0; i < fillLength; i++)
+      for (std::streamsize i = 0; i < fillLength; i++)
         strm << fill;
     }
 
@@ -636,20 +711,20 @@ void OpalMediaOptionOctets::SetValue(const BYTE * data, PINDEX length)
 
 /////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalMediaFormat::NeedsJitterOption()   { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;    return s; }
-const PString & OpalMediaFormat::MaxFrameSizeOption()  { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE;  return s; }
-const PString & OpalMediaFormat::FrameTimeOption()     { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;      return s; }
-const PString & OpalMediaFormat::ClockRateOption()     { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;      return s; }
-const PString & OpalMediaFormat::MaxBitRateOption()    { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;    return s; }
-const PString & OpalMediaFormat::TargetBitRateOption() { static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+const PString & OpalMediaFormat::NeedsJitterOption()   { static const PConstString s(PLUGINCODEC_OPTION_NEEDS_JITTER);    return s; }
+const PString & OpalMediaFormat::MaxFrameSizeOption()  { static const PConstString s(PLUGINCODEC_OPTION_MAX_FRAME_SIZE);  return s; }
+const PString & OpalMediaFormat::FrameTimeOption()     { static const PConstString s(PLUGINCODEC_OPTION_FRAME_TIME);      return s; }
+const PString & OpalMediaFormat::ClockRateOption()     { static const PConstString s(PLUGINCODEC_OPTION_CLOCK_RATE);      return s; }
+const PString & OpalMediaFormat::MaxBitRateOption()    { static const PConstString s(PLUGINCODEC_OPTION_MAX_BIT_RATE);    return s; }
+const PString & OpalMediaFormat::TargetBitRateOption() { static const PConstString s(PLUGINCODEC_OPTION_TARGET_BIT_RATE); return s; }
 
 #if OPAL_H323
-const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s = PLUGINCODEC_MEDIA_PACKETIZATION;  return s; }
-const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
+const PString & OpalMediaFormat::MediaPacketizationOption()  { static const PConstString s(PLUGINCODEC_MEDIA_PACKETIZATION);  return s; }
+const PString & OpalMediaFormat::MediaPacketizationsOption() { static const PConstString s(PLUGINCODEC_MEDIA_PACKETIZATIONS); return s; }
 #endif
 
-const PString & OpalMediaFormat::ProtocolOption()        { static PString s = "Protocol";           return s; }
-const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static PString s = "Max Tx Packet Size"; return s; }
+const PString & OpalMediaFormat::ProtocolOption()        { static const PConstString s("Protocol"); return s; }
+const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static const PConstString s(PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE); return s; }
 
 OpalMediaFormat::OpalMediaFormat(OpalMediaFormatInternal * info)
   : m_info(NULL)
@@ -759,32 +834,49 @@ OpalMediaFormat & OpalMediaFormat::operator=(const PString & wildcard)
 }
 
 
-void OpalMediaFormat::CloneContents(const OpalMediaFormat * c)
+PBoolean OpalMediaFormat::MakeUnique()
 {
-  PWaitAndSignal m(m_mutex);
+  PWaitAndSignal m1(m_mutex);
+  if (m_info == NULL)
+    return true;
 
-  m_info = (OpalMediaFormatInternal *)c->m_info->Clone();
+  PWaitAndSignal m2(m_info->media_format_mutex);
+
+  if (PContainer::MakeUnique())
+    return true;
+
+  m_info = (OpalMediaFormatInternal *)m_info->Clone();
   m_info->options.MakeUnique();
+  return false;
 }
 
 
-void OpalMediaFormat::CopyContents(const OpalMediaFormat & c)
+void OpalMediaFormat::AssignContents(const PContainer & c)
 {
-  PWaitAndSignal m(m_mutex);
-  m_info = c.m_info;
+  m_mutex.Wait();
+
+  PContainer::AssignContents(c);
+  m_info = ((const OpalMediaFormat &)c).m_info;
+
+  m_mutex.Signal();
 }
 
 
 void OpalMediaFormat::DestroyContents()
 {
-  PWaitAndSignal m(m_mutex);
-  delete m_info;
+  m_mutex.Wait();
+
+  if (m_info != NULL) {
+    m_info->media_format_mutex.Wait();
+    delete m_info;
+  }
+
+  m_mutex.Signal();
 }
 
 
 PObject * OpalMediaFormat::Clone() const
 {
-  PWaitAndSignal m(m_mutex);
   return new OpalMediaFormat(*this);
 }
 
@@ -873,30 +965,6 @@ bool OpalMediaFormat::ValidateMerge(const OpalMediaFormat & mediaFormat) const
 }
 
 
-OpalMediaFormatList OpalMediaFormat::GetMatchingRegisteredMediaFormats(RTP_DataFrame::PayloadTypes rtpPayloadType,
-                                                                       const unsigned clockRate,
-                                                                       const char * rtpEncodingName,
-                                                                       const char * protocol)
-{
-  OpalMediaFormatList matches;
-
-  PWaitAndSignal mutex(GetMediaFormatsListMutex());
-
-  const OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
-  OpalMediaFormatList::const_iterator format = registeredFormats.FindFormat(rtpPayloadType, clockRate, rtpEncodingName, protocol);
-  while (format != registeredFormats.end()) {
-    matches += *format;
-
-    if (++format == registeredFormats.end())
-      break;
-
-    format = registeredFormats.FindFormat(rtpPayloadType, clockRate, rtpEncodingName, protocol, format);
-  }
-
-  return matches;
-}
-
-
 OpalMediaFormatList OpalMediaFormat::GetAllRegisteredMediaFormats()
 {
   OpalMediaFormatList copy;
@@ -946,12 +1014,13 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
                                                  PINDEX   fs,
                                                  unsigned ft,
                                                  unsigned cr,
-                                                 time_t ts)
+                                                 time_t   ts)
   : formatName(fullName), mediaType(_mediaType), forceIsTransportable(false)
 {
   codecVersionTime = ts;
-  rtpPayloadType = pt;
-  rtpEncodingName = en;
+  rtpPayloadType   = pt;
+  rtpEncodingName  = en;
+  m_channels       = 1;    // this is the default - it's up to descendant classes to change it
 
   if (nj)
     AddOption(new OpalMediaOptionBoolean(OpalMediaFormat::NeedsJitterOption(), true, OpalMediaOption::OrMerge, true));
@@ -1425,10 +1494,10 @@ void OpalMediaFormatInternal::PrintOn(ostream & strm) const
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalAudioFormat::RxFramesPerPacketOption() { static PString s = PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET; return s; }
-const PString & OpalAudioFormat::TxFramesPerPacketOption() { static PString s = PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET; return s; }
-const PString & OpalAudioFormat::MaxFramesPerPacketOption(){ static PString s = "Max Frames Per Packet"; return s; }
-const PString & OpalAudioFormat::ChannelsOption()          { static PString s = "Channels"; return s; }
+const PString & OpalAudioFormat::RxFramesPerPacketOption() { static const PConstString s(PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET); return s; }
+const PString & OpalAudioFormat::TxFramesPerPacketOption() { static const PConstString s(PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET); return s; }
+const PString & OpalAudioFormat::MaxFramesPerPacketOption(){ static const PConstString s("Max Frames Per Packet"); return s; }
+const PString & OpalAudioFormat::ChannelsOption()          { static const PConstString s("Channels"); return s; }
 
 OpalAudioFormat::OpalAudioFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1481,7 +1550,7 @@ OpalAudioFormatInternal::OpalAudioFormatInternal(const char * fullName,
     AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::TxFramesPerPacketOption(), false, OpalMediaOption::AlwaysMerge, txFrames, 1, maxFrames));
 
   AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::MaxFramesPerPacketOption(), true,  OpalMediaOption::NoMerge,  maxFrames));
-  AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::ChannelsOption(),           false, OpalMediaOption::NoMerge,  1, 1, 5));
+  AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::ChannelsOption(),           false, OpalMediaOption::NoMerge,  m_channels, 1, 5));
 }
 
 
@@ -1509,18 +1578,18 @@ bool OpalAudioFormatInternal::Merge(const OpalMediaFormatInternal & mediaFormat)
 
 #if OPAL_VIDEO
 
-const PString & OpalVideoFormat::FrameWidthOption()               { static PString s = PLUGINCODEC_OPTION_FRAME_WIDTH;               return s; }
-const PString & OpalVideoFormat::FrameHeightOption()              { static PString s = PLUGINCODEC_OPTION_FRAME_HEIGHT;              return s; }
-const PString & OpalVideoFormat::MinRxFrameWidthOption()          { static PString s = PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH;        return s; }
-const PString & OpalVideoFormat::MinRxFrameHeightOption()         { static PString s = PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT;       return s; }
-const PString & OpalVideoFormat::MaxRxFrameWidthOption()          { static PString s = PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH;        return s; }
-const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static PString s = PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT;       return s; }
-const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static PString s = PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF;return s; }
-const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static PString s = PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD;       return s; }
-const PString & OpalVideoFormat::RateControlEnableOption()        { static PString s = "Rate Control Enable";                        return s; }
-const PString & OpalVideoFormat::RateControllerOption()           { static PString s = "Rate Controller";                            return s; }
-const PString & OpalVideoFormat::ContentRoleOption()              { static PString s = "Content Role";                               return s; }
-const PString & OpalVideoFormat::ContentRoleMaskOption()          { static PString s = "Content Role Mask";                          return s; }
+const PString & OpalVideoFormat::FrameWidthOption()               { static const PConstString s(PLUGINCODEC_OPTION_FRAME_WIDTH);               return s; }
+const PString & OpalVideoFormat::FrameHeightOption()              { static const PConstString s(PLUGINCODEC_OPTION_FRAME_HEIGHT);              return s; }
+const PString & OpalVideoFormat::MinRxFrameWidthOption()          { static const PConstString s(PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH);        return s; }
+const PString & OpalVideoFormat::MinRxFrameHeightOption()         { static const PConstString s(PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT);       return s; }
+const PString & OpalVideoFormat::MaxRxFrameWidthOption()          { static const PConstString s(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);        return s; }
+const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static const PConstString s(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);       return s; }
+const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static const PConstString s(PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF);return s; }
+const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static const PConstString s(PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD);       return s; }
+const PString & OpalVideoFormat::RateControlEnableOption()        { static const PConstString s("Rate Control Enable");                        return s; }
+const PString & OpalVideoFormat::RateControllerOption()           { static const PConstString s("Rate Controller");                            return s; }
+const PString & OpalVideoFormat::ContentRoleOption()              { static const PConstString s("Content Role");                               return s; }
+const PString & OpalVideoFormat::ContentRoleMaskOption()          { static const PConstString s("Content Role Mask");                          return s; }
 
 OpalVideoFormat::OpalVideoFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1570,7 +1639,7 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,    1000));
   AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          false, OpalMediaOption::NoMerge,     8192,                        100,  8192));
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          true,  OpalMediaOption::AlwaysMerge, PluginCodec_RTP_MaxPayloadSize, 100    ));
   AddOption(new OpalMediaOptionString  (OpalVideoFormat::RateControllerOption(),           false                                                                       ));
 
 
@@ -1586,7 +1655,7 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
                                     OpalMediaOption::NoMerge));
 
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::ContentRoleMaskOption(),
-                                        false, OpalMediaOption::AndMerge,
+                                        false, OpalMediaOption::IntersectionMerge,
                                         0, 0, OpalVideoFormat::ContentRoleMask));
 
   // For video the max bit rate and frame rate is adjustable by user
@@ -1642,6 +1711,23 @@ OpalMediaFormatList::OpalMediaFormatList(const OpalMediaFormat & format)
 }
 
 
+OpalMediaFormatList & OpalMediaFormatList::operator+=(const PString & wildcard)
+{
+  MakeUnique();
+
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  OpalMediaFormatList::const_iterator fmt;
+  while ((fmt = registeredFormats.FindFormat(wildcard, fmt)) != registeredFormats.end()) {
+    if (!HasFormat(*fmt))
+      OpalMediaFormatBaseList::Append(fmt->Clone());
+  }
+
+  return *this;
+}
+
+
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & format)
 {
   MakeUnique();
@@ -1687,19 +1773,51 @@ void OpalMediaFormatList::Remove(const PStringArray & maskList)
 
   PTRACE(4,"MediaFormat\tRemoving codecs " << setfill(',') << maskList);
 
-  for (PINDEX i = 0; i < maskList.GetSize(); i++) {
-    OpalMediaFormatList::const_iterator fmt;
-    while ((fmt = FindFormat(maskList[i])) != end())
-      erase(fmt);
+  PINDEX i;
+  PStringList notMasks;
+  const_iterator fmt;
+
+  for (i = 0; i < maskList.GetSize(); i++) {
+    PString mask = maskList[i];
+    if (mask[0] == '!')
+      notMasks.AppendString(mask);
+    else {
+      while ((fmt = FindFormat(mask)) != end())
+        erase(fmt);
+    }
   }
+
+  switch (notMasks.GetSize()) {
+    case 0 :
+      return;
+
+    case 1 :
+      while ((fmt = FindFormat(notMasks[0])) != end())
+        erase(fmt);
+      return;
+  }
+
+  OpalMediaFormatList formatsToKeep;
+  for (i = 0; i < notMasks.GetSize(); i++) {
+    PString keeper = notMasks[i].Mid(1);
+    fmt = const_iterator();
+    while ((fmt = FindFormat(keeper, fmt)) != end())
+      formatsToKeep += *fmt;
+  }
+
+  *this = formatsToKeep;
 }
 
 
-OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol, const_iterator iter) const
+OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol, const_iterator format) const
 {
+  if (format == const_iterator())
+    format = begin();
+  else
+    ++format;
+
   // First look for a matching encoding name
   if (name != NULL && *name != '\0') {
-    OpalMediaFormatList::const_iterator format ((iter == const_iterator()) ? begin() : iter);
     for (; format != end(); ++format) {
       // If encoding name matches exactly, then use it regardless of payload code.
       const char * otherName = format->GetEncodingName();
@@ -1715,7 +1833,6 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFram
   // someone to override a standard payload type with another encoding name, so
   // have to search all formats by name before trying by number.
   if (pt < RTP_DataFrame::DynamicBase) {
-    OpalMediaFormatList::const_iterator format ((iter == const_iterator()) ? begin() : iter);
     for (; format != end(); ++format) {
       if (format->GetPayloadType() == pt &&
           (clockRate == 0    || clockRate == format->GetClockRate()) && // if have clock rate, clock rate must match
@@ -1769,6 +1886,8 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PStrin
 
   if (iter == const_iterator())
     iter = begin();
+  else
+    ++iter;
 
   bool negative = search[0] == '!';
 
