@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 24156 $
+ * $Revision: 26163 $
  * $Author: rjongbloed $
- * $Date: 2010-03-23 23:12:09 -0500 (Tue, 23 Mar 2010) $
+ * $Date: 2011-07-11 00:36:48 -0500 (Mon, 11 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -65,7 +65,7 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , sessionID(_sessionID)
   , identifier(conn.GetCall().GetToken() + psprintf("_%u", sessionID))
   , mediaFormat(fmt)
-  , paused(false)
+  , m_paused(false)
   , isSource(isSourceStream)
   , isOpen(false)
   , defaultDataSize(mediaFormat.GetFrameSize()*mediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1))
@@ -73,6 +73,9 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , marker(true)
   , mismatchedPayloadTypes(0)
   , mediaPatch(NULL)
+  , m_payloadType(mediaFormat.GetPayloadType())
+  , m_frameTime(mediaFormat.GetFrameTime())
+  , m_frameSize(mediaFormat.GetFrameSize())
 {
   connection.SafeReference();
   PTRACE(5, "Media\tCreated " << (isSource ? "Source" : "Sink") << ' ' << this);
@@ -111,13 +114,16 @@ PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaForm
     return false;
 
   // If we are source, then update the sink side, and vice versa
-  if (mediaPatch != NULL && !fromPatch)
+  if (!fromPatch && mediaPatch != NULL)
     return mediaPatch->UpdateMediaFormat(newMediaFormat);
 
   if (!mediaFormat.Update(newMediaFormat))
     return false;
 
   PTRACE(4, "Media\tMedia format updated on " << *this);
+  m_payloadType = mediaFormat.GetPayloadType();
+  m_frameTime = mediaFormat.GetFrameTime();
+  m_frameSize = mediaFormat.GetFrameSize();
   return true;
 }
 
@@ -140,9 +146,6 @@ void OpalMediaStream::SetCommandNotifier(const PNotifier & notifier)
   if (!LockReadWrite())
     return;
 
-  if (mediaPatch != NULL)
-    mediaPatch->SetCommandNotifier(notifier, IsSink());
-
   commandNotifier = notifier;
 
   UnlockReadWrite();
@@ -161,14 +164,20 @@ PBoolean OpalMediaStream::Start()
   if (!Open())
     return false;
 
-  if (!LockReadOnly())
+  PSafeLockReadOnly safeLock(*this);
+  if (!safeLock.IsLocked())
     return false;
 
-  if (mediaPatch != NULL)
-    mediaPatch->Start();
+  if (mediaPatch == NULL)
+    return false;
 
-  UnlockReadOnly();
+  if (IsPaused()) {
+    PTRACE(4, "Media\tStarting (paused) stream " << *this);
+    return false;
+  }
 
+  PTRACE(4, "Media\tStarting stream " << *this);
+  mediaPatch->Start();
   return true;
 }
 
@@ -191,9 +200,11 @@ PBoolean OpalMediaStream::Close()
 
   isOpen = false;
 
-  if (mediaPatch == NULL)
-    UnlockReadWrite();
-  else {
+  UnlockReadWrite();
+
+  connection.OnClosedMediaStream(*this);
+
+  if (mediaPatch != NULL && LockReadWrite()) {
     PTRACE(4, "Media\tDisconnecting " << *this << " from patch thread " << *mediaPatch);
     OpalMediaPatch * patch = mediaPatch;
     mediaPatch = NULL;
@@ -209,10 +220,6 @@ PBoolean OpalMediaStream::Close()
     }
   }
 
-  if (connection.CloseMediaStream(*this))
-    return true;
-
-  connection.OnClosedMediaStream(*this);
   connection.RemoveMediaStream(*this);
   return true;
 }
@@ -229,16 +236,9 @@ PBoolean OpalMediaStream::WritePackets(RTP_DataFrameList & packets)
 }
 
 
-inline static unsigned CalculateTimestamp(PINDEX size, const OpalMediaFormat & mediaFormat)
+void OpalMediaStream::IncrementTimestamp(PINDEX size)
 {
-  unsigned frameTime = mediaFormat.GetFrameTime();
-  PINDEX frameSize = mediaFormat.GetFrameSize();
-
-  if (frameSize == 0)
-    return frameTime;
-
-  unsigned frames = (size + frameSize - 1) / frameSize;
-  return frames*frameTime;
+  timestamp += m_frameTime * (m_frameSize != 0 ? ((size + m_frameSize - 1) / m_frameSize) : 1);
 }
 
 
@@ -262,17 +262,14 @@ PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
   // If the ReadData() function did not change the timestamp then use the default
   // method or fixed frame times and sizes.
   if (oldTimestamp == timestamp)
-    timestamp += CalculateTimestamp(lastReadCount, mediaFormat);
+    IncrementTimestamp(lastReadCount);
 
-  packet.SetPayloadType(mediaFormat.GetPayloadType());
+  packet.SetPayloadType(m_payloadType);
   packet.SetPayloadSize(lastReadCount);
   packet.SetTimestamp(oldTimestamp); // Beginning of frame
   packet.SetMarker(marker);
   marker = false;
 
-  if (paused)
-    packet.SetPayloadSize(0);
-  
   return true;
 }
 
@@ -282,16 +279,13 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
   if (!isOpen)
     return false;
 
-  if (paused)
-    packet.SetPayloadSize(0);
-
   timestamp = packet.GetTimestamp();
 
   int size = packet.GetPayloadSize();
-  if (size > 0 && mediaFormat.IsTransportable()) {
-    if (packet.GetPayloadType() == mediaFormat.GetPayloadType()) {
+  if (size > 0 && m_payloadType < RTP_DataFrame::MaxPayloadType) {
+    if (packet.GetPayloadType() == m_payloadType) {
       PTRACE_IF(2, mismatchedPayloadTypes > 0,
-                "H323RTP\tPayload type matched again " << mediaFormat.GetPayloadType());
+                "H323RTP\tPayload type matched again " << m_payloadType);
       mismatchedPayloadTypes = 0;
     }
     else {
@@ -299,7 +293,7 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
       if (mismatchedPayloadTypes < MAX_PAYLOAD_TYPE_MISMATCHES) {
         PTRACE(2, "Media\tRTP data with mismatched payload type,"
                   " is " << packet.GetPayloadType() << 
-                  " expected " << mediaFormat.GetPayloadType() <<
+                  " expected " << m_payloadType <<
                   ", ignoring packet.");
         size = 0;
       }
@@ -307,48 +301,56 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
         PTRACE_IF(2, mismatchedPayloadTypes == MAX_PAYLOAD_TYPE_MISMATCHES,
                   "Media\tRTP data with consecutive mismatched payload types,"
                   " is " << packet.GetPayloadType() << 
-                  " expected " << mediaFormat.GetPayloadType() <<
+                  " expected " << m_payloadType <<
                   ", ignoring payload type from now on.");
       }
     }
   }
 
   if (size == 0) {
-    timestamp += CalculateTimestamp(1, mediaFormat);
-    packet.SetTimestamp(timestamp);
     PINDEX dummy;
-    return WriteData(NULL, 0, dummy);
-  }
-
-  marker = packet.GetMarker();
-  const BYTE * ptr = packet.GetPayloadPtr();
-
-  while (size > 0) {
-    unsigned oldTimestamp = timestamp;
-
-    PINDEX written;
-    if (!WriteData(ptr, size, written) || (written == 0)) {
-      PTRACE(2, "Media\tWritePacket failed with written " << written);
+    if (!InternalWriteData(NULL, 0, dummy))
       return false;
+  }
+  else {
+    marker = packet.GetMarker();
+    const BYTE * ptr = packet.GetPayloadPtr();
+
+    while (size > 0) {
+      PINDEX written;
+      if (!InternalWriteData(ptr, size, written))
+        return false;
+      size -= written;
+      ptr += written;
     }
 
-    // If the Write() function did not change the timestamp then use the default
-    // method of fixed frame times and sizes.
-    if (oldTimestamp == timestamp)
-      timestamp += CalculateTimestamp(size, mediaFormat);
-
-    size -= written;
-    ptr += written;
+    PTRACE_IF(1, size < 0, "Media\tRTP payload size too small, short " << -size << " bytes.");
   }
 
-  PTRACE_IF(1, size < 0, "Media\tRTP payload size too small, short " << -size << " bytes.");
-
   packet.SetTimestamp(timestamp);
 
   return true;
 }
 
 
+bool OpalMediaStream::InternalWriteData(const BYTE * data, PINDEX length, PINDEX & written)
+{
+  unsigned oldTimestamp = timestamp;
+
+  if (!WriteData(data, length, written) || (length > 0 && written == 0)) {
+    PTRACE(2, "Media\tWriteData failed, written=" << written);
+    return false;
+  }
+
+  // If the Write() function did not change the timestamp then use the default
+  // method of fixed frame times and sizes.
+  if (oldTimestamp == timestamp)
+    IncrementTimestamp(written);
+
+  return true;
+}
+
+
 PBoolean OpalMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & length)
 {
   if (!isOpen) {
@@ -382,7 +384,7 @@ PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX &
   written = length;
   RTP_DataFrame packet(length);
   memcpy(packet.GetPayloadPtr(), buffer, length);
-  packet.SetPayloadType(mediaFormat.GetPayloadType());
+  packet.SetPayloadType(m_payloadType);
   packet.SetTimestamp(timestamp);
   packet.SetMarker(marker);
   return WritePacket(packet);
@@ -425,15 +427,30 @@ PBoolean OpalMediaStream::RequiresPatchThread() const
 }
 
 
-void OpalMediaStream::EnableJitterBuffer() const
+bool OpalMediaStream::EnableJitterBuffer(bool) const
 {
+  return false;
 }
 
 
-void OpalMediaStream::SetPaused(bool p)
+bool OpalMediaStream::SetPaused(bool pause, bool fromPatch)
 {
-  PTRACE_IF(3, paused != p, "Media\t" << (p ? "Paused" : "Resumed") << " stream " << *this);
-  paused = p;
+  PSafeLockReadWrite safeLock(*this);
+  if (!safeLock.IsLocked())
+    return false;
+
+  // If we are source, then update the sink side, and vice versa
+  if (!fromPatch && mediaPatch != NULL)
+    return mediaPatch->SetPaused(pause);
+
+  if (m_paused == pause)
+    return false;
+
+  PTRACE(3, "Media\t" << (pause ? "Paused" : "Resumed") << " stream " << *this);
+  m_paused = pause;
+
+  connection.OnPauseMediaStream(*this, pause);
+  return true;
 }
 
 
@@ -475,18 +492,18 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 }
 
 
-void OpalMediaStream::AddFilter(const PNotifier & Filter, const OpalMediaFormat & Stage)
+void OpalMediaStream::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadWrite safeLock(*this);
+  PSafeLockReadOnly safeLock(*this);
   if (safeLock.IsLocked() && mediaPatch != NULL)
-    mediaPatch->AddFilter(Filter, Stage);
+    mediaPatch->AddFilter(filter, stage);
 }
 
 
-PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage)
+PBoolean OpalMediaStream::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadWrite safeLock(*this);
-  return safeLock.IsLocked() && mediaPatch != NULL && mediaPatch->RemoveFilter(Filter, Stage);
+  PSafeLockReadOnly safeLock(*this);
+  return safeLock.IsLocked() && mediaPatch != NULL && mediaPatch->RemoveFilter(filter, stage);
 }
 
 
@@ -500,17 +517,12 @@ void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromP
 #endif
 
 
-void OpalMediaStream::RemovePatch(OpalMediaPatch * /*patch*/ ) 
-{ 
-  SetPatch(NULL); 
-}
-
-
 void OpalMediaStream::OnStartMediaPatch() 
 { 
   connection.OnStartMediaPatch(*mediaPatch);
 }
 
+
 void OpalMediaStream::OnStopMediaPatch(OpalMediaPatch & patch)
 { 
   connection.OnStopMediaPatch(patch);
@@ -635,6 +647,8 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
      whatever the RTP sender throws at us. For sink, we set it to the
      maximum size based on MTU (or other criteria). */
   defaultDataSize = isSource ? conn.GetEndPoint().GetManager().GetMaxRtpPacketSize() : conn.GetMaxRtpPayloadSize();
+
+  PTRACE(5, "Media\tCreated RTP media session, RTP=" << &rtp);
 }
 
 
@@ -670,24 +684,20 @@ PBoolean OpalRTPMediaStream::Close()
 }
 
 
-void OpalRTPMediaStream::SetPaused(bool pause)
+bool OpalRTPMediaStream::SetPaused(bool pause, bool fromPatch)
 {
-  if (paused == pause)
-    return;
-
-  OpalMediaStream::SetPaused(pause);
+  if (!OpalMediaStream::SetPaused(pause, fromPatch))
+    return false;
 
   // If coming out of pause, reopen the RTP session, even though it is probably
   // still open, to make sure any pending error/statistic conditions are reset.
-  if (!paused)
+  if (!pause)
     rtpSession.Reopen(IsSource());
 
-  if (IsSource()) {
-    if (pause)
-      rtpSession.SetJitterBufferSize(0, 0);
-    else
-      EnableJitterBuffer();
-  }
+  if (IsSource())
+    EnableJitterBuffer(!pause);
+
+  return true;
 }
 
 
@@ -698,12 +708,8 @@ PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
     return false;
   }
 
-  if (paused)
-    packet.SetPayloadSize(0);
-  else {
-    if (!rtpSession.ReadBufferedData(packet))
-      return false;
-  }
+  if (!rtpSession.ReadBufferedData(packet))
+    return false;
 
   timestamp = packet.GetTimestamp();
   return true;
@@ -719,10 +725,10 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 
   timestamp = packet.GetTimestamp();
 
-  if (paused || packet.GetPayloadSize() == 0)
+  if (packet.GetPayloadSize() == 0)
     return true;
 
-  packet.SetPayloadType(mediaFormat.GetPayloadType());
+  packet.SetPayloadType(m_payloadType);
   return rtpSession.WriteData(packet);
 }
 
@@ -746,10 +752,13 @@ PBoolean OpalRTPMediaStream::RequiresPatchThread() const
 }
 
 
-void OpalRTPMediaStream::EnableJitterBuffer() const
+bool OpalRTPMediaStream::EnableJitterBuffer(bool enab) const
 {
+  if (IsSink() || !RequiresPatchThread())
+    return false;
+
   unsigned minJitter, maxJitter;
-  if (mediaFormat.NeedsJitterBuffer()) {
+  if (enab && mediaFormat.NeedsJitterBuffer()) {
     minJitter = minAudioJitterDelay*mediaFormat.GetTimeUnits();
     maxJitter = maxAudioJitterDelay*mediaFormat.GetTimeUnits();
   }
@@ -759,12 +768,13 @@ void OpalRTPMediaStream::EnableJitterBuffer() const
   rtpSession.SetJitterBufferSize(minJitter, maxJitter,
                                  mediaFormat.GetTimeUnits(),
                                  connection.GetEndPoint().GetManager().GetMaxRtpPacketSize());
+  return true;
 }
 
 
 PBoolean OpalRTPMediaStream::SetPatch(OpalMediaPatch * patch)
 {
-  if (!isOpen || IsSink())
+  if (!isOpen || IsSink() || mediaPatch == NULL)
     return OpalMediaStream::SetPatch(patch);
 
   rtpSession.Close(true);
@@ -829,11 +839,19 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
   if (buffer == NULL || size == 0)
     return m_channel->Read(buffer, size);
 
+  unsigned consecutiveZeroReads = 0;
   while (size > 0) {
     if (!m_channel->Read(buffer, size))
       return false;
 
     PINDEX lastReadCount = m_channel->GetLastReadCount();
+    if (lastReadCount != 0)
+      consecutiveZeroReads = 0;
+    else if (++consecutiveZeroReads > 10) {
+      PTRACE(1, "Media\tRaw channel returned success with zero data multiple consecutive times, aborting.");
+      return false;
+    }
+
     CollectAverage(buffer, lastReadCount);
 
     buffer += lastReadCount;
@@ -1030,6 +1048,8 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
   : OpalRawMediaStream(conn, mediaFormat, sessionID, isSource, channel, autoDel)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
+  , m_failSafeRate(mediaFormat.GetTimeUnits())
+  , m_failSafeDelay(50, 10)
 {
 }
 
@@ -1050,36 +1070,58 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                        true)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
+  , m_failSafeRate(mediaFormat.GetTimeUnits())
+  , m_failSafeDelay(50, 10)
 {
 }
 
 
 PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
-  /* For efficiency reasons we will not accept a packet size that is too small.
-     We move it up to the next even multiple, which has a danger of the remote not
-     sending an even number of our multiplier. */
-  const unsigned BufferTimeMilliseconds = 10;
-  PINDEX bufferSize = BufferTimeMilliseconds*mediaFormat.GetClockRate()/1000*sizeof(short);
+  PINDEX frameSize = frameTime*sizeof(short);
+  unsigned clockRate = mediaFormat.GetClockRate();
+  unsigned frameMilliseconds = (frameTime*1000+clockRate-1)/clockRate;
 
-  // Calculate sound buffers from global system settings
-  PINDEX soundChannelBuffers = (m_soundChannelBufferTime+BufferTimeMilliseconds-1)/BufferTimeMilliseconds;
+  /* For efficiency reasons we will not accept a packet size that is too small.
+     We move it up to the next even multiple of the minimum, which has a danger
+     of the remote not sending an even number of our multiplier, but 10ms seems
+     universally done by everyone out there. */
+  const unsigned MinBufferTimeMilliseconds = 10;
+  if (frameMilliseconds < MinBufferTimeMilliseconds) {
+    PINDEX minFrameCount = (MinBufferTimeMilliseconds+frameMilliseconds-1)/frameMilliseconds;
+    frameSize = minFrameCount*frameTime*sizeof(short);
+    frameMilliseconds = (minFrameCount*frameTime*1000+clockRate-1)/clockRate;
+  }
+
+  // Quantise dataSize up to multiple of the frame time
+  PINDEX frameCount = (dataSize+frameSize-1)/frameSize;
+  dataSize = frameCount*frameSize;
+
+  // Calculate number of sound buffers from global system settings
+  PINDEX soundChannelBuffers = (m_soundChannelBufferTime+frameMilliseconds-1)/frameMilliseconds;
   if (soundChannelBuffers < m_soundChannelBuffers)
     soundChannelBuffers = m_soundChannelBuffers;
 
-  // Quantise dataSize up to multiple of the frame time
-  PINDEX frameSize = frameTime*sizeof(short);
-  dataSize = (dataSize+frameSize-1)/frameSize * frameSize;
-
-  // Calculte buffers needed for the data we require and increase buffers if required
-  PINDEX dataBuffersNeeded = (dataSize+bufferSize-1)/bufferSize;
-  if (soundChannelBuffers < dataBuffersNeeded)
-    soundChannelBuffers = dataBuffersNeeded;
+  // Increase sound buffers if negotiated maximum larger than global settings
+  if (soundChannelBuffers < frameCount)
+    soundChannelBuffers = frameCount;
 
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
-         << dataSize << ", buffer size set to " << bufferSize << " and " << soundChannelBuffers << " buffers.");
+         << dataSize << ", buffer size set to " << frameSize << " and " << soundChannelBuffers << " buffers.");
   return OpalMediaStream::SetDataSize(dataSize, frameTime) &&
-         ((PSoundChannel *)m_channel)->SetBuffers(bufferSize, soundChannelBuffers);
+         ((PSoundChannel *)m_channel)->SetBuffers(frameSize, soundChannelBuffers);
+}
+
+
+PBoolean OpalAudioMediaStream::WriteData(const BYTE * data, PINDEX length, PINDEX & written)
+{
+  if (!OpalRawMediaStream::WriteData(data, length, written))
+    return false;
+
+  /* Note if the audio device is working correctly, then this actually will have
+     no effect, it will always be delaying 0ms */
+  m_failSafeDelay.Delay(length/sizeof(short)/m_failSafeRate);
+  return true;
 }
 
 
@@ -1100,11 +1142,13 @@ OpalVideoMediaStream::OpalVideoMediaStream(OpalConnection & conn,
                                            unsigned sessionID,
                                            PVideoInputDevice * in,
                                            PVideoOutputDevice * out,
-                                           PBoolean del)
-  : OpalMediaStream(conn, mediaFormat, sessionID, in != NULL),
-    inputDevice(in),
-    outputDevice(out),
-    autoDelete(del)
+                                           PBoolean delIn,
+                                           PBoolean delOut)
+  : OpalMediaStream(conn, mediaFormat, sessionID, in != NULL)
+  , m_inputDevice(in)
+  , m_outputDevice(out)
+  , m_autoDeleteInput(delIn)
+  , m_autoDeleteOutput(delOut)
 {
   PAssert(in != NULL || out != NULL, PInvalidParameter);
 }
@@ -1113,22 +1157,24 @@ OpalVideoMediaStream::OpalVideoMediaStream(OpalConnection & conn,
 OpalVideoMediaStream::~OpalVideoMediaStream()
 {
   Close();
-  if (autoDelete) {
-    delete inputDevice;
-    delete outputDevice;
-  }
+
+  if (m_autoDeleteInput)
+    delete m_inputDevice;
+
+  if (m_autoDeleteOutput)
+    delete m_outputDevice;
 }
 
 
 PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
-  if (inputDevice != NULL) {
-    PINDEX minDataSize = inputDevice->GetMaxFrameBytes();
+  if (m_inputDevice != NULL) {
+    PINDEX minDataSize = m_inputDevice->GetMaxFrameBytes();
     if (dataSize < minDataSize)
       dataSize = minDataSize;
   }
-  if (outputDevice != NULL) {
-    PINDEX minDataSize = outputDevice->GetMaxFrameBytes();
+  if (m_outputDevice != NULL) {
+    PINDEX minDataSize = m_outputDevice->GetMaxFrameBytes();
     if (dataSize < minDataSize)
       dataSize = minDataSize;
   }
@@ -1145,28 +1191,32 @@ PBoolean OpalVideoMediaStream::Open()
   unsigned width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
   unsigned height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
 
-  if (inputDevice != NULL) {
-    if (!inputDevice->SetColourFormatConverter(mediaFormat)) {
+  if (m_inputDevice != NULL) {
+    if (!m_inputDevice->SetColourFormatConverter(mediaFormat)) {
       PTRACE(1, "Media\tCould not set colour format in grabber to " << mediaFormat);
       return false;
     }
-    if (!inputDevice->SetFrameSizeConverter(width, height)) {
+    if (!m_inputDevice->SetFrameSizeConverter(width, height)) {
       PTRACE(1, "Media\tCould not set frame size in grabber to " << width << 'x' << height << " in " << mediaFormat);
       return false;
     }
-    if (!inputDevice->Start()) {
+    if (!m_inputDevice->SetFrameRate(mediaFormat.GetClockRate()/mediaFormat.GetFrameTime())) {
+      PTRACE(1, "Media\tCould not set frame rate in grabber to " << (mediaFormat.GetClockRate()/mediaFormat.GetFrameTime()));
+      return false;
+    }
+    if (!m_inputDevice->Start()) {
       PTRACE(1, "Media\tCould not start video grabber");
       return false;
     }
-    lastGrabTime = PTimer::Tick();
+    m_lastGrabTime = PTimer::Tick();
   }
 
-  if (outputDevice != NULL) {
-    if (!outputDevice->SetColourFormatConverter(mediaFormat)) {
+  if (m_outputDevice != NULL) {
+    if (!m_outputDevice->SetColourFormatConverter(mediaFormat)) {
       PTRACE(1, "Media\tCould not set colour format in video display to " << mediaFormat);
       return false;
     }
-    if (!outputDevice->SetFrameSizeConverter(width, height)) {
+    if (!m_outputDevice->SetFrameSizeConverter(width, height)) {
       PTRACE(1, "Media\tCould not set frame size in video display to " << width << 'x' << height << " in " << mediaFormat);
       return false;
     }
@@ -1183,11 +1233,11 @@ PBoolean OpalVideoMediaStream::Close()
   if (!OpalMediaStream::Close())
     return false;
 
-  if (inputDevice != NULL)
-    inputDevice->Close();
+  if (m_inputDevice != NULL)
+    m_inputDevice->Stop();
 
-  if (outputDevice != NULL)
-    outputDevice->Close();
+  if (m_outputDevice != NULL)
+    m_outputDevice->Stop();
 
   return true;
 }
@@ -1203,18 +1253,18 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
     return false;
   }
 
-  if (inputDevice == NULL) {
+  if (m_inputDevice == NULL) {
     PTRACE(1, "Media\tTried to read from video display device");
     return false;
   }
 
-  if (size < inputDevice->GetMaxFrameBytes()) {
-    PTRACE(1, "Media\tTried to read with insufficient buffer size - " << size << " < " << inputDevice->GetMaxFrameBytes());
+  if (size < m_inputDevice->GetMaxFrameBytes()) {
+    PTRACE(1, "Media\tTried to read with insufficient buffer size - " << size << " < " << m_inputDevice->GetMaxFrameBytes());
     return false;
   }
 
   unsigned width, height;
-  inputDevice->GetFrameSize(width, height);
+  m_inputDevice->GetFrameSize(width, height);
   OpalVideoTranscoder::FrameHeader * frame = (OpalVideoTranscoder::FrameHeader *)PAssertNULL(data);
   frame->x = frame->y = 0;
   frame->width = width;
@@ -1222,12 +1272,12 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
 
   PINDEX bytesReturned = size - sizeof(OpalVideoTranscoder::FrameHeader);
   unsigned flags = 0;
-  if (!inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags))
+  if (!m_inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags))
     return false;
 
   PTimeInterval currentGrabTime = PTimer::Tick();
-  timestamp += (int)((currentGrabTime - lastGrabTime).GetMilliSeconds()*OpalMediaFormat::VideoClockRate/1000);
-  lastGrabTime = currentGrabTime;
+  timestamp += (int)((currentGrabTime - m_lastGrabTime).GetMilliSeconds()*OpalMediaFormat::VideoClockRate/1000);
+  m_lastGrabTime = currentGrabTime;
 
   if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0)
     ExecuteCommand(OpalVideoUpdatePicture());
@@ -1237,15 +1287,16 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
   if (length > 0)
     length += sizeof(PluginCodec_Video_FrameHeader);
 
-  if (outputDevice == NULL)
+  if (m_outputDevice == NULL)
     return true;
 
-  if (outputDevice->Start())
-    return outputDevice->SetFrameData(0, 0, width, height, OPAL_VIDEO_FRAME_DATA_PTR(frame), true, flags);
+  if (m_outputDevice->Start())
+    return m_outputDevice->SetFrameData(0, 0, width, height, OPAL_VIDEO_FRAME_DATA_PTR(frame), true, flags);
 
   PTRACE(1, "Media\tCould not start video display device");
-  delete outputDevice;
-  outputDevice = NULL;
+  if (m_autoDeleteOutput)
+    delete m_outputDevice;
+  m_outputDevice = NULL;
   return true;
 }
 
@@ -1260,7 +1311,7 @@ PBoolean OpalVideoMediaStream::WriteData(const BYTE * data, PINDEX length, PINDE
     return false;
   }
 
-  if (outputDevice == NULL) {
+  if (m_outputDevice == NULL) {
     PTRACE(1, "Media\tTried to write to video capture device");
     return false;
   }
@@ -1274,19 +1325,19 @@ PBoolean OpalVideoMediaStream::WriteData(const BYTE * data, PINDEX length, PINDE
 
   const OpalVideoTranscoder::FrameHeader * frame = (const OpalVideoTranscoder::FrameHeader *)data;
 
-  if (!outputDevice->SetFrameSize(frame->width, frame->height)) {
+  if (!m_outputDevice->SetFrameSize(frame->width, frame->height)) {
     PTRACE(1, "Media\tCould not resize video display device to " << frame->width << 'x' << frame->height);
     return false;
   }
 
-  if (!outputDevice->Start()) {
+  if (!m_outputDevice->Start()) {
     PTRACE(1, "Media\tCould not start video display device");
     return false;
   }
 
-  return outputDevice->SetFrameData(frame->x, frame->y,
-                                    frame->width, frame->height,
-                                    OPAL_VIDEO_FRAME_DATA_PTR(frame), marker);
+  return m_outputDevice->SetFrameData(frame->x, frame->y,
+                                      frame->width, frame->height,
+                                      OPAL_VIDEO_FRAME_DATA_PTR(frame), marker);
 }
 
 
@@ -1316,10 +1367,10 @@ OpalUDPMediaStream::~OpalUDPMediaStream()
 }
 
 
-PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & Packet)
+PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
-  Packet.SetPayloadType(mediaFormat.GetPayloadType());
-  Packet.SetPayloadSize(0);
+  packet.SetPayloadType(m_payloadType);
+  packet.SetPayloadSize(0);
 
   if (IsSink()) {
     PTRACE(1, "Media\tTried to read from sink media stream");
@@ -1334,8 +1385,8 @@ PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & Packet)
   }
 
   if (rawData.GetSize() > 0) {
-    Packet.SetPayloadSize(rawData.GetSize());
-    memcpy(Packet.GetPayloadPtr(), rawData.GetPointer(), rawData.GetSize());
+    packet.SetPayloadSize(rawData.GetSize());
+    memcpy(packet.GetPayloadPtr(), rawData.GetPointer(), rawData.GetSize());
   }
 
   return true;
