@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25924 $
+ * $Revision: 26246 $
  * $Author: rjongbloed $
- * $Date: 2011-05-30 18:15:54 -0500 (Mon, 30 May 2011) $
+ * $Date: 2011-07-25 22:42:23 -0500 (Mon, 25 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -1885,6 +1885,20 @@ void OpalManager_C::HandleMediaStream(const OpalMessage & command, OpalMessageBu
       stream->SetPaused(false);
       break;
   }
+
+  if (m_apiVersion < 25)
+    return;
+
+  if (command.m_param.m_mediaStream.m_volume != 0) {
+    unsigned volume;
+    if (command.m_param.m_mediaStream.m_volume < 0)
+      volume = 0;
+    else if (command.m_param.m_mediaStream.m_volume > 100)
+      volume = 100;
+    else
+      volume = command.m_param.m_mediaStream.m_volume;
+    connection->SetAudioVolume(stream->IsSource(), volume);
+  }
 }
 
 
