@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24190 $
+ * $Revision: 26128 $
  * $Author: rjongbloed $
- * $Date: 2010-04-06 04:23:26 -0500 (Tue, 06 Apr 2010) $
+ * $Date: 2011-07-07 21:36:57 -0500 (Thu, 07 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -91,6 +91,13 @@ OpalRTPConnection::~OpalRTPConnection()
 }
 
 
+void OpalRTPConnection::OnReleased()
+{
+  OpalConnection::OnReleased();
+  CloseSession(0);
+}
+
+
 unsigned OpalRTPConnection::GetNextSessionID(const OpalMediaType & mediaType, bool isSource)
 {
   unsigned nextSessionId = m_rtpSessions.GetNextSessionID();
@@ -117,11 +124,13 @@ RTP_Session * OpalRTPConnection::GetSession(unsigned sessionID) const
   return m_rtpSessions.GetSession(sessionID);
 }
 
+
 OpalMediaSession * OpalRTPConnection::GetMediaSession(unsigned sessionID) const
 {
   return m_rtpSessions.GetMediaSession(sessionID);
 }
 
+
 RTP_Session * OpalRTPConnection::UseSession(const OpalTransport & transport, unsigned sessionID, const OpalMediaType & mediaType, RTP_QOS * rtpqos)
 {
   RTP_Session * rtpSession = m_rtpSessions.GetSession(sessionID);
@@ -159,7 +168,7 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
 
   WORD firstPort = manager.GetRtpIpPortPair();
   WORD nextPort = firstPort;
-  while (!rtpSession->Open(localAddress, nextPort, nextPort, manager.GetRtpIpTypeofService(), natMethod, rtpqos)) {
+  while (!rtpSession->Open(localAddress, nextPort, nextPort, manager.GetMediaTypeOfService(mediaType), natMethod, rtpqos)) {
     nextPort = manager.GetRtpIpPortPair();
     if (nextPort == firstPort) {
       PTRACE(1, "RTPCon\tNo ports available for RTP session " << sessionID << ","
@@ -234,7 +243,7 @@ bool OpalRTPConnection::ChangeSessionID(unsigned fromSessionID, unsigned toSessi
 }
 
 
-void OpalRTPConnection::ReleaseSession(unsigned sessionID, PBoolean clearAll/* = PFalse */)
+void OpalRTPConnection::CloseSession(unsigned sessionID)
 {
 #ifdef HAS_LIBZRTP
   //check is security mode ZRTP
@@ -252,7 +261,7 @@ void OpalRTPConnection::ReleaseSession(unsigned sessionID, PBoolean clearAll/* =
   printf("release session %i\n", sessionID);
 #endif
 
-  m_rtpSessions.ReleaseSession(sessionID, clearAll);
+  m_rtpSessions.CloseSession(sessionID);
 }
 
 
@@ -264,21 +273,39 @@ PBoolean OpalRTPConnection::IsRTPNATEnabled(const PIPSocket::Address & localAddr
   return static_cast<OpalRTPEndPoint &>(endpoint).IsRTPNATEnabled(*this, localAddr, peerAddr, sigAddr, incoming);
 }
 
+
 #if OPAL_VIDEO
-void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & command, INT /*extra*/)
+void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & command, INT sessionID)
 {
+  RTP_Session * session = m_rtpSessions.GetSession(sessionID);
+  if (session == NULL)
+    return;
+
+  PCaselessString rtcp_fb;
+  OpalMediaStreamPtr videoStream = GetMediaStream(sessionID, true);
+  if (videoStream != NULL)
+    rtcp_fb = videoStream->GetMediaFormat().GetOptionString("RTCP-FB");
+
   if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    OpalMediaStreamPtr videoStream = GetMediaStream(OpalMediaType::Video(), false);
-    if (videoStream != NULL) {
-      RTP_Session * session = m_rtpSessions.GetSession(videoStream->GetSessionID());
-      if (session != NULL) {
-        session->SendIntraFrameRequest();
+    bool no_AVPF_PLI = rtcp_fb.Find("pli") == P_MAX_INDEX;
+    bool no_AVPF_FIR = rtcp_fb.Find("fir") == P_MAX_INDEX;
+
+    if (no_AVPF_PLI && no_AVPF_FIR)
+      session->SendIntraFrameRequest(true, false);  // Fall back to RFC2032
+    else if (no_AVPF_PLI)
+      session->SendIntraFrameRequest(false, false); // Unusual, but possible, use RFC5104 FIR
+    else if (no_AVPF_FIR)
+      session->SendIntraFrameRequest(false, true);  // More common, use RFC4585 PLI
+    else
+      session->SendIntraFrameRequest(false, PIsDescendant(&command, OpalVideoPictureLoss));
+
 #if OPAL_STATISTICS
-        m_VideoUpdateRequestsSent++;
+    m_VideoUpdateRequestsSent++;
 #endif
-      }
-    }
   }
+  else if (PIsDescendant(&command, OpalTemporalSpatialTradeOff) &&
+           rtcp_fb.Find("tstr") != P_MAX_INDEX)
+    session->SendTemporalSpatialTradeOff(dynamic_cast<OpalTemporalSpatialTradeOff &>(command).GetTradeOff());
 }
 #else
 void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & /*command*/, INT /*extra*/)
@@ -309,14 +336,17 @@ void OpalRTPConnection::AttachRFC2833HandlerToPatch(PBoolean isSource, OpalMedia
 
 PBoolean OpalRTPConnection::SendUserInputTone(char tone, unsigned duration)
 {
-  if (
+  if (GetRealSendUserInputMode() == SendUserInputAsRFC2833) {
+    if (
 #if OPAL_T38_CAPABILITY
-      ciscoNSEHandler->SendToneAsync(tone, duration) ||
+        ciscoNSEHandler->SendToneAsync(tone, duration) ||
 #endif
-       rfc2833Handler->SendToneAsync(tone, duration))
-    return true;
+         rfc2833Handler->SendToneAsync(tone, duration))
+      return true;
+
+    PTRACE(2, "RTPCon\tCould not send tone '" << tone << "' via RFC2833.");
+  }
 
-  PTRACE(2, "RTPCon\tCould not send tone '" << tone << "' via RFC2833.");
   return OpalConnection::SendUserInputTone(tone, duration);
 }
 
@@ -340,7 +370,7 @@ PBoolean OpalRTPConnection::GetMediaInformation(unsigned sessionID,
   else
     info.data = info.control = address;
 
-  info.rfc2833 = rfc2833Handler->GetPayloadType();
+  info.rfc2833 = rfc2833Handler->GetRxMediaFormat().GetPayloadType();
   PTRACE(3, "RTPCon\tGetMediaInformation for session " << sessionID
          << " data=" << info.data << " rfc2833=" << info.rfc2833);
   return PTrue;
@@ -378,22 +408,21 @@ OpalMediaStream * OpalRTPConnection::CreateMediaStream(const OpalMediaFormat & m
 }
 
 
-void OpalRTPConnection::AdjustMediaFormats(bool local,
-                          OpalMediaFormatList & mediaFormats,
-                               OpalConnection * otherConnection) const
+void OpalRTPConnection::AdjustMediaFormats(bool   local,
+                           const OpalConnection * otherConnection,
+                            OpalMediaFormatList & mediaFormats) const
 {
-  OpalConnection::AdjustMediaFormats(local, mediaFormats, otherConnection);
-
-  if (otherConnection != NULL && otherConnection != this)
-    return;
-
-  OpalMediaFormatList::iterator fmt = mediaFormats.begin();
-  while (fmt != mediaFormats.end()) {
-    if (fmt->IsTransportable())
-      ++fmt;
-    else
-      mediaFormats -= *fmt++;
+  if (otherConnection == NULL && local) {
+    OpalMediaFormatList::iterator fmt = mediaFormats.begin();
+    while (fmt != mediaFormats.end()) {
+      if (fmt->IsTransportable())
+        ++fmt;
+      else
+        mediaFormats -= *fmt++;
+    }
   }
+
+  OpalConnection::AdjustMediaFormats(local, otherConnection, mediaFormats);
 }
 
 
@@ -428,7 +457,7 @@ void OpalRTPConnection::SessionFailing(RTP_Session & session)
   // if so, clear the call
   if (m_rtpSessions.AllSessionsFailing()) {
     PTRACE(2, "RTPCon\tClearing call as all RTP session are failing");
-    ClearCall();
+    Release();
   }
 }
 
@@ -453,10 +482,9 @@ OpalMediaSession::OpalMediaSession(const OpalMediaSession & _obj)
 
 OpalRTPMediaSession::OpalRTPMediaSession(OpalConnection & conn,
                                          const OpalMediaType & mediaType,
-                                         unsigned sessionId,
-                                         RTP_Session * rtp)
+                                         unsigned sessionId)
   : OpalMediaSession(conn, mediaType, sessionId)
-  , rtpSession(rtp)
+  , rtpSession(NULL)
 {
 }
 
@@ -467,14 +495,35 @@ OpalRTPMediaSession::OpalRTPMediaSession(const OpalRTPMediaSession & obj)
 {
 }
 
+
+OpalRTPMediaSession::~OpalRTPMediaSession()
+{
+  if (rtpSession != NULL) {
+    PTRACE(4, "RTP\tDeleting session " << rtpSession->GetSessionID());
+    ((OpalRTPEndPoint &)connection.GetEndPoint()).SetConnectionByRtpLocalPort(rtpSession, NULL);
+    delete rtpSession;
+  }
+}
+
+
+void OpalRTPMediaSession::Attach(RTP_Session * rtp)
+{
+  if (PAssert(rtpSession == NULL, "Cannot attach with already existing session")) {
+    rtpSession = rtp;
+    ((OpalRTPEndPoint &)connection.GetEndPoint()).SetConnectionByRtpLocalPort(rtpSession, &connection);
+  }
+}
+
+
 void OpalRTPMediaSession::Close()
 {
   if (rtpSession != NULL) {
-    PTRACE(3, "RTP\tDeleting session " << rtpSession->GetSessionID());
+    PTRACE(3, "RTP\tClosing session " << rtpSession->GetSessionID());
+    ((OpalRTPEndPoint &)connection.GetEndPoint()).SetConnectionByRtpLocalPort(rtpSession, NULL);
+    if (rtpSession->GetPacketsReceived() > 0 || rtpSession->GetPacketsSent() > 0)
+      rtpSession->SendBYE();
     rtpSession->Close(PTrue);
     rtpSession->SetJitterBufferSize(0, 0);
-    delete rtpSession;
-    rtpSession = NULL;
   }
 }
 
@@ -507,23 +556,22 @@ OpalMediaStream * OpalRTPMediaSession::CreateMediaStream(const OpalMediaFormat &
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalRTPSessionManager::OpalRTPSessionManager(OpalConnection & conn)
-  : connection(conn)
+OpalRTPSessionManager::OpalRTPSessionManager(OpalRTPConnection & connection)
+  : m_connection(connection)
+{
+}
+
+
+OpalRTPSessionManager::OpalRTPSessionManager(const OpalRTPSessionManager & other)
+  : PObject(other)
+  , m_connection(other.m_connection)
+  , sessions(other.sessions)
 {
 }
 
 
 OpalRTPSessionManager::~OpalRTPSessionManager()
 {
-  PWaitAndSignal m(m_mutex);
-  if (sessions.IsUnique()) {
-    while (sessions.GetSize() > 0) {
-      PINDEX id = sessions.GetKeyAt(0);
-      PTRACE(3, "RTP\tClosing session " << id);
-      sessions[id].Close();
-      sessions.RemoveAt(id);
-    }
-  }
 }
 
 
@@ -551,30 +599,39 @@ void OpalRTPSessionManager::AddSession(RTP_Session * rtpSession, const OpalMedia
   unsigned sessionID = rtpSession->GetSessionID();
   OpalMediaSession * session = sessions.GetAt(sessionID);
   if (session == NULL) {
-    sessions.Insert(POrdinalKey(sessionID), new OpalRTPMediaSession(connection, mediaType, sessionID, rtpSession));
+    session = new OpalRTPMediaSession(m_connection, mediaType, sessionID);
+    sessions.Insert(POrdinalKey(sessionID), session);
     PTRACE(3, "RTP\tCreating new session " << *rtpSession);
   }
-  else {
-    OpalRTPMediaSession * s = dynamic_cast<OpalRTPMediaSession *>(session);
-    if (PAssert(s != NULL,             "RTP session type does not match") &&
-        PAssert(s->rtpSession == NULL, "Cannot add already existing session"))
-      s->rtpSession = rtpSession;
-  }
+
+  OpalRTPMediaSession * s = dynamic_cast<OpalRTPMediaSession *>(session);
+  if (PAssert(s != NULL, "RTP session type does not match"))
+    s->Attach(rtpSession);
 }
 
 
 void OpalRTPSessionManager::AddMediaSession(OpalMediaSession * mediaSession, const OpalMediaType & /*mediaType*/)
 {
   PWaitAndSignal m(m_mutex);
-  
+
   PAssert(sessions.GetAt(mediaSession->sessionId) == NULL, "Cannot add already existing session");
   sessions.Insert(POrdinalKey(mediaSession->sessionId), mediaSession);
 }
 
 
-void OpalRTPSessionManager::ReleaseSession(unsigned PTRACE_PARAM(sessionID), PBoolean /*clearAll*/)
+void OpalRTPSessionManager::CloseSession(unsigned sessionID)
 {
-  PTRACE(3, "RTP\tReleasing session " << sessionID);
+  PWaitAndSignal mutex(m_mutex);
+  if (sessionID == 0) {
+    for (PINDEX i = 0; i < sessions.GetSize(); ++i) {
+      PTRACE(3, "RTP\tClosing session " << sessions.GetKeyAt(i));
+      sessions.GetDataAt(i).Close();
+    }
+  }
+  else {
+    PTRACE(3, "RTP\tClosing session " << sessionID);
+    sessions[sessionID].Close();
+  }
 }
 
 
@@ -607,7 +664,7 @@ RTP_Session * OpalRTPSessionManager::GetSession(unsigned sessionID) const
   }
 
   PTRACE(3, "RTP\tFound existing RTP session " << sessionID);
-  return ((OpalRTPMediaSession *)session)->rtpSession;
+  return ((OpalRTPMediaSession *)session)->GetSession();
 }
 
 
@@ -629,7 +686,7 @@ bool OpalRTPSessionManager::ChangeSessionID(unsigned fromSessionID, unsigned toS
 
   OpalRTPMediaSession * rtpSession = dynamic_cast<OpalRTPMediaSession *>(session);
   if (rtpSession != NULL)
-    rtpSession->rtpSession->SetSessionID(toSessionID);
+    rtpSession->GetSession()->SetSessionID(toSessionID);
 
   session->sessionId = toSessionID;
   return sessions.SetAt(POrdinalKey(toSessionID), session);
