@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25780 $
+ * $Revision: 26320 $
  * $Author: rjongbloed $
- * $Date: 2011-05-16 21:05:05 -0500 (Mon, 16 May 2011) $
+ * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -193,6 +193,15 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
   }
 
   transcoder->SetInstanceID(instance, instanceLen); // Make sure this is done first
+
+  /* Set the actual media formats used to create the transcoder.
+     This information got lost as OpalTranscoderKey is just two PStrings
+     and not OpalMediaFormats. Need to copy all of the actual options
+     over the top of the global ones just created. */
+  transcoder->inputMediaFormat = srcFormat;
+  transcoder->outputMediaFormat = destFormat;
+
+  // This is called to make sure options are sent to plug ins.
   if (transcoder->UpdateMediaFormats(srcFormat, destFormat))
     return transcoder;
 
