@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21710 $
- * $Author: csoutheren $
- * $Date: 2008-12-08 07:17:29 +0000 (Mon, 08 Dec 2008) $
+ * $Revision: 25210 $
+ * $Author: rjongbloed $
+ * $Date: 2011-02-28 08:24:23 -0600 (Mon, 28 Feb 2011) $
  */
 
 #include <ptlib.h>
@@ -55,7 +55,7 @@ const OpalMediaFormat & GetOpalT38()
       T38MediaFormat()
         : OpalMediaFormat(OPAL_T38,
                           "fax",
-                          RTP_DataFrame::DynamicBase,
+                          RTP_DataFrame::T38,
                           "t38",
                           false, // No jitter for data
                           1440, // 100's bits/sec
@@ -76,11 +76,16 @@ const OpalMediaFormat & GetOpalT38()
 /////////////////////////////////////////////////////////////////////////////
 
 OpalFaxMediaType::OpalFaxMediaType()
-  : OpalMediaTypeDefinition("fax", "image", 3)
-{ }
+  : OpalMediaTypeDefinition("fax", "image", 3) // Must be 3 for H.323 operation
+{
+}
+
 
 PString OpalFaxMediaType::GetRTPEncoding() const
-{ return "udptl"; }
+{
+  return "udptl";
+}
+
 
 RTP_UDP * OpalFaxMediaType::CreateRTPSession(OpalRTPConnection &, unsigned sessionID, bool remoteIsNAT)
 {
@@ -91,6 +96,7 @@ RTP_UDP * OpalFaxMediaType::CreateRTPSession(OpalRTPConnection &, unsigned sessi
   return new RTP_UDP(params);
 }
 
+
 OpalMediaSession * OpalFaxMediaType::CreateMediaSession(OpalConnection & conn, unsigned sessionID) const
 {
   return new OpalRTPMediaSession(conn, m_mediaType, sessionID);
